@@ -32,6 +32,7 @@
 #include "overlay.hex"
 #include <d3d10.h>
 #include <d3dx10.h>
+#include <time.h>
 
 DXGIData *dxgi = NULL;
 
@@ -90,6 +91,9 @@ class D10State: protected Pipe {
 		ID3D10Texture2D *pTexture;
 		ID3D10ShaderResourceView *pSRView;
 
+		clock_t timeT;
+		unsigned int frameCount;
+
 		D10State(IDXGISwapChain *, ID3D10Device *);
 		~D10State();
 		void init();
@@ -125,6 +129,9 @@ D10State::D10State(IDXGISwapChain *pSwapChain, ID3D10Device *pDevice) {
 	pTexture = NULL;
 	pSRView = NULL;
 
+	timeT = clock();
+	frameCount = 0;
+
 	pDevice->AddRef();
 	initRefCount = pDevice->Release();
 }
@@ -145,7 +152,7 @@ void D10State::blit(unsigned int x, unsigned int y, unsigned int w, unsigned int
 
 	UCHAR* pTexels = (UCHAR*)mappedTex.pData;
 
-	for (int r=0;r< uiHeight; ++r) {
+	for (unsigned int r=0;r< uiHeight; ++r) {
 		unsigned char *sptr = a_ucTexture + r * uiWidth * 4;
 		unsigned char *dptr = reinterpret_cast<unsigned char *>(pTexels) + r * mappedTex.RowPitch;
 		memcpy(dptr, sptr, uiWidth * 4);
@@ -286,10 +293,10 @@ void D10State::init() {
 	D3D10_BLEND_DESC blend;
 	ZeroMemory(&blend, sizeof(blend));
 	blend.BlendEnable[0] = TRUE;
-	blend.SrcBlend = D3D10_BLEND_SRC_ALPHA;
+	blend.SrcBlend = D3D10_BLEND_ONE;
 	blend.DestBlend = D3D10_BLEND_INV_SRC_ALPHA;
 	blend.BlendOp = D3D10_BLEND_OP_ADD;
-	blend.SrcBlendAlpha = D3D10_BLEND_SRC_ALPHA;
+	blend.SrcBlendAlpha = D3D10_BLEND_ONE;
 	blend.DestBlendAlpha = D3D10_BLEND_INV_SRC_ALPHA;
 	blend.BlendOpAlpha = D3D10_BLEND_OP_ADD;
 	blend.RenderTargetWriteMask[0] = D3D10_COLOR_WRITE_ENABLE_ALL;
@@ -377,12 +384,28 @@ D10State::~D10State() {
 }
 
 void D10State::draw() {
-	HRESULT hr;
+	clock_t t = clock();
+	float elapsed = static_cast<float>(t - timeT) / CLOCKS_PER_SEC;
+	++frameCount;
+	if (elapsed > OVERLAY_FPS_INTERVAL) {
+		OverlayMsg om;
+		om.omh.uiMagic = OVERLAY_MAGIC_NUMBER;
+		om.omh.uiType = OVERLAY_MSGTYPE_FPS;
+		om.omh.iLength = sizeof(OverlayMsgFps);
+		om.omf.fps = frameCount / elapsed;
+
+		sendMessage(om);
+
+		frameCount = 0;
+		timeT = t;
+	}
+
 	dwMyThread = GetCurrentThreadId();
 
 	checkMessage(vp.Width, vp.Height);
 
 	if (a_ucTexture && pSRView && (uiLeft != uiRight)) {
+		HRESULT hr;
 		pOrigStateBlock->Capture();
 		pMyStateBlock->Apply();
 
@@ -583,12 +606,12 @@ extern "C" __declspec(dllexport) void __cdecl PrepareDXGI() {
 	if ((ovi.dwMajorVersion >= 7) || ((ovi.dwMajorVersion == 6) && (ovi.dwBuildNumber >= 6001))) {
 		HMODULE hD3D10 = LoadLibrary("D3D10.DLL");
 		HMODULE hDXGI = LoadLibrary("DXGI.DLL");
-		HRESULT hr;
 
 		if (hDXGI != NULL && hD3D10 != NULL) {
 			CreateDXGIFactoryType pCreateDXGIFactory = reinterpret_cast<CreateDXGIFactoryType>(GetProcAddress(hDXGI, "CreateDXGIFactory"));
 			ods("Got %p", pCreateDXGIFactory);
 			if (pCreateDXGIFactory) {
+				HRESULT hr;
 				IDXGIFactory * pFactory;
 				hr = pCreateDXGIFactory(__uuidof(IDXGIFactory), (void**)(&pFactory));
 				if (pFactory) {
@@ -659,7 +682,7 @@ extern "C" __declspec(dllexport) void __cdecl PrepareDXGI() {
 								unsigned char *b = (unsigned char *) pResize;
 								unsigned char *a = (unsigned char *) hRef;
 								dxgi->iOffsetResize = b-a;
-								ods("DXGI: Successfully found ResizeBuffers offset: %ls: %d", dxgi->wcDXGIFileName, dxgi->iOffsetPresent);
+								ods("DXGI: Successfully found ResizeBuffers offset: %ls: %d", dxgi->wcDXGIFileName, dxgi->iOffsetResize);
 							}
 						}
 
