@@ -30,6 +30,7 @@
 
 #include "lib.h"
 #include <d3d9.h>
+#include <time.h>
 
 Direct3D9Data *d3dd = NULL;
 
@@ -42,7 +43,7 @@ struct D3DTLVERTEX {
 };
 static const DWORD D3DFVF_TLVERTEX = D3DFVF_XYZRHW | D3DFVF_TEX1;
 
-class DevState : protected Pipe {
+class DevState : public Pipe {
 	public:
 		IDirect3DDevice9 *dev;
 		IDirect3DStateBlock9 *pSB;
@@ -55,6 +56,9 @@ class DevState : protected Pipe {
 		D3DTLVERTEX vertices[4];
 		LPDIRECT3DTEXTURE9 texTexture;
 
+		clock_t timeT;
+		unsigned int frameCount;
+
 		DevState();
 
 		void createCleanState();
@@ -81,6 +85,9 @@ DevState::DevState() {
 	myRefCount = 0;
 	texTexture = NULL;
 
+	timeT = clock();
+	frameCount = 0;
+
 	for (int i=0;i<4;++i) {
 		vertices[i].x = vertices[i].y = 0.0f;
 		vertices[i].tu = vertices[i].tv = 0.0f;
@@ -100,7 +107,7 @@ void DevState::releaseData() {
 void DevState::blit(unsigned int x, unsigned int y, unsigned int w, unsigned int h) {
 	ods("D3D9: Blit %d %d %d %d", x, y, w, h);
 
-	if (! texTexture)
+	if (! texTexture || !a_ucTexture)
 		return;
 
 	D3DLOCKED_RECT lr;
@@ -192,6 +199,22 @@ void DevState::releaseAll() {
 }
 
 void DevState::draw() {
+	clock_t t = clock();
+	float elapsed = static_cast<float>(t - timeT) / CLOCKS_PER_SEC;
+	++frameCount;
+	if (elapsed > OVERLAY_FPS_INTERVAL) {
+		OverlayMsg om;
+		om.omh.uiMagic = OVERLAY_MAGIC_NUMBER;
+		om.omh.uiType = OVERLAY_MSGTYPE_FPS;
+		om.omh.iLength = sizeof(OverlayMsgFps);
+		om.omf.fps = frameCount / elapsed;
+
+		sendMessage(om);
+
+		frameCount = 0;
+		timeT = t;
+	}
+
 	D3DVIEWPORT9 vp;
 	dev->GetViewport(&vp);
 
@@ -257,7 +280,7 @@ void DevState::createCleanState() {
 	dev->SetRenderState(D3DRS_WRAP0, FALSE); // 0x80
 
 	dev->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
-	dev->SetRenderState(D3DRS_SRCBLEND,  D3DBLEND_SRCALPHA);
+	dev->SetRenderState(D3DRS_SRCBLEND,  D3DBLEND_ONE);
 	dev->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
 
 	dev->SetRenderState(D3DRS_ALPHATESTENABLE, TRUE);
@@ -429,6 +452,9 @@ static ULONG __stdcall myRelease(IDirect3DDevice9 *idd) {
 			ds->refCount--;
 		}
 
+		if (ds->refCount <= 1)
+			ds->disconnect();
+
 		if (ds->refCount >= 0)
 			return ds->refCount + ds->initRefCount;
 
@@ -701,7 +727,7 @@ extern "C" __declspec(dllexport) void __cdecl PrepareD3D9() {
 					ods("Direct3DCreate9Ex is not in D3D9 library");
 				} else {
 					IDirect3D9Ex *id3d9 = NULL;
-					HRESULT hr = d3dc9ex(D3D_SDK_VERSION, &id3d9);
+					d3dc9ex(D3D_SDK_VERSION, &id3d9);
 					if (id3d9) {
 						void ***vtbl = (void ***) id3d9;
 						void *pCreateEx = (*vtbl)[20];
