@@ -37,231 +37,13 @@ static HHOOK hhookWnd = 0;
 HMODULE hSelf = NULL;
 static BOOL bMumble = FALSE;
 static BOOL bDebug = FALSE;
+static BOOL bBlackListed = FALSE;
 
 static HardHook hhLoad;
 static HardHook hhLoadW;
-void *HardHook::pCode = NULL;
-unsigned int HardHook::uiCode = 0;
 
 static SharedData *sd;
 
-HardHook::HardHook() {
-	int i;
-	baseptr = NULL;
-	for (i=0;i<6;i++)
-		orig[i]=replace[i]=0;
-}
-
-static unsigned int modrmbytes(unsigned char a, unsigned char b) {
-	unsigned char lower = (a & 0x0f);
-	if (a >= 0xc0) {
-		return 0;
-	} else if (a >= 0x80) {
-		if ((lower == 4) || (lower == 12))
-			return 5;
-		else
-			return 4;
-	} else if (a >= 0x40) {
-		if ((lower == 4) || (lower == 12))
-			return 2;
-		else
-			return 1;
-
-	} else {
-		if ((lower == 4) || (lower == 12)) {
-			if ((b & 0x07) == 0x05)
-				return 5;
-			else
-				return 1;
-		} else if ((lower == 5) || (lower == 13))
-			return 4;
-		return 0;
-	}
-}
-
-void *HardHook::cloneCode(void **porig) {
-	if (! pCode || uiCode > 4000) {
-		uiCode = 0;
-		pCode = VirtualAlloc(NULL, 4096, MEM_COMMIT | MEM_RESERVE, PAGE_EXECUTE_READWRITE);
-	}
-
-	unsigned char *o = (unsigned char *) *porig;
-	unsigned char *n = (unsigned char *) pCode;
-	n += uiCode;
-	unsigned int idx = 0;
-
-	while (*o == 0xe9) {
-		int *iptr = reinterpret_cast<int *>(o+1);
-		o += *iptr + 5;
-
-		ods("Chaining from %p to %p", *porig, o);
-		*porig = o;
-	}
-
-	do {
-		unsigned char opcode = o[idx];
-		unsigned char a = o[idx+1];
-		unsigned char b = o[idx+2];
-		unsigned int extra = 0;
-
-		n[idx] = opcode;
-		idx++;
-
-		switch (opcode) {
-			case 0x50: // PUSH
-			case 0x51:
-			case 0x52:
-			case 0x53:
-			case 0x54:
-			case 0x55:
-			case 0x56:
-			case 0x57:
-			case 0x58: // POP
-			case 0x59:
-			case 0x5a:
-			case 0x5b:
-			case 0x5c:
-			case 0x5d:
-			case 0x5e:
-			case 0x5f:
-				break;
-			case 0x68:
-				extra = 4;
-				break;
-			case 0x81: // CMP immediate
-				extra = modrmbytes(a,b) + 5;
-				break;
-			case 0x83:	// CMP
-				extra = modrmbytes(a,b) + 2;
-				break;
-			case 0x8b:	// MOV
-				extra = modrmbytes(a,b) + 1;
-				break;
-			default:
-				fods("Unknown opcode at %d: %2x %2x %2x %2x %2x %2x %2x %2x %2x %2x %2x %2x", idx-1, o[0], o[1], o[2], o[3], o[4], o[5], o[6], o[7], o[8], o[9], o[10], o[11]);
-				return NULL;
-				break;
-		}
-		for (unsigned int i=0;i<extra;++i)
-			n[idx+i] = o[idx+i];
-		idx += extra;
-
-	} while (idx < 6);
-
-	n[idx++] = 0xe9;
-	int offs = o - n - 5;
-
-	int *iptr = reinterpret_cast<int *>(&n[idx]);
-	*iptr = offs;
-	idx += 4;
-
-	uiCode += idx;
-	FlushInstructionCache(GetCurrentProcess(), n, idx);
-
-	return n;
-}
-
-void HardHook::setup(voidFunc func, voidFunc replacement) {
-	int i;
-	DWORD oldProtect, restoreProtect;
-
-	if (baseptr)
-		return;
-
-	unsigned char *fptr = reinterpret_cast<unsigned char *>(func);
-	unsigned char *nptr = reinterpret_cast<unsigned char *>(replacement);
-
-	ods("HH: Asked to replace %p with %p", func, replacement);
-
-	if (VirtualProtect(fptr, 16, PAGE_EXECUTE_READ, &oldProtect)) {
-		call = (voidFunc) cloneCode((void **) &fptr);
-
-		if (call) {
-			bTrampoline = true;
-		} else {
-			bTrampoline = false;
-			call = func;
-		}
-
-		unsigned char **iptr = reinterpret_cast<unsigned char **>(&replace[1]);
-		*iptr = nptr;
-		replace[0] = 0x68;
-		replace[5] = 0xc3;
-
-		for (i=0;i<6;i++)
-			orig[i]=fptr[i];
-
-		baseptr = fptr;
-		inject(true);
-
-		VirtualProtect(fptr, 16, oldProtect, &restoreProtect);
-	} else {
-		ods("Failed initial vprotect");
-	}
-}
-
-void HardHook::setupInterface(IUnknown *unkn, LONG funcoffset, voidFunc replacement) {
-	ods("HH: Replacing %p function #%ld", unkn, funcoffset);
-	void **ptr = reinterpret_cast<void **>(unkn);
-	ptr = reinterpret_cast<void **>(ptr[0]);
-	setup(reinterpret_cast<voidFunc>(ptr[funcoffset]), replacement);
-}
-
-void HardHook::inject(bool force) {
-	DWORD oldProtect, restoreProtect;
-	int i;
-
-	if (! baseptr)
-		return;
-	if (! force && bTrampoline)
-		return;
-
-	if (VirtualProtect(baseptr, 6, PAGE_EXECUTE_READWRITE, &oldProtect)) {
-		for (i=0;i<6;i++)
-			baseptr[i] = replace[i];
-		VirtualProtect(baseptr, 6, oldProtect, &restoreProtect);
-		FlushInstructionCache(GetCurrentProcess(),baseptr, 6);
-	}
-	for (i=0;i<6;i++)
-		if (baseptr[i] != replace[i])
-			ods("HH: Injection failure at byte %d", i);
-}
-
-void HardHook::restore(bool force) {
-	DWORD oldProtect, restoreProtect;
-	int i;
-
-	if (! baseptr)
-		return;
-	if (! force && bTrampoline)
-		return;
-
-	if (VirtualProtect(baseptr, 6, PAGE_EXECUTE_READWRITE, &oldProtect)) {
-		for (i=0;i<6;i++)
-			baseptr[i] = orig[i];
-		VirtualProtect(baseptr, 6, oldProtect, &restoreProtect);
-		FlushInstructionCache(GetCurrentProcess(),baseptr, 6);
-	}
-}
-
-void HardHook::print() {
-	ods("HH: %02x %02x %02x %02x %02x => %02x %02x %02x %02x %02x (%02x %02x %02x %02x %02x)",
-	    orig[0], orig[1], orig[2], orig[3], orig[4],
-	    replace[0], replace[1], replace[2], replace[3], replace[4],
-	    baseptr[0], baseptr[1], baseptr[2], baseptr[3], baseptr[4]);
-}
-
-void HardHook::check() {
-	if (memcmp(baseptr, replace, 6) != 0) {
-		if (memcmp(baseptr, orig, 6) == 0) {
-			ods("HH: Restoring function %p", baseptr);
-			inject(true);
-		} else {
-			ods("HH: Function %p replaced by third party. Lost.");
-		}
-	}
-}
-
 FakeInterface::FakeInterface(IUnknown *orig, int entries) {
 	this->pOriginal = orig;
 	pAssembly = VirtualAlloc(NULL, entries * 256, MEM_COMMIT | MEM_RESERVE, PAGE_EXECUTE_READWRITE);
@@ -346,22 +128,29 @@ Mutex::~Mutex() {
 	LeaveCriticalSection(&cs);
 }
 
-void __cdecl fods(const char *format, ...) {
-	char    buf[4096], *p = buf;
-	va_list args;
+void __cdecl _ods_out(const char *format, va_list *args) {
+	char buf[4096], *p = buf + 2;
+
+	buf[0] = 'M'; // Add a prefix
+	buf[1] = ':';
+
+	// Format but be aware of space taken by prefix
+	int len = _vsnprintf_s(p, sizeof(buf) - 3, _TRUNCATE, format, *args);
 
-	va_start(args, format);
-	int len = _vsnprintf_s(p, sizeof(buf) - 1, _TRUNCATE, format, args);
-	va_end(args);
 
 	if (len <= 0)
 		return;
 
 	p += len;
 
-	while (p > buf  &&  isspace(p[-1]))
+	// Truncate trailing spaces
+	while (p > (buf + 2) && isspace(p[-1]))
 		*--p = '\0';
 
+	// Add custom termination
+	if (p > (buf + sizeof(buf) - 3)) { // Make sure there's space
+		p = buf + sizeof(buf) - 3;
+	}
 	*p++ = '\r';
 	*p++ = '\n';
 	*p   = '\0';
@@ -369,32 +158,31 @@ void __cdecl fods(const char *format, ...) {
 	OutputDebugStringA(buf);
 }
 
+void __cdecl fods(const char *format, ...) {
+	va_list args;
+
+	va_start(args, format);
+	_ods_out(format, &args);
+	va_end(args);
+}
 
 void __cdecl ods(const char *format, ...) {
 #ifndef DEBUG
 	if (!bDebug)
 		return;
 #endif
-	char    buf[4096], *p = buf;
 	va_list args;
 
 	va_start(args, format);
-	int len = _vsnprintf_s(p, sizeof(buf) - 1, _TRUNCATE, format, args);
+	_ods_out(format, &args);
 	va_end(args);
+}
 
-	if (len <= 0)
-		return;
-
-	p += len;
-
-	while (p > buf  &&  isspace(p[-1]))
-		*--p = '\0';
-
-	*p++ = '\r';
-	*p++ = '\n';
-	*p   = '\0';
-
-	OutputDebugStringA(buf);
+void __cdecl checkForWPF() {
+	if (!bBlackListed && (GetModuleHandleW(L"wpfgfx_v0300.dll") || GetModuleHandleW(L"wpfgfx_v0400.dll"))) {
+		ods("Blacklisted for loading WPF library");
+		bBlackListed = TRUE;
+	}
 }
 
 Pipe::Pipe() {
@@ -404,6 +192,8 @@ Pipe::Pipe() {
 
 	omMsg.omh.iLength = -1;
 
+	dwAlreadyRead = 0;
+
 	uiWidth = uiHeight = 0;
 	uiLeft = uiRight = uiTop = uiBottom = 0;
 }
@@ -464,6 +254,17 @@ void Pipe::checkMessage(unsigned int w, unsigned int h) {
 
 		uiWidth = 0;
 		uiHeight = 0;
+
+		OverlayMsg om;
+		om.omh.uiMagic = OVERLAY_MAGIC_NUMBER;
+		om.omh.uiType = OVERLAY_MSGTYPE_PID;
+		om.omh.iLength = sizeof(OverlayMsgPid);
+		om.omp.pid = GetCurrentProcessId();
+
+		if (!sendMessage(om))
+			return;
+
+		ods("Pipe: SentPid");
 	}
 
 	if ((uiWidth != w) || (uiHeight != h)) {
@@ -485,6 +286,8 @@ void Pipe::checkMessage(unsigned int w, unsigned int h) {
 		ods("Pipe: SentInit %d %d", w, h);
 	}
 
+	std::vector<RECT> blits;
+
 	while (1) {
 		DWORD dwBytesLeft;
 		DWORD dwBytesRead;
@@ -495,30 +298,48 @@ void Pipe::checkMessage(unsigned int w, unsigned int h) {
 			return;
 		}
 
+		if (! dwBytesLeft)
+			break;
+
 		if (omMsg.omh.iLength == -1) {
-			if (dwBytesLeft < sizeof(OverlayMsgHeader))
-				break;
-
-			ReadFile(hSocket, omMsg.headerbuffer, sizeof(OverlayMsgHeader), &dwBytesRead, NULL);
-			dwBytesLeft -= sizeof(OverlayMsgHeader);
-
-			if (dwBytesRead != sizeof(OverlayMsgHeader)) {
-				ods("Pipe: Short header read %d", dwBytesRead);
+			if (! ReadFile(hSocket, reinterpret_cast<unsigned char *>(omMsg.headerbuffer) + dwAlreadyRead, sizeof(OverlayMsgHeader) - dwAlreadyRead, &dwBytesRead, NULL)) {
+				ods("Pipe: Read header fail");
 				disconnect();
 				return;
 			}
+
+			dwBytesLeft -= dwBytesRead;
+			dwAlreadyRead += dwBytesRead;
+
+			if (dwAlreadyRead != sizeof(OverlayMsgHeader)) {
+				break;
+			}
+
+			dwAlreadyRead = 0;
+
+			if (omMsg.omh.uiMagic != OVERLAY_MAGIC_NUMBER) {
+				ods("Pipe: Invalid magic number %x", omMsg.omh.uiMagic);
+				disconnect();
+				return;
+			}
+
+			if (static_cast<int>(dwBytesLeft) < omMsg.omh.iLength)
+				continue;
 		}
 
-		if (static_cast<int>(dwBytesLeft) < omMsg.omh.iLength)
-			break;
-
-		ReadFile(hSocket, omMsg.msgbuffer, omMsg.omh.iLength, &dwBytesRead, NULL);
-		if (dwBytesRead != omMsg.omh.iLength) {
-			ods("Pipe: Short body read %d/%d", dwBytesRead, omMsg.omh.iLength);
+		if (! ReadFile(hSocket, reinterpret_cast<unsigned char *>(omMsg.msgbuffer) + dwAlreadyRead, omMsg.omh.iLength - dwAlreadyRead, &dwBytesRead, NULL)) {
+			ods("Pipe: Read data fail");
 			disconnect();
 			return;
 		}
 
+		dwAlreadyRead += dwBytesRead;
+
+		if (static_cast<int>(dwBytesLeft) < omMsg.omh.iLength)
+			continue;
+
+		dwAlreadyRead = 0;
+
 		switch (omMsg.omh.uiType) {
 			case OVERLAY_MSGTYPE_SHMEM: {
 					wchar_t memname[2048];
@@ -550,6 +371,7 @@ void Pipe::checkMessage(unsigned int w, unsigned int h) {
 						ods("Pipe: Failed to map memory");
 						CloseHandle(hMemory);
 						hMemory = NULL;
+						break;
 					}
 
 					MEMORY_BASIC_INFORMATION mbi;
@@ -575,8 +397,21 @@ void Pipe::checkMessage(unsigned int w, unsigned int h) {
 				}
 				break;
 			case OVERLAY_MSGTYPE_BLIT: {
-					if (a_ucTexture)
-						blit(omMsg.omb.x, omMsg.omb.y, omMsg.omb.w, omMsg.omb.h);
+					RECT r = {omMsg.omb.x, omMsg.omb.y, omMsg.omb.x + omMsg.omb.w, omMsg.omb.y + omMsg.omb.h};
+
+					std::vector<RECT>::iterator i = blits.begin();
+					while (i != blits.end()) {
+						RECT is;
+						if (::IntersectRect(&is, &r, & *i)) {
+							::UnionRect(&is, &r, & *i);
+							r = is;
+							blits.erase(i);
+							i = blits.begin();
+						} else {
+							++i;
+						}
+					}
+					blits.push_back(r);
 				}
 				break;
 			case OVERLAY_MSGTYPE_ACTIVE: {
@@ -593,18 +428,13 @@ void Pipe::checkMessage(unsigned int w, unsigned int h) {
 		}
 		omMsg.omh.iLength = -1;
 	}
-}
 
-static bool bBlackListed = false;
-static const char *blacklist[] = {
-	"iexplore.exe",
-	"ieuser.exe",
-	"vlc.exe",
-	"dbgview.exe",
-	"opera.exe",
-	"explorer.exe",
-	NULL
-};
+	if (!a_ucTexture)
+		return;
+
+	for (std::vector<RECT>::iterator i = blits.begin(); i != blits.end(); ++i)
+		blit((*i).left, (*i).top, (*i).right - (*i).left, (*i).bottom - (*i).top);
+}
 
 typedef HMODULE(__stdcall *LoadLibraryAType)(const char *);
 static HMODULE WINAPI MyLoadLibrary(const char *lpFileName) {
@@ -614,7 +444,7 @@ static HMODULE WINAPI MyLoadLibrary(const char *lpFileName) {
 	HMODULE h = oLoadLibrary(lpFileName);
 //	ods("Library %s loaded to %p", lpFileName, h);
 
-	if (! bMumble) {
+	if (! bBlackListed) {
 		checkD3D9Hook();
 		checkDXGIHook();
 		checkOpenGLHook();
@@ -632,7 +462,9 @@ static HMODULE WINAPI MyLoadLibraryW(const wchar_t *lpFileName) {
 	HMODULE h = oLoadLibrary(lpFileName);
 	ods("Library %ls wloaded to %p", lpFileName, h);
 
-	if (! bMumble) {
+	checkForWPF();
+
+	if (! bBlackListed) {
 		checkD3D9Hook();
 		checkDXGIHook();
 		checkOpenGLHook();
@@ -685,43 +517,124 @@ extern "C" __declspec(dllexport) unsigned int __cdecl GetOverlayMagicVersion() {
 
 extern "C" BOOL WINAPI DllMain(HINSTANCE, DWORD fdwReason, LPVOID) {
 	char procname[1024+64];
-	GetModuleFileName(NULL, procname, 1024);
-
+	GetModuleFileNameA(NULL, procname, 1024);
 	switch (fdwReason) {
 		case DLL_PROCESS_ATTACH: {
 				Mutex::init();
 				char *p = strrchr(procname, '\\');
+				if (!p) {
+					// No blacklisting if the file has no path
+				} else if ((_stricmp(p+1, "mumble.exe")==0) || (_stricmp(p+1, "mumble11x.exe")==0)) {
+					ods("Attached to self");
+					bBlackListed = TRUE;
+					bMumble = TRUE;
+				} else {
+					int i = 0;
+					DWORD buffsize = MAX_PATH * 20; // Initial buffer size for registry operation
 
-				if (p) {
-					if ((_stricmp(p+1, "mumble.exe")==0) || (_stricmp(p+1, "mumble11x.exe")==0))
-						bMumble = TRUE;
-					int i =0;
-					while (blacklist[i]) {
-						if (_stricmp(p+1,blacklist[i])==0) {
-							bBlackListed = true;
-							return TRUE;
+					bool usewhitelist;
+					HKEY key = NULL;
+
+					bool success = true;
+					char *buffer = new char[buffsize];
+
+					DWORD tmpsize = buffsize - 1;
+					success = (RegOpenKeyExA(HKEY_CURRENT_USER, "Software\\Mumble\\Mumble\\overlay", NULL, KEY_READ, &key) == ERROR_SUCCESS) &&
+					          (RegQueryValueExA(key, "usewhitelist", NULL, NULL, (LPBYTE)buffer, &tmpsize) == ERROR_SUCCESS);
+
+					if (success) {
+						buffer[tmpsize] = '\0';
+						usewhitelist = (_stricmp(buffer, "true") == 0);
+						tmpsize = buffsize - 1;
+
+						DWORD ret;
+						while ((ret = RegQueryValueExA(key, usewhitelist ? "whitelist" : "blacklist", NULL, NULL, (LPBYTE)buffer, &tmpsize)) == ERROR_MORE_DATA) {
+							// Increase the buffsize according to the required size RegQuery wrote into tmpszie so we can read the whole value
+							delete []buffer;
+							buffsize = tmpsize + 1;
+							buffer = new char[buffsize];
 						}
-						i++;
-					}
-					strcpy_s(p+1, 64, "nooverlay");
 
-					HANDLE h = CreateFile(procname, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
+						success = (ret == ERROR_SUCCESS);
+					}
+
+					if (key)
+						RegCloseKey(key);
+
+					if (success) {
+						buffer[tmpsize] = '\0';
+						unsigned int pos = 0;
+
+						if (usewhitelist) {
+							bool onwhitelist = false;
+							while (buffer[pos] != 0 && pos < buffsize) {
+								if (_stricmp(procname, buffer + pos) == 0 || _stricmp(p+1, buffer + pos) == 0) {
+									fods("Overlay enabled for whitelisted '%s'", buffer + pos);
+									onwhitelist = true;
+									break;
+								}
+								pos += strlen(buffer + pos) + 1;
+							}
+
+							if (!onwhitelist) {
+								ods("No whitelist entry found for '%s', auto-blacklisted", procname);
+								bBlackListed = TRUE;
+								break;
+							}
+						} else {
+							while (buffer[pos] != 0 && pos < buffsize) {
+								if (_stricmp(procname, buffer + pos) == 0 || _stricmp(p+1, buffer + pos) == 0) {
+									fods("Overlay blacklist entry found for '%s'", buffer + pos);
+									bBlackListed = TRUE;
+									break;
+								}
+								pos += strlen(buffer + pos) + 1;
+							}
+						}
+					} else {
+						// If there is no list in the registry fallback to using the default blacklist
+						fods("Overlay fallback to default blacklist");
+						while (overlayBlacklist[i]) {
+							if (_stricmp(procname, overlayBlacklist[i]) == 0 || _stricmp(p+1, overlayBlacklist[i])==0) {
+								fods("Overlay default blacklist entry found for '%s'", overlayBlacklist[i]);
+								bBlackListed = TRUE;
+								break;
+							}
+							i++;
+						}
+					}
+
+					// Make sure to always free/destroy buffer & heap
+					delete []buffer;
+
+					if (bBlackListed)
+						return TRUE;
+
+					char fname[sizeof(procname)];
+					p = fname + (p - procname);
+					strncpy_s(fname, sizeof(fname), procname, p - procname + 1);
+
+					strcpy_s(p+1, 64, "nooverlay");
+					HANDLE h = CreateFile(fname, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
 					if (h != INVALID_HANDLE_VALUE) {
 						CloseHandle(h);
-						fods("Overlay disable %s found", procname);
+						fods("Overlay disable %s found", fname);
+						bBlackListed = TRUE;
 						return TRUE;
 					}
 
 					strcpy_s(p+1, 64, "debugoverlay");
-
-					h = CreateFile(procname, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
+					h = CreateFile(fname, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
 					if (h != INVALID_HANDLE_VALUE) {
 						CloseHandle(h);
-						fods("Overlay debug %s found", procname);
+						fods("Overlay debug %s found", fname);
 						bDebug = TRUE;
 					}
-				}
 
+					checkForWPF();
+					if (bBlackListed)
+						return TRUE;
+				}
 				ods("Lib: ProcAttach: %s", procname);
 
 				hHookMutex = CreateMutex(NULL, false, "MumbleHookMutex");
@@ -763,13 +676,11 @@ extern "C" BOOL WINAPI DllMain(HINSTANCE, DWORD fdwReason, LPVOID) {
 					checkD3D9Hook(true);
 					checkDXGIHook(true);
 					checkOpenGLHook();
-					ods("Injected");
+					ods("Injected into %s", procname);
 				}
 			}
 			break;
 		case DLL_PROCESS_DETACH: {
-				if (bBlackListed)
-					return TRUE;
 				ods("Lib: ProcDetach: %s", procname);
 				hhLoad.restore(true);
 				hhLoadW.restore(true);
@@ -783,11 +694,16 @@ extern "C" BOOL WINAPI DllMain(HINSTANCE, DWORD fdwReason, LPVOID) {
 			break;
 		case DLL_THREAD_ATTACH: {
 				static bool bTriedHook = false;
-				if (sd && ! bTriedHook && ! bMumble) {
+				if (!bBlackListed && sd && ! bTriedHook) {
 					bTriedHook = true;
-					checkD3D9Hook();
-					checkDXGIHook();
-					checkOpenGLHook();
+					checkForWPF();
+
+					if (!bBlackListed) {
+						checkD3D9Hook();
+						checkDXGIHook();
+						checkOpenGLHook();
+						ods("Injected to thread of %s", procname);
+					}
 				}
 			}
 			break;
