@@ -60,6 +60,22 @@ struct OverlayMsgActive {
 	unsigned int x, y, w, h;
 };
 
+#define OVERLAY_MSGTYPE_PID 4
+struct OverlayMsgPid {
+	unsigned int pid;
+};
+
+#define OVERLAY_MSGTYPE_FPS 5
+struct OverlayMsgFps {
+	float fps;
+};
+#define OVERLAY_FPS_INTERVAL 0.25f
+
+#define OVERLAY_MSGTYPE_INTERACTIVE 6
+struct OverlayMsgInteractive {
+	bool state;
+};
+
 struct OverlayMsg {
 	union {
 		char headerbuffer[1];
@@ -71,7 +87,37 @@ struct OverlayMsg {
 		struct OverlayMsgInit omi;
 		struct OverlayMsgBlit omb;
 		struct OverlayMsgActive oma;
+		struct OverlayMsgPid omp;
+		struct OverlayMsgFps omf;
+		struct OverlayMsgInteractive omin;
 	};
 };
 
+static const char *overlayBlacklist[] = {
+	"iexplore.exe",
+	"ieuser.exe",
+	"vlc.exe",
+	"dbgview.exe",
+	"opera.exe",
+	"chrome.exe",
+	"acrord32.exe",
+	"explorer.exe",
+	"wmpnscfg.exe",
+	"firefox.exe",
+	"wlmail.exe",	// Windows Live Suite (mshtml.dll)
+	"msnmsgr.exe",
+	"MovieMaker.exe",
+	"WLXPhotoGallery.exe",
+	"psi.exe", // Secunia PSI (uses mshtml.dll)
+	"Photoshop.exe",
+	"blender.exe",
+	"googleearth.exe",
+	"XBMC.exe", // http://xbmc.org/
+	"BOXEE.exe", // http://www.boxee.tv/
+	"hammer.exe", // VALVE Hammer Editor
+	"hlmv.exe", // Half-Life Model Viewer
+	"hlfaceposer.exe", // Face Poser (from Source SDK)
+	NULL
+};
+
 #endif
