@@ -30,51 +30,9 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-#define _USE_MATH_DEFINES
+#include "../mumble_plugin_win32.h"
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <windows.h>
-#include <tlhelp32.h>
-#include <math.h>
-
-#include "../mumble_plugin.h"
-
-HANDLE h = NULL;
-
-static DWORD getProcess(const wchar_t *exename) {
-	PROCESSENTRY32 pe;
-	DWORD pid = 0;
-
-	pe.dwSize = sizeof(pe);
-	HANDLE hSnap = CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);
-	if (hSnap != INVALID_HANDLE_VALUE) {
-		BOOL ok = Process32First(hSnap, &pe);
-
-		while (ok) {
-			if (wcscmp(pe.szExeFile, exename)==0) {
-				pid = pe.th32ProcessID;
-				break;
-			}
-			ok = Process32Next(hSnap, &pe);
-		}
-		CloseHandle(hSnap);
-	}
-	return pid;
-}
-
-static bool peekProc(VOID *base, VOID *dest, SIZE_T len) {
-	SIZE_T r;
-	BOOL ok=ReadProcessMemory(h, base, dest, len, &r);
-	return (ok && (r == len));
-}
-
-static void about(HWND h) {
-	::MessageBox(h, L"Reads audio position information from Call of Duty: Modern Warfare 2 Multiplayer v1.0.182.", L"Mumble CoDMW2 MP Plugin", MB_OK);
-}
-
-
-static int fetch(float *avatar_pos, float *avatar_front, float *avatar_top, float *camera_pos, float *camera_front, float *camera_top, std::string &context, std::wstring &identity) {
+static int fetch(float *avatar_pos, float *avatar_front, float *avatar_top, float *camera_pos, float *camera_front, float *camera_top, std::string &, std::wstring &) {
 	float viewHor, viewVer;
 	char state;
 
@@ -96,7 +54,7 @@ static int fetch(float *avatar_pos, float *avatar_front, float *avatar_top, floa
 
 			0x007F7A34		byte	Magical state value
 	*/
-	ok = peekProc((BYTE *) 0x007F7A34, &state, 1); // Magical state value
+	ok = peekProc((BYTE *) 0x007F8AB4, &state, 1); // Magical state value
 	if (! ok)
 		return false;
 	/*
@@ -111,11 +69,11 @@ static int fetch(float *avatar_pos, float *avatar_front, float *avatar_top, floa
 	if (state != 1)
 		return true; // This results in all vectors beeing zero which tells mumble to ignore them.
 
-	ok = peekProc((BYTE *) 0x008F0F78, avatar_pos+2, 4) &&	//Z
-	     peekProc((BYTE *) 0x008F0F7C, avatar_pos, 4) &&	//X
-	     peekProc((BYTE *) 0x008F0F80, avatar_pos+1, 4) && //Y
-	     peekProc((BYTE *) 0x008F0F88, &viewHor, 4) && //Hor
-	     peekProc((BYTE *) 0x008F0F84, &viewVer, 4); //Ver
+	ok = peekProc((BYTE *) 0x008F1FF8, avatar_pos+2, 4) &&	//Z
+	     peekProc((BYTE *) 0x008F1FFC, avatar_pos, 4) &&	//X
+	     peekProc((BYTE *) 0x008F2000, avatar_pos+1, 4) && //Y
+	     peekProc((BYTE *) 0x008F2008, &viewHor, 4) && //Hor
+	     peekProc((BYTE *) 0x008F2004, &viewVer, 4); //Ver
 
 	if (! ok)
 		return false;
@@ -165,54 +123,55 @@ static int fetch(float *avatar_pos, float *avatar_front, float *avatar_top, floa
 	return true;
 }
 
-static int trylock() {
-	h = NULL;
-	DWORD pid=getProcess(L"iw4mp.exe");
-	if (!pid)
-		return false;
-
-	h=OpenProcess(PROCESS_VM_READ, false, pid);
-	if (!h)
+static int trylock(const std::multimap<std::wstring, unsigned long long int> &pids) {
+	if (! initialize(pids, L"iw4mp.exe"))
 		return false;
 
 	float apos[3], afront[3], atop[3], cpos[3], cfront[3], ctop[3];
 	std::string context;
 	std::wstring identity;
 
-	if (fetch(apos, afront, atop, cpos, cfront, ctop, context, identity))
+	if (fetch(apos, afront, atop, cpos, cfront, ctop, context, identity)) {
 		return true;
-
-	CloseHandle(h);
-	h = NULL;
-	return false;
-}
-
-static void unlock() {
-	if (h) {
-		CloseHandle(h);
-		h = NULL;
+	} else {
+		generic_unlock();
+		return false;
 	}
 }
 
 static const std::wstring longdesc() {
-	return std::wstring(L"Supports Call of Duty: Modern Warfare 2 MP v1.0.182 only. No context or identity support yet.");
+	return std::wstring(L"Supports Call of Duty: Modern Warfare 2 MP v1.2.208 only. No context or identity support yet.");
 }
 
-static std::wstring description(L"Call of Duty: Modern Warfare 2 MP v1.0.182");
+static std::wstring description(L"Call of Duty: Modern Warfare 2 MP v1.2.208");
 static std::wstring shortname(L"Call of Duty: Modern Warfare 2 MP");
 
+static int trylock1() {
+	return trylock(std::multimap<std::wstring, unsigned long long int>());
+}
+
 static MumblePlugin codmw2plug = {
 	MUMBLE_PLUGIN_MAGIC,
 	description,
 	shortname,
-	about,
 	NULL,
-	trylock,
-	unlock,
+	NULL,
+	trylock1,
+	generic_unlock,
 	longdesc,
 	fetch
 };
 
+static MumblePlugin2 codmw2plug2 = {
+	MUMBLE_PLUGIN_MAGIC_2,
+	MUMBLE_PLUGIN_VERSION,
+	trylock
+};
+
 extern "C" __declspec(dllexport) MumblePlugin *getMumblePlugin() {
 	return &codmw2plug;
 }
+
+extern "C" __declspec(dllexport) MumblePlugin2 *getMumblePlugin2() {
+	return &codmw2plug2;
+}
