@@ -29,78 +29,15 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <windows.h>
-#include <tlhelp32.h>
-#include <string>
-#include <sstream>
-
-#define _USE_MATH_DEFINES
-#include <math.h>
-
-#include "../mumble_plugin.h"
+#include "../mumble_plugin_win32.h"
 
 using namespace std;
 
-HANDLE h;
 BYTE *posptr;
 BYTE *rotptr;
 BYTE *stateptr;
 BYTE *hostptr;
 
-static DWORD getProcess(const wchar_t *exename) {
-	PROCESSENTRY32 pe;
-	DWORD pid = 0;
-
-	pe.dwSize = sizeof(pe);
-	HANDLE hSnap = CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);
-	if (hSnap != INVALID_HANDLE_VALUE) {
-		BOOL ok = Process32First(hSnap, &pe);
-
-		while (ok) {
-			if (wcscmp(pe.szExeFile, exename)==0) {
-				pid = pe.th32ProcessID;
-				break;
-			}
-			ok = Process32Next(hSnap, &pe);
-		}
-		CloseHandle(hSnap);
-	}
-	return pid;
-}
-
-static BYTE *getModuleAddr(DWORD pid, const wchar_t *modname) {
-	MODULEENTRY32 me;
-	BYTE *addr = NULL;
-	me.dwSize = sizeof(me);
-	HANDLE hSnap = CreateToolhelp32Snapshot(TH32CS_SNAPMODULE, pid);
-	if (hSnap != INVALID_HANDLE_VALUE) {
-		BOOL ok = Module32First(hSnap, &me);
-
-		while (ok) {
-			if (wcscmp(me.szModule, modname)==0) {
-				addr = me.modBaseAddr;
-				break;
-			}
-			ok = Module32Next(hSnap, &me);
-		}
-		CloseHandle(hSnap);
-	}
-	return addr;
-}
-
-
-static bool peekProc(VOID *base, VOID *dest, SIZE_T len) {
-	SIZE_T r;
-	BOOL ok=ReadProcessMemory(h, base, dest, len, &r);
-	return (ok && (r == len));
-}
-
-static void about(HWND h) {
-	::MessageBox(h, L"Reads audio position information from Garry's Mod 11 (Build 3943). IP:Port context support.", L"Mumble Gmod Plugin", MB_OK);
-}
-
 static bool calcout(float *pos, float *rot, float *opos, float *front, float *top) {
 	float h = rot[0];
 	float v = rot[1];
@@ -129,7 +66,7 @@ static bool calcout(float *pos, float *rot, float *opos, float *front, float *to
 	return true;
 }
 
-static int fetch(float *avatar_pos, float *avatar_front, float *avatar_top, float *camera_pos, float *camera_front, float *camera_top, string &context, wstring &identity) {
+static int fetch(float *avatar_pos, float *avatar_front, float *avatar_top, float *camera_pos, float *camera_front, float *camera_top, string &context, wstring &) {
 	for (int i=0;i<3;i++)
 		avatar_pos[i] = avatar_front[i] = avatar_top[i] = camera_pos[i] = camera_front[i] = camera_top[i] = 0.0f;
 
@@ -182,22 +119,15 @@ static int fetch(float *avatar_pos, float *avatar_front, float *avatar_top, floa
 	return false;
 }
 
-static int trylock() {
-	h = NULL;
+static int trylock(const std::multimap<std::wstring, unsigned long long int> &pids) {
 	posptr = rotptr = NULL;
 
-	DWORD pid=getProcess(L"hl2.exe");
-	if (!pid)
+	if (! initialize(pids, L"hl2.exe", L"client.dll"))
 		return false;
-	BYTE *mod=getModuleAddr(pid, L"client.dll");
-	if (!mod)
-		return false;
-	BYTE *mod_engine=getModuleAddr(pid, L"engine.dll");
+
+	BYTE *mod_engine=getModuleAddr(L"engine.dll");
 	if (!mod_engine)
 		return false;
-	h=OpenProcess(PROCESS_VM_READ, false, pid);
-	if (!h)
-		return false;
 
 	// Check if we really have HL2DM running
 	/*
@@ -208,14 +138,14 @@ static int trylock() {
 		ip:port string		engine.dll+0x3909c4  (zero terminated ip:port, string)
 	*/
 	// Remember addresses for later
-	posptr = mod + 0x3dcad4;
-	rotptr = mod + 0x3dcae0;
-	stateptr = mod + 0x37e180;
-	hostptr = mod_engine + 0x3909c4;
+	posptr = pModule + 0x445820;
+	rotptr = pModule + 0x3F6E34;
+	stateptr = pModule + 0x3E3ACC;
+	hostptr = mod_engine + 0x383C8E;
 
 	// Check if we are really running hl2dm
-	char sMagic[4];
-	if (!peekProc(mod + 0x3a5674, sMagic, 4) || strncmp("Dm/$", sMagic, 4) !=0)
+	char sMagic[17];
+	if (!peekProc(mod_engine + 0x3782D0, sMagic, 17) || strncmp(".?AVIEngineVGui@@", sMagic, 17) !=0)
 		return false;
 
 	// Check if we can get meaningfull data from it
@@ -224,42 +154,47 @@ static int trylock() {
 	wstring sidentity;
 	string scontext;
 
-	if (fetch(apos, afront, atop, cpos, cfront, ctop, scontext, sidentity))
+	if (fetch(apos, afront, atop, cpos, cfront, ctop, scontext, sidentity)) {
 		return true;
-
-	// If it failed clean up
-	CloseHandle(h);
-	h = NULL;
-	return false;
-}
-
-static void unlock() {
-	if (h) {
-		CloseHandle(h);
-		h = NULL;
+	} else {
+		generic_unlock();
+		return false;
 	}
-	return;
 }
 
 static const wstring longdesc() {
-	return wstring(L"Supports HL2DM build 3945. No identity support yet.");
+	return wstring(L"Supports HL2DM build 4421. No identity support yet.");
 }
 
-static wstring description(L"Half-Life 2: Deathmatch (Build 3945)");
+static wstring description(L"Half-Life 2: Deathmatch (Build 4421)");
 static wstring shortname(L"Half-Life 2: Deathmatch");
 
+static int trylock1() {
+	return trylock(std::multimap<std::wstring, unsigned long long int>());
+}
+
 static MumblePlugin hl2dmplug = {
 	MUMBLE_PLUGIN_MAGIC,
 	description,
 	shortname,
-	about,
 	NULL,
-	trylock,
-	unlock,
+	NULL,
+	trylock1,
+	generic_unlock,
 	longdesc,
 	fetch
 };
 
+static MumblePlugin2 hl2dmplug2 = {
+	MUMBLE_PLUGIN_MAGIC_2,
+	MUMBLE_PLUGIN_VERSION,
+	trylock
+};
+
 extern "C" __declspec(dllexport) MumblePlugin *getMumblePlugin() {
 	return &hl2dmplug;
 }
+
+extern "C" __declspec(dllexport) MumblePlugin2 *getMumblePlugin2() {
+	return &hl2dmplug2;
+}
