@@ -28,72 +28,11 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <windows.h>
-#include <tlhelp32.h>
+#include "../mumble_plugin_win32.h"
 
-#define _USE_MATH_DEFINES
-#include <math.h>
-
-#include "../mumble_plugin.h"
-
-HANDLE h;
 BYTE *posptr;
 BYTE *rotptr;
-BYTE *stateptr;
-
-static DWORD getProcess(const wchar_t *exename) {
-	PROCESSENTRY32 pe;
-	DWORD pid = 0;
-
-	pe.dwSize = sizeof(pe);
-	HANDLE hSnap = CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);
-	if (hSnap != INVALID_HANDLE_VALUE) {
-		BOOL ok = Process32First(hSnap, &pe);
-
-		while (ok) {
-			if (wcscmp(pe.szExeFile, exename)==0) {
-				pid = pe.th32ProcessID;
-				break;
-			}
-			ok = Process32Next(hSnap, &pe);
-		}
-		CloseHandle(hSnap);
-	}
-	return pid;
-}
-
-static BYTE *getModuleAddr(DWORD pid, const wchar_t *modname) {
-	MODULEENTRY32 me;
-	BYTE *addr = NULL;
-	me.dwSize = sizeof(me);
-	HANDLE hSnap = CreateToolhelp32Snapshot(TH32CS_SNAPMODULE, pid);
-	if (hSnap != INVALID_HANDLE_VALUE) {
-		BOOL ok = Module32First(hSnap, &me);
-
-		while (ok) {
-			if (wcscmp(me.szModule, modname)==0) {
-				addr = me.modBaseAddr;
-				break;
-			}
-			ok = Module32Next(hSnap, &me);
-		}
-		CloseHandle(hSnap);
-	}
-	return addr;
-}
-
-
-static bool peekProc(VOID *base, VOID *dest, SIZE_T len) {
-	SIZE_T r;
-	BOOL ok=ReadProcessMemory(h, base, dest, len, &r);
-	return (ok && (r == len));
-}
-
-static void about(HWND h) {
-	::MessageBox(h, L"Reads audio position information from Left 4 Dead 2 (Build 4079).", L"Mumble L4D Plugin", MB_OK);
-}
+// BYTE *stateptr;
 
 static bool calcout(float *pos, float *rot, float *opos, float *front, float *top) {
 	float h = rot[0];
@@ -127,25 +66,15 @@ static bool calcout(float *pos, float *rot, float *opos, float *front, float *to
 	return true;
 }
 
-static int trylock() {
-	h = NULL;
+static int trylock(const std::multimap<std::wstring, unsigned long long int> &pids) {
 	posptr = rotptr = NULL;
 
-	DWORD pid=getProcess(L"left4dead2.exe");
-	if (!pid)
+	if (! initialize(pids, L"left4dead2.exe", L"client.dll"))
 		return false;
 
-	BYTE *mod=getModuleAddr(pid, L"client.dll");
-	if (!mod)
-		return false;
-
-	h=OpenProcess(PROCESS_VM_READ, false, pid);
-	if (!h)
-		return false;
-
-	posptr = mod + 0x6DA6A0;
-	rotptr = mod + 0x626554;
-	stateptr = mod + 0x61FF6C;
+	posptr = pModule + 0x6F8340;
+	rotptr = pModule + 0x6AD900;
+	// stateptr = pModule + 0x690A3C;
 
 	float pos[3];
 	float rot[3];
@@ -157,17 +86,9 @@ static int trylock() {
 	if (ok)
 		return calcout(pos, rot, opos, top, front);
 
-	CloseHandle(h);
-	h = NULL;
-	return false;
-}
+	generic_unlock();
 
-static void unlock() {
-	if (h) {
-		CloseHandle(h);
-		h = NULL;
-	}
-	return;
+	return false;
 }
 
 static int fetch(float *avatar_pos, float *avatar_front, float *avatar_top, float *camera_pos, float *camera_front, float *camera_top, std::string &context, std::wstring &identity) {
@@ -176,14 +97,15 @@ static int fetch(float *avatar_pos, float *avatar_front, float *avatar_top, floa
 
 	float ipos[3], rot[3];
 	bool ok;
-	char state;
+	// char state;
 
+	// stateptr returns byte values: 0 when map is not loaded; 8 when loaded
 	ok = peekProc(posptr, ipos, 12) &&
-	     peekProc(rotptr, rot, 12) &&
-	     peekProc(stateptr, &state, 1);
+	     peekProc(rotptr, rot, 12);
+	// peekProc(stateptr, &state, 1);
 
-	if (state == 0)
-		return true; // This results in all vectors beeing zero which tells Mumble to ignore them.
+	// if (state == 0)
+	// 	return true; // This results in all vectors beeing zero which tells Mumble to ignore them.
 
 	if (ok) {
 		int res = calcout(ipos, rot, avatar_pos, avatar_front, avatar_top);
@@ -205,24 +127,38 @@ static int fetch(float *avatar_pos, float *avatar_front, float *avatar_top, floa
 }
 
 static const std::wstring longdesc() {
-	return std::wstring(L"Supports L4D2 build 4079. No identity or context support yet.");
+	return std::wstring(L"Supports L4D2 build 4391. No identity or context support yet.");
 }
 
-static std::wstring description(L"Left 4 Dead 2 (Build 4079)");
+static std::wstring description(L"Left 4 Dead 2 (Build 4391)");
 static std::wstring shortname(L"Left 4 Dead 2");
 
+static int trylock1() {
+	return trylock(std::multimap<std::wstring, unsigned long long int>());
+}
+
 static MumblePlugin l4d2plug = {
 	MUMBLE_PLUGIN_MAGIC,
 	description,
 	shortname,
-	about,
 	NULL,
-	trylock,
-	unlock,
+	NULL,
+	trylock1,
+	generic_unlock,
 	longdesc,
 	fetch
 };
 
+static MumblePlugin2 l4d2plug2 = {
+	MUMBLE_PLUGIN_MAGIC_2,
+	MUMBLE_PLUGIN_VERSION,
+	trylock
+};
+
 extern "C" __declspec(dllexport) MumblePlugin *getMumblePlugin() {
 	return &l4d2plug;
 }
+
+extern "C" __declspec(dllexport) MumblePlugin2 *getMumblePlugin2() {
+	return &l4d2plug2;
+}
