@@ -29,78 +29,15 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <windows.h>
-#include <tlhelp32.h>
-#include <string>
-#include <sstream>
-
-#define _USE_MATH_DEFINES
-#include <math.h>
-
-#include "../mumble_plugin.h"
+#include "../mumble_plugin_win32.h"
 
 using namespace std;
 
-HANDLE h;
 BYTE *posptr;
 BYTE *rotptr;
 BYTE *stateptr;
 BYTE *hostptr;
 
-static DWORD getProcess(const wchar_t *exename) {
-	PROCESSENTRY32 pe;
-	DWORD pid = 0;
-
-	pe.dwSize = sizeof(pe);
-	HANDLE hSnap = CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);
-	if (hSnap != INVALID_HANDLE_VALUE) {
-		BOOL ok = Process32First(hSnap, &pe);
-
-		while (ok) {
-			if (wcscmp(pe.szExeFile, exename)==0) {
-				pid = pe.th32ProcessID;
-				break;
-			}
-			ok = Process32Next(hSnap, &pe);
-		}
-		CloseHandle(hSnap);
-	}
-	return pid;
-}
-
-static BYTE *getModuleAddr(DWORD pid, const wchar_t *modname) {
-	MODULEENTRY32 me;
-	BYTE *addr = NULL;
-	me.dwSize = sizeof(me);
-	HANDLE hSnap = CreateToolhelp32Snapshot(TH32CS_SNAPMODULE, pid);
-	if (hSnap != INVALID_HANDLE_VALUE) {
-		BOOL ok = Module32First(hSnap, &me);
-
-		while (ok) {
-			if (wcscmp(me.szModule, modname)==0) {
-				addr = me.modBaseAddr;
-				break;
-			}
-			ok = Module32Next(hSnap, &me);
-		}
-		CloseHandle(hSnap);
-	}
-	return addr;
-}
-
-
-static bool peekProc(VOID *base, VOID *dest, SIZE_T len) {
-	SIZE_T r;
-	BOOL ok=ReadProcessMemory(h, base, dest, len, &r);
-	return (ok && (r == len));
-}
-
-static void about(HWND h) {
-	::MessageBox(h, L"Reads audio position information from Team Fortress 2 (Build 4097). IP:Port context support.", L"Mumble TF2 Plugin", MB_OK);
-}
-
 static bool calcout(float *pos, float *rot, float *opos, float *front, float *top) {
 	float h = rot[0];
 	float v = rot[1];
@@ -129,9 +66,9 @@ static bool calcout(float *pos, float *rot, float *opos, float *front, float *to
 	return true;
 }
 
-static int fetch(float *avatar_pos, float *avatar_front, float *avatar_top, float *camera_pos, float *camera_front, float *camera_top, string &context, wstring &identity) {
+static int fetch(float *avatar_pos, float *avatar_front, float *avatar_top, float *camera_pos, float *camera_front, float *camera_top, string &context, wstring &) {
 	for (int i=0;i<3;i++)
-		avatar_pos[i] = avatar_front[i] = avatar_top[i] = 0;
+		avatar_pos[i] = avatar_front[i] = avatar_top[i] = camera_pos[i] = camera_front[i] = camera_top[i] = 0.0f;
 
 	float ipos[3], rot[3];
 	bool ok;
@@ -157,7 +94,7 @@ static int fetch(float *avatar_pos, float *avatar_front, float *avatar_top, floa
 
 	new_context << "<context>"
 	<< "<game>tf2</game>"
-	<< "<hostport>" << chHostStr << "</hostport>"
+	<< "<hostport>" << sHost << "</hostport>"
 	<< "</context>";
 	context = new_context.str();
 
@@ -171,38 +108,27 @@ static int fetch(float *avatar_pos, float *avatar_front, float *avatar_top, floa
 	if (state == 0 || state == 1 || state == 3)
 		return true; // Deactivate plugin
 
+	ok = calcout(ipos, rot, avatar_pos, avatar_front, avatar_top);
 	if (ok) {
-		int res = calcout(ipos, rot, avatar_pos, avatar_front, avatar_top);
-		if (res) {
-			for (int i=0;i<3;++i) {
-				camera_pos[i] = avatar_pos[i];
-				camera_front[i] = avatar_front[i];
-				camera_top[i] = avatar_top[i];
-			}
-			return res;
+		for (int i=0;i<3;++i) {
+			camera_pos[i] = avatar_pos[i];
+			camera_front[i] = avatar_front[i];
+			camera_top[i] = avatar_top[i];
 		}
+		return true;
 	}
 
 	return false;
 }
 
-static int trylock() {
-
-	h = NULL;
+static int trylock(const std::multimap<std::wstring, unsigned long long int> &pids) {
 	posptr = rotptr = NULL;
 
-	DWORD pid=getProcess(L"hl2.exe");
-	if (!pid)
-		return false;
-	BYTE *mod=getModuleAddr(pid, L"client.dll");
-	if (!mod)
-		return false;
-	BYTE *mod_engine=getModuleAddr(pid, L"engine.dll");
-	if (!mod_engine)
+	if (! initialize(pids, L"hl2.exe", L"client.dll"))
 		return false;
 
-	h=OpenProcess(PROCESS_VM_READ, false, pid);
-	if (!h)
+	BYTE *mod_engine=getModuleAddr(L"engine.dll");
+	if (!mod_engine)
 		return false;
 
 	// Check if we really have TF2 running
@@ -217,21 +143,21 @@ static int trylock() {
 	*/
 
 	// Remember addresses for later
-	posptr = mod_engine + 0x555BB4;
-	rotptr = mod_engine + 0x3CDC30;
-	stateptr = mod + 0x4EABCC;
-	hostptr = mod_engine + 0x3C91A4;
+	posptr = pModule + 0x6EFCC0;
+	rotptr = pModule + 0x6F6544;
+	stateptr = pModule + 0x606C64;
+	hostptr = mod_engine + 0x3D3E94;
 
 	/*
 	// Gamecheck
 	char sMagic[9];
-	if (!peekProc(mod + 0x4FB3E3, sMagic, 9) || strncmp("teamJet@@", sMagic, 9)!=0)
+	if (!peekProc(pModule + 0x617DE3, sMagic, 9) || strncmp("teamJet@@", sMagic, 9)!=0)
 		return false;
 	*/
 
 	// Gamecheck
 	char sMagic[13];
-	if (!peekProc(mod_engine + 0x53FBE0, sMagic, 13) || strncmp("DemomanTaunts", sMagic, 13)!=0)
+	if (!peekProc(mod_engine + 0x54A670, sMagic, 13) || strncmp("DemomanTaunts", sMagic, 13)!=0)
 		return false;
 
 	// Check if we can get meaningful data from it
@@ -240,42 +166,47 @@ static int trylock() {
 	wstring sidentity;
 	string scontext;
 
-	if (fetch(apos, afront, atop, cpos, cfront, ctop, scontext, sidentity))
+	if (fetch(apos, afront, atop, cpos, cfront, ctop, scontext, sidentity)) {
 		return true;
-
-	// If it failed clean up
-	CloseHandle(h);
-	h = NULL;
-	return false;
-}
-
-static void unlock() {
-	if (h) {
-		CloseHandle(h);
-		h = NULL;
+	} else {
+		generic_unlock();
+		return false;
 	}
-	return;
 }
 
 static const std::wstring longdesc() {
-	return std::wstring(L"Supports TF2 build 4097. No identity support yet.");
+	return std::wstring(L"Supports TF2 build 4421. No identity support yet.");
 }
 
-static std::wstring description(L"Team Fortress 2 (Build 4097)");
+static std::wstring description(L"Team Fortress 2 (Build 4421)");
 static std::wstring shortname(L"Team Fortress 2");
 
+static int trylock1() {
+	return trylock(std::multimap<std::wstring, unsigned long long int>());
+}
+
 static MumblePlugin tf2plug = {
 	MUMBLE_PLUGIN_MAGIC,
 	description,
 	shortname,
-	about,
 	NULL,
-	trylock,
-	unlock,
+	NULL,
+	trylock1,
+	generic_unlock,
 	longdesc,
 	fetch
 };
 
+static MumblePlugin2 tf2plug2 = {
+	MUMBLE_PLUGIN_MAGIC_2,
+	MUMBLE_PLUGIN_VERSION,
+	trylock
+};
+
 extern "C" __declspec(dllexport) MumblePlugin *getMumblePlugin() {
 	return &tf2plug;
 }
+
+extern "C" __declspec(dllexport) MumblePlugin2 *getMumblePlugin2() {
+	return &tf2plug2;
+}
