@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -40,7 +40,7 @@ class ServerUser;
 class ChanACL : public QObject {
 	private:
 		Q_OBJECT
-		Q_DISABLE_COPY(ChanACL);
+		Q_DISABLE_COPY(ChanACL)
 	public:
 		enum Perm {
 			None = 0x0,
