@@ -121,6 +121,10 @@ int Connection::activityTime() const {
 	return qtLastPacket.elapsed();
 }
 
+void Connection::resetActivityTime() {
+	qtLastPacket.restart();
+}
+
 void Connection::socketRead() {
 	while (true) {
 		qint64 iAvailable = qtsSocket->bytesAvailable();
@@ -141,7 +145,6 @@ void Connection::socketRead() {
 
 		QByteArray qbaBuffer = qtsSocket->read(iPacketLength);
 		iPacketLength = -1;
-		qtLastPacket.restart();
 		iAvailable -= iPacketLength;
 
 		emit message(uiType, qbaBuffer);
@@ -149,10 +152,7 @@ void Connection::socketRead() {
 }
 
 void Connection::socketError(QAbstractSocket::SocketError err) {
-	if (! bDisconnectedEmitted) {
-		bDisconnectedEmitted = true;
-		emit connectionClosed(err, qtsSocket->errorString());
-	}
+	emit connectionClosed(err, qtsSocket->errorString());
 }
 
 void Connection::socketSslErrors(const QList<QSslError> &qlErr) {
@@ -164,10 +164,7 @@ void Connection::proceedAnyway() {
 }
 
 void Connection::socketDisconnected() {
-	if (! bDisconnectedEmitted) {
-		bDisconnectedEmitted = true;
-		emit connectionClosed(QAbstractSocket::UnknownSocketError, QString());
-	}
+	emit connectionClosed(QAbstractSocket::UnknownSocketError, QString());
 }
 
 void Connection::messageToNetwork(const ::google::protobuf::Message &msg, unsigned int msgType, QByteArray &cache) {
