@@ -56,7 +56,10 @@
 	MUMBLE_MH_MSG(UserList) \
 	MUMBLE_MH_MSG(VoiceTarget) \
 	MUMBLE_MH_MSG(PermissionQuery) \
-	MUMBLE_MH_MSG(CodecVersion)
+	MUMBLE_MH_MSG(CodecVersion) \
+	MUMBLE_MH_MSG(UserStats) \
+	MUMBLE_MH_MSG(RequestBlob) \
+	MUMBLE_MH_MSG(ServerConfig)
 
 class MessageHandler {
 	public:
@@ -82,6 +85,22 @@ inline ::std::string u8(const QString &str) {
 	return ::std::string(qba.constData(), qba.length());
 }
 
+inline QByteArray blob(const ::std::string &str) {
+	return QByteArray(str.data(), static_cast<int>(str.length()));
+}
+
+inline ::std::string blob(const QByteArray &str) {
+	return ::std::string(str.constData(), str.length());
+}
+
+inline QByteArray sha1(const QByteArray &blob) {
+	return QCryptographicHash::hash(blob, QCryptographicHash::Sha1);
+}
+
+inline QByteArray sha1(const QString &str) {
+	return QCryptographicHash::hash(str.toUtf8(), QCryptographicHash::Sha1);
+}
+
 #else
 class Message;
 #endif
