@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -31,15 +31,21 @@
 #include "OSInfo.h"
 #include "Version.h"
 
+#ifdef Q_WS_WIN
+#include <intrin.h>
+#endif
+
 #ifdef Q_WS_X11
 #include <sys/utsname.h>
 #endif
 
 #ifdef Q_WS_MAC
+#include <sys/types.h>
+#include <sys/sysctl.h>
 #include <mach-o/arch.h>
 #endif
 
-QString OSInfo::getMacHash(const QHostAddress &qhaBind) {
+QString OSInfo::getMacHash(const QList<QHostAddress> &qlBind) {
 	QString first, second, third;
 	foreach(const QNetworkInterface &qni, QNetworkInterface::allInterfaces()) {
 		if (! qni.isValid())
@@ -62,7 +68,7 @@ QString OSInfo::getMacHash(const QHostAddress &qhaBind) {
 
 		foreach(const QNetworkAddressEntry &qnae, qni.addressEntries()) {
 			const QHostAddress &qha = qnae.ip();
-			if (((qhaBind == QHostAddress::Any) && (qha.protocol() ==  QAbstractSocket::IPv4Protocol)) || (qha == qhaBind)) {
+			if (qlBind.isEmpty() || qlBind.contains(qha)) {
 				if (first.isEmpty() || first > hash)
 					first = hash;
 			}
@@ -88,6 +94,13 @@ QString OSInfo::getOS() {
 }
 
 QString OSInfo::getOSVersion() {
+	static QString qsCached;
+
+	if (! qsCached.isNull())
+		return qsCached.isEmpty() ? QString() : qsCached;
+
+	QString os;
+
 #if defined(Q_WS_WIN)
 	OSVERSIONINFOEXW ovi;
 	memset(&ovi, 0, sizeof(ovi));
@@ -95,9 +108,7 @@ QString OSInfo::getOSVersion() {
 	ovi.dwOSVersionInfoSize=sizeof(ovi);
 	GetVersionEx(reinterpret_cast<OSVERSIONINFOW *>(&ovi));
 
-	QString os;
 	os.sprintf("%d.%d.%d.%d", ovi.dwMajorVersion, ovi.dwMinorVersion, ovi.dwBuildNumber, (ovi.wProductType == VER_NT_WORKSTATION) ? 1 : 0);
-	return os;
 #elif defined(Q_OS_MAC)
 	SInt32 major, minor, bugfix;
 	OSErr err = Gestalt(gestaltSystemVersionMajor, &major);
@@ -112,9 +123,7 @@ QString OSInfo::getOSVersion() {
 	const NXArchInfo *ai = local ? NXGetArchInfoFromCpuType(local->cputype, CPU_SUBTYPE_MULTIPLE) : NULL;
 	const char *arch = ai ? ai->name : "unknown";
 
-	QString os;
 	os.sprintf("%i.%i.%i (%s)", major, minor, bugfix, arch);
-	return os;
 #else
 #ifdef Q_OS_LINUX
 	QProcess qp;
@@ -123,35 +132,43 @@ QString OSInfo::getOSVersion() {
 	args << QLatin1String("-d");
 	qp.start(QLatin1String("lsb_release"), args);
 	if (qp.waitForFinished(5000)) {
-		QString os = QString::fromUtf8(qp.readAll()).simplified();
+		os = QString::fromUtf8(qp.readAll()).simplified();
 		if (os.startsWith(QLatin1Char('"')) && os.endsWith(QLatin1Char('"')))
 			os = os.mid(1, os.length() - 2).trimmed();
-		if (! os.isEmpty())
-			return os;
 	}
-	qWarning("OSInfo: Failed to execute lsb_release");
+	if (os.isEmpty())
+		qWarning("OSInfo: Failed to execute lsb_release");
+
 	qp.terminate();
 	if (! qp.waitForFinished(1000))
 		qp.kill();
 #endif
-	struct utsname un;
-	if (uname(&un) == 0) {
-		QString os;
-		os.sprintf("%s %s", un.sysname, un.release);
-		return os;
+	if (os.isEmpty()) {
+		struct utsname un;
+		if (uname(&un) == 0) {
+			os.sprintf("%s %s", un.sysname, un.release);
+		}
 	}
-	return QString();
 #endif
+
+	if (! os.isNull())
+		qsCached = os;
+	else
+		qsCached = QLatin1String("");
+
+	return qsCached;
 }
 
-void OSInfo::fillXml(QDomDocument &doc, QDomElement &root, const QString &os, const QString &osver, const QHostAddress &qhaBind) {
+void OSInfo::fillXml(QDomDocument &doc, QDomElement &root, const QString &os, const QString &osver, const QList<QHostAddress> &qlBind) {
 	QDomElement tag;
 	QDomText t;
 	bool bIs64;
+	bool bSSE2 = false;
+	QString cpu_id, cpu_extid;
 
 	tag=doc.createElement(QLatin1String("machash"));
 	root.appendChild(tag);
-	t=doc.createTextNode(OSInfo::getMacHash(qhaBind));
+	t=doc.createTextNode(OSInfo::getMacHash(qlBind));
 	tag.appendChild(t);
 
 	tag=doc.createElement(QLatin1String("version"));
@@ -183,6 +200,9 @@ void OSInfo::fillXml(QDomDocument &doc, QDomElement &root, const QString &os, co
 	BOOL bIsWow64 = FALSE;
 	IsWow64Process(GetCurrentProcess(), &bIsWow64);
 	bIs64 = bIsWow64;
+#elif defined(Q_WS_MAC)
+	size_t len = sizeof(bool);
+	sysctlbyname("hw.cpu64bit_capable", &bIs64, &len, NULL, 0);
 #else
 	bIs64 = (QSysInfo::WordSize == 64);
 #endif
@@ -191,4 +211,46 @@ void OSInfo::fillXml(QDomDocument &doc, QDomElement &root, const QString &os, co
 	t=doc.createTextNode(QString::number(bIs64 ? 1 : 0));
 	tag.appendChild(t);
 
+#if defined(Q_WS_WIN)
+#define regstr(x) QString::fromLatin1(reinterpret_cast<const char *>(& x), 4)
+	int chop;
+	int cpuinfo[4];
+
+	__cpuid(cpuinfo, 1);
+	bSSE2 = (cpuinfo[3] & 0x04000000);
+
+	__cpuid(cpuinfo, 0);
+
+	cpu_id = regstr(cpuinfo[1]) + regstr(cpuinfo[3]) + regstr(cpuinfo[2]);
+
+	for (unsigned int j=2; j<=4;++j) {
+		__cpuid(cpuinfo, 0x80000000 + j);
+		cpu_extid += regstr(cpuinfo[0]) + regstr(cpuinfo[1]) + regstr(cpuinfo[2]) + regstr(cpuinfo[3]);
+	}
+
+	cpu_id = cpu_id.trimmed();
+	chop = cpu_id.indexOf(QLatin1Char('\0'));
+	if (chop != -1)
+		cpu_id.truncate(chop);
+
+	cpu_extid = cpu_extid.trimmed();
+	chop = cpu_extid.indexOf(QLatin1Char('\0'));
+	if (chop != -1)
+		cpu_extid.truncate(chop);
+#endif
+
+	tag=doc.createElement(QLatin1String("cpu_id"));
+	root.appendChild(tag);
+	t=doc.createTextNode(cpu_id);
+	tag.appendChild(t);
+
+	tag=doc.createElement(QLatin1String("cpu_extid"));
+	root.appendChild(tag);
+	t=doc.createTextNode(cpu_extid);
+	tag.appendChild(t);
+
+	tag=doc.createElement(QLatin1String("cpu_sse2"));
+	root.appendChild(tag);
+	t=doc.createTextNode(QString::number(bSSE2 ? 1 : 0));
+	tag.appendChild(t);
 }
