@@ -1,4 +1,5 @@
 /* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
+   Copyright (C) 2010, Stefan Hacker <dd0t@users.sourceforge.net>
 
    All rights reserved.
 
@@ -31,11 +32,13 @@
 #ifndef _VERSION_H
 #define _VERSION_H
 
+#include "murmur_pch.h"
+
 #define MUMXTEXT(X) #X
 #define MUMTEXT(X) MUMXTEXT(X)
 
 #ifndef MUMBLE_VERSION
-#define MUMBLE_VERSION 1.2.2~201002071401-4ab478
+#define MUMBLE_VERSION 1.2.3
 #endif
 #ifndef MUMBLE_VERSION
 #define MUMBLE_RELEASE "Compiled " __DATE__ " " __TIME__
@@ -43,4 +46,13 @@
 #define MUMBLE_RELEASE MUMTEXT(MUMBLE_VERSION)
 #endif
 
+class MumbleVersion {
+	public:
+		static unsigned int getRaw(const QString &version = QLatin1String(MUMTEXT(MUMBLE_VERSION_STRING)));
+		static bool get(int *major, int *minor, int *patch, const QString &version = QLatin1String(MUMTEXT(MUMBLE_VERSION_STRING)));
+
+		static unsigned int toRaw(int major, int minor, int patch);
+		static void fromRaw(unsigned int version, int *major, int *minor, int *patch);
+};
+
 #endif
