@@ -88,7 +88,8 @@ ACLEditor::ACLEditor(int channelid, const MumbleProto::ACL &mea, QWidget *p) : Q
 	setWindowTitle(tr("Mumble - Edit %1").arg(Channel::get(iId)->qsName));
 
 	qleChannelName->setText(pChannel->qsName);
-	if (channelid == 0) qleChannelName->setEnabled(false);
+	if (channelid == 0)
+		qleChannelName->setEnabled(false);
 
 	rteChannelDescription->setText(pChannel->qsDesc);
 
@@ -97,34 +98,33 @@ ACLEditor::ACLEditor(int channelid, const MumbleProto::ACL &mea, QWidget *p) : Q
 
 	QGridLayout *grid = new QGridLayout(qgbACLpermissions);
 
-	l=new QLabel(tr("Deny"), qgbACLpermissions);
-	grid->addWidget(l,0,1);
-	l=new QLabel(tr("Allow"), qgbACLpermissions);
-	grid->addWidget(l,0,2);
+	l = new QLabel(tr("Deny"), qgbACLpermissions);
+	grid->addWidget(l, 0, 1);
+	l = new QLabel(tr("Allow"), qgbACLpermissions);
+	grid->addWidget(l, 0, 2);
 
-	int idx=1;
-	for (int i=0;i< ((iId == 0) ? 30 : 16);++i) {
+	int idx = 1;
+	for (int i = 0; i < ((iId == 0) ? 30 : 16); ++i) {
 		ChanACL::Perm perm = static_cast<ChanACL::Perm>(1 << i);
 		QString name = ChanACL::permName(perm);
 
 		if (! name.isEmpty()) {
 			QCheckBox *qcb;
 			l = new QLabel(name, qgbACLpermissions);
-			grid->addWidget(l,idx,0);
-			qcb=new QCheckBox(qgbACLpermissions);
+			grid->addWidget(l, idx, 0);
+			qcb = new QCheckBox(qgbACLpermissions);
 			qcb->setToolTip(tr("Deny %1").arg(name));
 			qcb->setWhatsThis(tr("This revokes the %1 privilege. If a privilege is both allowed and denied, it is denied.<br />%2").arg(name).arg(ChanACL::whatsThis(perm)));
 			connect(qcb, SIGNAL(clicked(bool)), this, SLOT(ACLPermissions_clicked()));
-			grid->addWidget(qcb,idx,1);
+			grid->addWidget(qcb, idx, 1);
 
 			qlACLDeny << qcb;
 
-
-			qcb=new QCheckBox(qgbACLpermissions);
+			qcb = new QCheckBox(qgbACLpermissions);
 			qcb->setToolTip(tr("Allow %1").arg(name));
 			qcb->setWhatsThis(tr("This grants the %1 privilege. If a privilege is both allowed and denied, it is denied.<br />%2").arg(name).arg(ChanACL::whatsThis(perm)));
 			connect(qcb, SIGNAL(clicked(bool)), this, SLOT(ACLPermissions_clicked()));
-			grid->addWidget(qcb,idx,2);
+			grid->addWidget(qcb, idx, 2);
 
 			qlACLAllow << qcb;
 
@@ -133,7 +133,7 @@ ACLEditor::ACLEditor(int channelid, const MumbleProto::ACL &mea, QWidget *p) : Q
 			++idx;
 		}
 	}
-	QSpacerItem *si = new QSpacerItem(0,0, QSizePolicy::Minimum, QSizePolicy::Expanding);
+	QSpacerItem *si = new QSpacerItem(0, 0, QSizePolicy::Minimum, QSizePolicy::Expanding);
 	grid->addItem(si, idx, 0);
 
 	connect(qcbGroupAdd->lineEdit(), SIGNAL(returnPressed()), qpbGroupAddAdd, SLOT(animateClick()));
@@ -158,7 +158,7 @@ ACLEditor::ACLEditor(int channelid, const MumbleProto::ACL &mea, QWidget *p) : Q
 
 	qlACLs << def;
 
-	for (int i=0;i<mea.acls_size();++i) {
+	for (int i = 0; i < mea.acls_size(); ++i) {
 		const MumbleProto::ACL_ChanACL &as = mea.acls(i);
 
 		ChanACL *acl = new ChanACL(NULL);
@@ -176,15 +176,18 @@ ACLEditor::ACLEditor(int channelid, const MumbleProto::ACL &mea, QWidget *p) : Q
 		qlACLs << acl;
 	}
 
-	for (int i=0;i<mea.groups_size();++i) {
+	for (int i = 0; i < mea.groups_size(); ++i) {
 		const MumbleProto::ACL_ChanGroup &gs = mea.groups(i);
 
 		ACLGroup *gp = new ACLGroup(u8(gs.name()));
-		for (int j=0;j<gs.add_size();++j)
+		gp->bInherit = gs.inherit();
+		gp->bInherited = gs.inherited();
+		gp->bInheritable = gs.inheritable();
+		for (int j = 0; j < gs.add_size(); ++j)
 			gp->qsAdd.insert(gs.add(j));
-		for (int j=0;j<gs.remove_size();++j)
+		for (int j = 0; j < gs.remove_size(); ++j)
 			gp->qsRemove.insert(gs.remove(j));
-		for (int j=0;j<gs.inherited_members_size();++j)
+		for (int j = 0; j < gs.inherited_members_size(); ++j)
 			gp->qsTemporary.insert(gs.inherited_members(j));
 
 		qlGroups << gp;
@@ -290,6 +293,8 @@ void ACLEditor::accept() {
 				continue;
 			MumbleProto::ACL_ChanGroup *mpg = msg.add_groups();
 			mpg->set_name(u8(gp->qsName));
+			mpg->set_inherit(gp->bInherit);
+			mpg->set_inheritable(gp->bInheritable);
 			foreach(int pid, gp->qsAdd)
 				if (pid >= 0)
 					mpg->add_add(pid);
@@ -312,9 +317,9 @@ const QString ACLEditor::userName(int pid) {
 
 int ACLEditor::id(const QString &uname) {
 	QString name = uname.toLower();
-	if (qhIDCache.contains(name))
+	if (qhIDCache.contains(name)) {
 		return qhIDCache.value(name);
-	else {
+	} else {
 		if (! qhNameWait.contains(name)) {
 			MumbleProto::QueryUsers mpuq;
 			mpuq.add_names(u8(name));
@@ -332,7 +337,7 @@ void ACLEditor::returnQuery(const MumbleProto::QueryUsers &mqu) {
 	if (mqu.names_size() != mqu.ids_size())
 		return;
 
-	for (int i=0;i < mqu.names_size(); ++i) {
+	for (int i = 0; i < mqu.names_size(); ++i) {
 		int pid = mqu.ids(i);
 		QString name = u8(mqu.names(i));
 		QString lname = name.toLower();
@@ -396,7 +401,7 @@ void ACLEditor::refillComboBoxes() {
 
 void ACLEditor::refillACL() {
 	int idx = qlwACLs->currentRow();
-	bool previnh = bInheritACL;
+	bool previousinherit = bInheritACL;
 	bInheritACL = qcbACLInherit->isChecked();
 
 	qlwACLs->clear();
@@ -410,19 +415,19 @@ void ACLEditor::refillACL() {
 			continue;
 		QString text;
 		if (acl->iUserId == -1)
-			text=QString::fromLatin1("@%1").arg(acl->qsGroup);
+			text = QString::fromLatin1("@%1").arg(acl->qsGroup);
 		else
-			text=userName(acl->iUserId);
-		QListWidgetItem *item=new QListWidgetItem(text, qlwACLs);
+			text = userName(acl->iUserId);
+		QListWidgetItem *item = new QListWidgetItem(text, qlwACLs);
 		if (acl->bInherited) {
 			QFont f = item->font();
 			f.setItalic(true);
 			item->setFont(f);
 		}
 	}
-	if (bInheritACL && ! previnh && (idx != 0))
+	if (bInheritACL && ! previousinherit && (idx != 0))
 		idx += numInheritACL;
-	if (! bInheritACL && previnh)
+	if (! bInheritACL && previousinherit)
 		idx -= numInheritACL;
 
 	qlwACLs->setCurrentRow(idx);
@@ -443,18 +448,22 @@ void ACLEditor::refillGroupNames() {
 		qcbGroupList->addItem(name);
 	}
 
-	int wantindex = qcbGroupList->findText(text, Qt::MatchExactly);
+	int wantindex = qcbGroupList->findText(text, Qt::MatchFixedString);
 	qcbGroupList->setCurrentIndex(wantindex);
 }
 
 ACLGroup *ACLEditor::currentGroup() {
-	QString group = qcbGroupList->currentText().toLower();
+	QString group = qcbGroupList->currentText();
 
-	foreach(ACLGroup *gp, qlGroups) {
-		if (gp->qsName == group) {
+	foreach(ACLGroup *gp, qlGroups)
+		if (gp->qsName == group)
+			return gp;
+
+	group = group.toLower();
+
+	foreach(ACLGroup *gp, qlGroups)
+		if (gp->qsName == group)
 			return gp;
-		}
-	}
 
 	return NULL;
 }
@@ -517,27 +526,26 @@ void ACLEditor::refillGroupInherit() {
 void ACLEditor::groupEnableCheck() {
 	ACLGroup *gp = currentGroup();
 
-	bool ena = true;
-
+	bool enabled;
 	if (! gp)
-		ena = false;
+		enabled = false;
 	else
-		ena = gp->bInherit;
+		enabled = gp->bInherit;
 
-	qlwGroupRemove->setEnabled(ena);
-	qlwGroupInherit->setEnabled(ena);
-	qcbGroupRemove->setEnabled(ena);
-	qpbGroupRemoveAdd->setEnabled(ena);
-	qpbGroupRemoveRemove->setEnabled(ena);
-	qpbGroupInheritRemove->setEnabled(ena);
+	qlwGroupRemove->setEnabled(enabled);
+	qlwGroupInherit->setEnabled(enabled);
+	qcbGroupRemove->setEnabled(enabled);
+	qpbGroupRemoveAdd->setEnabled(enabled);
+	qpbGroupRemoveRemove->setEnabled(enabled);
+	qpbGroupInheritRemove->setEnabled(enabled);
 
-	ena = (gp != NULL);
-	qlwGroupAdd->setEnabled(ena);
-	qcbGroupAdd->setEnabled(ena);
-	qpbGroupAddAdd->setEnabled(ena);
-	qpbGroupAddRemove->setEnabled(ena);
-	qcbGroupInherit->setEnabled(ena);
-	qcbGroupInheritable->setEnabled(ena);
+	enabled = (gp != NULL);
+	qlwGroupAdd->setEnabled(enabled);
+	qcbGroupAdd->setEnabled(enabled);
+	qpbGroupAddAdd->setEnabled(enabled);
+	qpbGroupAddRemove->setEnabled(enabled);
+	qcbGroupInherit->setEnabled(enabled);
+	qcbGroupInheritable->setEnabled(enabled);
 
 	if (gp) {
 		qcbGroupInherit->setChecked(gp->bInherit);
@@ -549,36 +557,37 @@ void ACLEditor::groupEnableCheck() {
 void ACLEditor::ACLEnableCheck() {
 	ChanACL *as = currentACL();
 
-	bool ena = true;
+	bool enabled;
 	if (! as)
-		ena = false;
+		enabled = false;
 	else
-		ena = ! as->bInherited;
+		enabled = ! as->bInherited;
 
-	qpbACLRemove->setEnabled(ena);
-	qpbACLUp->setEnabled(ena);
-	qpbACLDown->setEnabled(ena);
-	qcbACLApplyHere->setEnabled(ena);
-	qcbACLApplySubs->setEnabled(ena);
-	qcbACLGroup->setEnabled(ena);
-	qcbACLUser->setEnabled(ena);
+	qpbACLRemove->setEnabled(enabled);
+	qpbACLUp->setEnabled(enabled);
+	qpbACLDown->setEnabled(enabled);
+	qcbACLApplyHere->setEnabled(enabled);
+	qcbACLApplySubs->setEnabled(enabled);
+	qcbACLGroup->setEnabled(enabled);
+	qcbACLUser->setEnabled(enabled);
 
-	int idx;
-	for (idx=0;idx<qlACLAllow.count();idx++) {
+	for (int idx = 0; idx < qlACLAllow.count(); idx++) {
 		// Only enable other checkboxes if writeacl isn't set
-		bool enathis = ena && (qlPerms[idx] == ChanACL::Write || !(as && static_cast<int>(as->pAllow) & ChanACL::Write) || qlPerms[idx] == ChanACL::Speak);
-		qlACLAllow[idx]->setEnabled(enathis);
-		qlACLDeny[idx]->setEnabled(enathis);
+		bool enablethis = enabled && (qlPerms[idx] == ChanACL::Write || !(as && (as->pAllow & ChanACL::Write)) || qlPerms[idx] == ChanACL::Speak);
+		qlACLAllow[idx]->setEnabled(enablethis);
+		qlACLDeny[idx]->setEnabled(enablethis);
 	}
 
 	if (as) {
 		qcbACLApplyHere->setChecked(as->bApplyHere);
 		qcbACLApplySubs->setChecked(as->bApplySubs);
-		for (idx=0;idx<qlACLAllow.count();idx++) {
+
+		for (int idx = 0; idx < qlACLAllow.count(); idx++) {
 			ChanACL::Perm p = qlPerms[idx];
-			qlACLAllow[idx]->setChecked(static_cast<int>(as->pAllow) & p);
-			qlACLDeny[idx]->setChecked(static_cast<int>(as->pDeny) & p);
+			qlACLAllow[idx]->setChecked(as->pAllow & p);
+			qlACLDeny[idx]->setChecked(as->pDeny & p);
 		}
+
 		qcbACLGroup->clear();
 		qcbACLGroup->addItem(QString());
 		qcbACLGroup->addItem(QLatin1String("all"));
@@ -589,8 +598,10 @@ void ACLEditor::ACLEnableCheck() {
 		qcbACLGroup->addItem(QLatin1String("~in"));
 		qcbACLGroup->addItem(QLatin1String("~sub"));
 		qcbACLGroup->addItem(QLatin1String("~out"));
+
 		foreach(ACLGroup *gs, qlGroups)
 			qcbACLGroup->addItem(gs->qsName);
+
 		if (as->iUserId == -1) {
 			qcbACLUser->clearEditText();
 			qcbACLGroup->addItem(as->qsGroup);
@@ -609,9 +620,10 @@ void ACLEditor::ACLEnableCheck() {
 }
 
 void ACLEditor::on_qtwTab_currentChanged(int index) {
-	if (index == 0)	// Switched to property tab, update password field
+	if (index == 0) {
+		// Switched to property tab, update password field
 		updatePasswordField();
-	else if (index == 2) {
+	} else if (index == 2) {
 		// Switched to ACL tab, update ACL list
 		updatePasswordACL();
 		refillACL();
@@ -620,25 +632,24 @@ void ACLEditor::on_qtwTab_currentChanged(int index) {
 
 void ACLEditor::updatePasswordField() {
 	pcaPassword = NULL;
-	QList<ChanACL *>::iterator i = qlACLs.end();
-	while (i != qlACLs.begin()) {
-		i--;
-
+	foreach(ChanACL *acl, qlACLs) {
 		// Check for sth that applies to '#<something>' AND grants 'Enter' AND may grant 'Speak', 'Whisper',
 		// 'TextMessage', 'Link' but NOTHING else AND does not deny anything, then '<something>' is the password.
-		if ((*i)->qsGroup.startsWith(QLatin1Char('#')) &&
-		        (*i)->bApplyHere &&
-		        !(*i)->bInherited &&
-		        ((*i)->pAllow & ChanACL::Enter) &&
-		        0 == ((*i)->pAllow & (~(ChanACL::Enter | ChanACL::Speak | ChanACL::Whisper | ChanACL::TextMessage | ChanACL::LinkChannel))) &&
-		        (*i)->pDeny == ChanACL::None) {
-			qleChannelPassword->setText((*i)->qsGroup.mid(1));
-			pcaPassword = (*i);
-			break;
+		if (acl->qsGroup.startsWith(QLatin1Char('#')) &&
+		        acl->bApplyHere &&
+		        !acl->bInherited &&
+		        (acl->pAllow & ChanACL::Enter) &&
+		        (acl->pAllow == (ChanACL::Enter | ChanACL::Speak | ChanACL::Whisper | ChanACL::TextMessage | ChanACL::LinkChannel) || // Backwards compat with old behaviour that didn't deny traverse
+		         acl->pAllow == (ChanACL::Enter | ChanACL::Speak | ChanACL::Whisper | ChanACL::TextMessage | ChanACL::LinkChannel | ChanACL::Traverse)) &&
+		        acl->pDeny == ChanACL::None) {
+			pcaPassword = acl;
 		}
 	}
-	if (!pcaPassword)
+	if (pcaPassword)
+		qleChannelPassword->setText(pcaPassword->qsGroup.mid(1));
+	else
 		qleChannelPassword->clear();
+
 }
 
 void ACLEditor::updatePasswordACL() {
@@ -646,38 +657,41 @@ void ACLEditor::updatePasswordACL() {
 		// Remove the password if we had one to begin with
 		if (pcaPassword && qlACLs.removeOne(pcaPassword)) {
 			delete pcaPassword;
-			QList<ChanACL *>::iterator i = qlACLs.end();
-			while (i != qlACLs.begin()) {
-				i--;
-				if ((*i)->qsGroup == QLatin1String("all") &&
-				        (*i)->bInherited == false &&
-				        (*i)->bApplyHere == true &&
-				        (*i)->bApplySubs == true &&
-				        (*i)->pAllow == ChanACL::None &&
-				        (*i)->pDeny == (ChanACL::Enter | ChanACL::Speak | ChanACL::Whisper | ChanACL::TextMessage | ChanACL::LinkChannel)) {
-					qlACLs.removeOne(*i);
-					delete(*i);
-					break;
+
+			// Search and remove the @all deny ACL
+			ChanACL *denyall = NULL;
+			foreach(ChanACL *acl, qlACLs) {
+				if (acl->qsGroup == QLatin1String("all") &&
+				        acl->bInherited == false &&
+				        acl->bApplyHere == true &&
+				        acl->pAllow == ChanACL::None &&
+				        (acl->pDeny == (ChanACL::Enter | ChanACL::Speak | ChanACL::Whisper | ChanACL::TextMessage | ChanACL::LinkChannel) || // Backwards compat with old behaviour that didn't deny traverse
+				         acl->pDeny == (ChanACL::Enter | ChanACL::Speak | ChanACL::Whisper | ChanACL::TextMessage | ChanACL::LinkChannel | ChanACL::Traverse))) {
+					denyall = acl;
 				}
 			}
+			if (denyall) {
+				qlACLs.removeOne(denyall);
+				delete denyall;
+			}
 		}
 	} else {
 		// Add or Update
 		if (pcaPassword == NULL || !qlACLs.contains(pcaPassword)) {
 			pcaPassword = new ChanACL(NULL);
 			pcaPassword->bApplyHere = true;
-			pcaPassword->bApplySubs = true;
+			pcaPassword->bApplySubs = false;
 			pcaPassword->bInherited = false;
 			pcaPassword->pAllow = ChanACL::None;
-			pcaPassword->pDeny = ChanACL::Enter | ChanACL::Speak | ChanACL::Whisper | ChanACL::TextMessage | ChanACL::LinkChannel;
+			pcaPassword->pDeny = ChanACL::Enter | ChanACL::Speak | ChanACL::Whisper | ChanACL::TextMessage | ChanACL::LinkChannel | ChanACL::Traverse;
 			pcaPassword->qsGroup = QLatin1String("all");
 			qlACLs << pcaPassword;
 
 			pcaPassword = new ChanACL(NULL);
 			pcaPassword->bApplyHere = true;
-			pcaPassword->bApplySubs = true;
+			pcaPassword->bApplySubs = false;
 			pcaPassword->bInherited = false;
-			pcaPassword->pAllow = ChanACL::Enter | ChanACL::Speak | ChanACL::Whisper | ChanACL::TextMessage | ChanACL::LinkChannel;
+			pcaPassword->pAllow = ChanACL::Enter | ChanACL::Speak | ChanACL::Whisper | ChanACL::TextMessage | ChanACL::LinkChannel | ChanACL::Traverse;
 			pcaPassword->pDeny = ChanACL::None;
 			pcaPassword->qsGroup = QString(QLatin1String("#%1")).arg(qleChannelPassword->text());
 			qlACLs << pcaPassword;
@@ -709,6 +723,7 @@ void ACLEditor::on_qpbACLRemove_clicked() {
 	ChanACL *as = currentACL();
 	if (! as || as->bInherited)
 		return;
+
 	qlACLs.removeAll(as);
 	delete as;
 	refillACL();
@@ -722,6 +737,7 @@ void ACLEditor::on_qpbACLUp_clicked() {
 	int idx = qlACLs.indexOf(as);
 	if (idx <= numInheritACL + 1)
 		return;
+
 	qlACLs.swap(idx - 1, idx);
 	qlwACLs->setCurrentRow(qlwACLs->currentRow() - 1);
 	refillACL();
@@ -735,6 +751,7 @@ void ACLEditor::on_qpbACLDown_clicked() {
 	int idx = qlACLs.indexOf(as) + 1;
 	if (idx >= qlACLs.count())
 		return;
+
 	qlACLs.swap(idx - 1, idx);
 	qlwACLs->setCurrentRow(qlwACLs->currentRow() + 1);
 	refillACL();
@@ -769,10 +786,10 @@ void ACLEditor::on_qcbACLGroup_activated(const QString &text) {
 
 	if (text.isEmpty()) {
 		qcbACLGroup->setCurrentIndex(1);
-		as->qsGroup=QLatin1String("all");
+		as->qsGroup = QLatin1String("all");
 	} else {
 		qcbACLUser->clearEditText();
-		as->qsGroup=text;
+		as->qsGroup = text;
 	}
 	refillACL();
 }
@@ -788,7 +805,7 @@ void ACLEditor::on_qcbACLUser_activated() {
 		as->iUserId = -1;
 		if (qcbACLGroup->currentIndex() == 0) {
 			qcbACLGroup->setCurrentIndex(1);
-			as->qsGroup=QLatin1String("all");
+			as->qsGroup = QLatin1String("all");
 		}
 		refillACL();
 	} else {
@@ -805,12 +822,11 @@ void ACLEditor::ACLPermissions_clicked() {
 	if (! as || as->bInherited)
 		return;
 
-	int a, d, idx;
-	a = 0;
-	d = 0;
+	int allowed = 0;
+	int denied = 0;
 
-	bool ena = true;
-	for (idx=0;idx<qlACLAllow.count();idx++) {
+	bool enabled = true;
+	for (int idx = 0; idx < qlACLAllow.count(); idx++) {
 		ChanACL::Perm p = qlPerms[idx];
 		if (qlACLAllow[idx]->isChecked() && qlACLDeny[idx]->isChecked()) {
 			if (source == qlACLAllow[idx])
@@ -819,20 +835,20 @@ void ACLEditor::ACLPermissions_clicked() {
 				qlACLAllow[idx]->setChecked(false);
 		}
 
-		qlACLAllow[idx]->setEnabled(ena || p == ChanACL::Speak);
-		qlACLDeny[idx]->setEnabled(ena || p == ChanACL::Speak);
+		qlACLAllow[idx]->setEnabled(enabled || p == ChanACL::Speak);
+		qlACLDeny[idx]->setEnabled(enabled || p == ChanACL::Speak);
 
 		if (p == ChanACL::Write && qlACLAllow[idx]->isChecked())
-			ena = false;
+			enabled = false;
 
 		if (qlACLAllow[idx]->isChecked())
-			a |= p;
+			allowed |= p;
 		if (qlACLDeny[idx]->isChecked())
-			d |= p;
+			denied |= p;
 	}
 
-	as->pAllow=static_cast<ChanACL::Permissions>(a);
-	as->pDeny=static_cast<ChanACL::Permissions>(d);
+	as->pAllow = static_cast<ChanACL::Permissions>(allowed);
+	as->pDeny = static_cast<ChanACL::Permissions>(denied);
 }
 
 void ACLEditor::on_qcbGroupList_activated(const QString &text) {
@@ -869,6 +885,7 @@ void ACLEditor::on_qpbGroupRemove_clicked() {
 	ACLGroup *gs = currentGroup();
 	if (! gs)
 		return;
+
 	if (gs->bInherited) {
 		gs->bInheritable = true;
 		gs->bInherit = true;
@@ -889,6 +906,7 @@ void ACLEditor::on_qcbGroupInherit_clicked(bool checked) {
 	ACLGroup *gs = currentGroup();
 	if (! gs)
 		return;
+
 	gs->bInherit = checked;
 	groupEnableCheck();
 }
@@ -897,6 +915,7 @@ void ACLEditor::on_qcbGroupInheritable_clicked(bool checked) {
 	ACLGroup *gs = currentGroup();
 	if (! gs)
 		return;
+
 	gs->bInheritable = checked;
 }
 
@@ -920,7 +939,7 @@ void ACLEditor::on_qpbGroupAddRemove_clicked() {
 	if (! gs)
 		return;
 
-	QListWidgetItem *item= qlwGroupAdd->currentItem();
+	QListWidgetItem *item = qlwGroupAdd->currentItem();
 	if (! item)
 		return;
 
@@ -948,7 +967,7 @@ void ACLEditor::on_qpbGroupRemoveRemove_clicked() {
 	if (! gs)
 		return;
 
-	QListWidgetItem *item= qlwGroupRemove->currentItem();
+	QListWidgetItem *item = qlwGroupRemove->currentItem();
 	if (! item)
 		return;
 
@@ -961,7 +980,7 @@ void ACLEditor::on_qpbGroupInheritRemove_clicked() {
 	if (! gs)
 		return;
 
-	QListWidgetItem *item= qlwGroupInherit->currentItem();
+	QListWidgetItem *item = qlwGroupInherit->currentItem();
 	if (! item)
 		return;
 
