@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -75,7 +75,6 @@ class ASIOInit : public DeferInit {
 void ASIOInit::initialize() {
 	HKEY hkDevs;
 	HKEY hk;
-	DWORD idx = 0;
 	WCHAR keyname[255];
 	DWORD keynamelen = 255;
 	FILETIME ft;
@@ -94,6 +93,7 @@ void ASIOInit::initialize() {
 	blacklist << QLatin1String("{232685c6-6548-49d8-846d-4141a3ef7560}"); // ASIO4ALL
 #endif
 	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, L"Software\\ASIO", 0, KEY_READ, &hkDevs) == ERROR_SUCCESS) {
+		DWORD idx = 0;
 		while (RegEnumKeyEx(hkDevs, idx++, keyname, &keynamelen, NULL, NULL, NULL, &ft)  == ERROR_SUCCESS) {
 			QString name=QString::fromUtf16(reinterpret_cast<ushort *>(keyname),keynamelen);
 			if (RegOpenKeyEx(hkDevs, keyname, 0, KEY_READ, &hk) == ERROR_SUCCESS) {
@@ -125,10 +125,8 @@ void ASIOInit::initialize() {
 }
 
 void ASIOInit::destroy() {
-	if (airASIO)
-		delete airASIO;
-	if (crASIO)
-		delete crASIO;
+	delete airASIO;
+	delete crASIO;
 }
 
 static class ASIOInit asioinit;
@@ -221,10 +219,10 @@ void ASIOConfig::on_qpbQuery_clicked() {
 			iasio->getBufferSize(&minSize, &maxSize, &prefSize, &granSize);
 			SleepEx(10, false);
 
-			QString str = tr("%1 (ver %2)").arg(QLatin1String(buff)).arg(ver);
+			QString str = tr("%1 (version %2)").arg(QLatin1String(buff)).arg(ver);
 			qlName->setText(str);
 
-			str = tr("%1 -> %2 samples buffer, with %3 sample resolution (%4 preferred) at %5Hz").arg(minSize).arg(maxSize).arg(granSize).arg(prefSize).arg(srate,0,'f',0);
+			str = tr("%1 -> %2 samples buffer, with %3 sample resolution (%4 preferred) at %5 Hz").arg(minSize).arg(maxSize).arg(granSize).arg(prefSize).arg(srate,0,'f',0);
 
 			qlBuffers->setText(str);
 
@@ -244,6 +242,7 @@ void ASIOConfig::on_qpbQuery_clicked() {
 				switch (aci.type) {
 					case ASIOSTFloat32LSB:
 					case ASIOSTInt32LSB:
+					case ASIOSTInt24LSB:
 					case ASIOSTInt16LSB: {
 							QListWidget *widget = qlwUnused;
 							QVariant v = static_cast<int>(cnum);
@@ -266,11 +265,11 @@ void ASIOConfig::on_qpbQuery_clicked() {
 			char err[255];
 			iasio->getErrorMessage(err);
 			SleepEx(10, false);
-			QMessageBox::critical(this, tr("Mumble"), tr("ASIO Initialization failed: %1").arg(QLatin1String(err)), QMessageBox::Ok, QMessageBox::NoButton);
+			QMessageBox::critical(this, QLatin1String("Mumble"), tr("ASIO Initialization failed: %1").arg(QLatin1String(err)), QMessageBox::Ok, QMessageBox::NoButton);
 		}
 		iasio->Release();
 	} else {
-		QMessageBox::critical(this, tr("Mumble"), tr("Failed to instantiate ASIO driver"), QMessageBox::Ok, QMessageBox::NoButton);
+		QMessageBox::critical(this, QLatin1String("Mumble"), tr("Failed to instantiate ASIO driver"), QMessageBox::Ok, QMessageBox::NoButton);
 	}
 }
 
@@ -279,25 +278,23 @@ void ASIOConfig::on_qpbConfig_clicked() {
 	CLSID clsid;
 	IASIO *iasio;
 
-	bool ok = false;
-
 	CLSIDFromString(const_cast<wchar_t *>(reinterpret_cast<const wchar_t *>(qsCls.utf16())), &clsid);
 	if (CoCreateInstance(clsid, NULL, CLSCTX_INPROC_SERVER, clsid, reinterpret_cast<void **>(&iasio)) == S_OK) {
 		SleepEx(10, false);
 		if (iasio->init(winId())) {
 			SleepEx(10, false);
-			ok = (iasio->controlPanel() == ASE_OK);
+			iasio->controlPanel();
 			SleepEx(10, false);
 		} else {
 			SleepEx(10, false);
 			char err[255];
 			iasio->getErrorMessage(err);
 			SleepEx(10, false);
-			QMessageBox::critical(this, tr("Mumble"), tr("ASIO Initialization failed: %1").arg(QLatin1String(err)), QMessageBox::Ok, QMessageBox::NoButton);
+			QMessageBox::critical(this, QLatin1String("Mumble"), tr("ASIO Initialization failed: %1").arg(QLatin1String(err)), QMessageBox::Ok, QMessageBox::NoButton);
 		}
 		iasio->Release();
 	} else {
-		QMessageBox::critical(this, tr("Mumble"), tr("Failed to instantiate ASIO driver"), QMessageBox::Ok, QMessageBox::NoButton);
+		QMessageBox::critical(this, QLatin1String("Mumble"), tr("Failed to instantiate ASIO driver"), QMessageBox::Ok, QMessageBox::NoButton);
 	}
 }
 
@@ -414,7 +411,7 @@ ASIOInput::ASIOInput() {
 	iNumSpeaker=g.s.qlASIOspeaker.count();
 
 	if ((iNumMic == 0) || (iNumSpeaker == 0)) {
-		QMessageBox::warning(NULL, tr("Mumble"), tr("You need to select at least one microphone and one speaker source to use ASIO. "
+		QMessageBox::warning(NULL, QLatin1String("Mumble"), tr("You need to select at least one microphone and one speaker source to use ASIO. "
 		                     "If you just need microphone sampling, use DirectSound."),  QMessageBox::Ok, QMessageBox::NoButton);
 		return;
 	}
@@ -517,7 +514,7 @@ ASIOInput::ASIOInput() {
 		iasio = NULL;
 	}
 
-	QMessageBox::critical(NULL, tr("Mumble"), tr("Opening selected ASIO device failed. No input will be done."),
+	QMessageBox::critical(NULL, QLatin1String("Mumble"), tr("Opening selected ASIO device failed. No input will be done."),
 	                      QMessageBox::Ok, QMessageBox::NoButton);
 
 }
@@ -531,14 +528,12 @@ ASIOInput::~ASIOInput() {
 		iasio->Release();
 		iasio = NULL;
 	}
-	if (abiInfo) {
-		delete [] abiInfo;
-		abiInfo = NULL;
-	}
-	if (aciInfo) {
-		delete [] aciInfo;
-		aciInfo = NULL;
-	}
+
+	delete [] abiInfo;
+	abiInfo = NULL;
+
+	delete [] aciInfo;
+	aciInfo = NULL;
 }
 
 void ASIOInput::run() {
@@ -573,6 +568,13 @@ ASIOInput::addBuffer(ASIOSampleType sampType, int interleave, void *src, float *
 					dst[i*interleave]=buf[i] * m;
 			}
 			break;
+		case ASIOSTInt24LSB: {
+				const float m = 1.0f / static_cast<float>(0x7FFFFFFF - 0xFF);
+				const unsigned char * RESTRICT buf=static_cast<unsigned char *>(src);
+				for (int i=0;i<lBufSize;i++)
+					dst[i * interleave] = (buf[i*3] << 24 | buf[i*3+1] << 16 | buf[i*3+2] << 8) * m;
+			}
+			break;
 		case ASIOSTFloat32LSB: {
 				const float * RESTRICT buf=static_cast<float *>(src);
 				for (int i=0;i<lBufSize;i++)
