@@ -31,6 +31,7 @@
 #include <math.h>
 
 #include "Audio.h"
+#include "AudioInput.h"
 #include "AudioOutput.h"
 #include "Global.h"
 #include "PacketDataStream.h"
@@ -49,22 +50,6 @@ CodecInit ciInit;
 void CodecInit::initialize() {
 	CELTCodec *codec = NULL;
 
-	codec = new CELTCodec061(QLatin1String("0.6.1"));
-	if (codec->isValid()) {
-		codec->report();
-		g.qmCodecs.insert(codec->bitstreamVersion(), codec);
-	} else {
-		delete codec;
-	}
-
-	codec = new CELTCodec061(QLatin1String("0.6.2"));
-	if (codec->isValid()) {
-		codec->report();
-		g.qmCodecs.insert(codec->bitstreamVersion(), codec);
-	} else {
-		delete codec;
-	}
-
 	codec = new CELTCodec070(QLatin1String("0.7.0"));
 	if (codec->isValid()) {
 		codec->report();
@@ -79,6 +64,44 @@ void CodecInit::initialize() {
 			delete codec;
 		}
 	}
+
+	codec = new CELTCodec080(QLatin1String("0.9.0"));
+	if (codec->isValid()) {
+		codec->report();
+		g.qmCodecs.insert(codec->bitstreamVersion(), codec);
+	} else {
+		delete codec;
+	}
+
+	codec = new CELTCodec080(QLatin1String("0.10.0"));
+	if (codec->isValid()) {
+		codec->report();
+		g.qmCodecs.insert(codec->bitstreamVersion(), codec);
+	} else {
+		delete codec;
+		codec = new CELTCodec080(QLatin1String("1.0.0"));
+		if (codec->isValid()) {
+			codec->report();
+			g.qmCodecs.insert(codec->bitstreamVersion(), codec);
+		} else {
+			delete codec;
+		}
+	}
+
+	codec = new CELTCodec011(QLatin1String("0.11.0"));
+	if (codec->isValid()) {
+		codec->report();
+		g.qmCodecs.insert(codec->bitstreamVersion(), codec);
+	} else {
+		delete codec;
+		codec = new CELTCodec011(QLatin1String("2.0.0"));
+		if (codec->isValid()) {
+			codec->report();
+			g.qmCodecs.insert(codec->bitstreamVersion(), codec);
+		} else {
+			delete codec;
+		}
+	}
 }
 
 void CodecInit::destroy() {
@@ -100,6 +123,7 @@ CELTCodec::CELTCodec(const QString &version) {
 	bValid = false;
 	cmMode = NULL;
 	qsVersion = version;
+	iBitstreamVersion = INT_MIN;
 	qlCELT.setLoadHints(QLibrary::ResolveAllSymbolsHint);
 
 	QStringList alternatives;
@@ -161,13 +185,9 @@ CELTCodec::CELTCodec(const QString &version) {
 	RESOLVE(celt_mode_info);
 
 	RESOLVE(celt_encoder_destroy);
-	RESOLVE(celt_encode_float);
-	RESOLVE(celt_encode);
 	RESOLVE(celt_encoder_ctl);
 
 	RESOLVE(celt_decoder_destroy);
-	RESOLVE(celt_decode_float);
-	RESOLVE(celt_decode);
 	RESOLVE(celt_decoder_ctl);
 }
 
@@ -181,11 +201,10 @@ bool CELTCodec::isValid() const {
 }
 
 int CELTCodec::bitstreamVersion() const {
-	int v = INT_MIN;
+	if (cmMode && iBitstreamVersion == INT_MIN)
+		celt_mode_info(cmMode, CELT_GET_BITSTREAM_VERSION, reinterpret_cast<celt_int32 *>(&iBitstreamVersion));
 
-	if (cmMode)
-		celt_mode_info(cmMode, CELT_GET_BITSTREAM_VERSION, reinterpret_cast<celt_int32 *>(&v));
-	return v;
+	return iBitstreamVersion;
 }
 
 QString CELTCodec::version() const {
@@ -196,30 +215,14 @@ void CELTCodec::report() const {
 	qWarning("CELT bitstream %08x from %s", bitstreamVersion(), qPrintable(qlCELT.fileName()));
 }
 
-CELTCodec061::CELTCodec061(const QString &version) : CELTCodec(version) {
-	RESOLVE(celt_mode_create);
-
-	RESOLVE(celt_encoder_create);
-
-	RESOLVE(celt_decoder_create);
-
-	if (bValid) {
-		cmMode = celt_mode_create(SAMPLE_RATE, 1, SAMPLE_RATE / 100, NULL);
-	}
-}
-
-CELTEncoder *CELTCodec061::encoderCreate() {
-	return celt_encoder_create(cmMode);
-}
-
-CELTDecoder *CELTCodec061::decoderCreate() {
-	return celt_decoder_create(cmMode);
-}
-
 CELTCodec070::CELTCodec070(const QString &version) : CELTCodec(version) {
 	RESOLVE(celt_mode_create);
 	RESOLVE(celt_encoder_create);
 	RESOLVE(celt_decoder_create);
+	RESOLVE(celt_encode_float);
+	RESOLVE(celt_encode);
+	RESOLVE(celt_decode_float);
+	RESOLVE(celt_decode);
 	RESOLVE(celt_strerror);
 
 	if (bValid) {
@@ -235,13 +238,83 @@ CELTDecoder *CELTCodec070::decoderCreate() {
 	return celt_decoder_create(cmMode, 1, NULL);
 }
 
+int CELTCodec070::encode(CELTEncoder *st, const celt_int16 *pcm, unsigned char *compressed, int nbCompressedBytes) {
+	return celt_encode(st, pcm, NULL, compressed, nbCompressedBytes);
+}
+
+int CELTCodec070::decode_float(CELTDecoder *st, const unsigned char *data, int len, float *pcm) {
+	return celt_decode_float(st, data, len, pcm);
+}
+
+CELTCodec080::CELTCodec080(const QString &version) : CELTCodec(version) {
+	RESOLVE(celt_mode_create);
+	RESOLVE(celt_encoder_create);
+	RESOLVE(celt_decoder_create);
+	RESOLVE(celt_encode_float);
+	RESOLVE(celt_encode);
+	RESOLVE(celt_decode_float);
+	RESOLVE(celt_decode);
+	RESOLVE(celt_strerror);
+
+	if (bValid) {
+		cmMode = celt_mode_create(SAMPLE_RATE, SAMPLE_RATE / 100, NULL);
+	}
+}
+
+CELTEncoder *CELTCodec080::encoderCreate() {
+	return celt_encoder_create(cmMode, 1, NULL);
+}
+
+CELTDecoder *CELTCodec080::decoderCreate() {
+	return celt_decoder_create(cmMode, 1, NULL);
+}
+
+int CELTCodec080::encode(CELTEncoder *st, const celt_int16 *pcm, unsigned char *compressed, int nbCompressedBytes) {
+	return celt_encode(st, pcm, SAMPLE_RATE / 100, compressed, nbCompressedBytes);
+}
+
+int CELTCodec080::decode_float(CELTDecoder *st, const unsigned char *data, int len, float *pcm) {
+	return celt_decode_float(st, data, len, pcm, SAMPLE_RATE / 100);
+}
+
+CELTCodec011::CELTCodec011(const QString &version) : CELTCodec(version) {
+	RESOLVE(celt_mode_create);
+	RESOLVE(celt_encoder_create_custom);
+	RESOLVE(celt_decoder_create_custom);
+	RESOLVE(celt_encode_float);
+	RESOLVE(celt_encode);
+	RESOLVE(celt_decode_float);
+	RESOLVE(celt_decode);
+	RESOLVE(celt_strerror);
+
+	if (bValid) {
+		cmMode = celt_mode_create(SAMPLE_RATE, SAMPLE_RATE / 100, NULL);
+	}
+}
+
+CELTEncoder *CELTCodec011::encoderCreate() {
+	return celt_encoder_create_custom(cmMode, 1, NULL);
+}
+
+CELTDecoder *CELTCodec011::decoderCreate() {
+	return celt_decoder_create_custom(cmMode, 1, NULL);
+}
+
+int CELTCodec011::encode(CELTEncoder *st, const celt_int16 *pcm, unsigned char *compressed, int nbCompressedBytes) {
+	return celt_encode(st, pcm, SAMPLE_RATE / 100, compressed, nbCompressedBytes);
+}
+
+int CELTCodec011::decode_float(CELTDecoder *st, const unsigned char *data, int len, float *pcm) {
+	return celt_decode_float(st, data, len, pcm, SAMPLE_RATE / 100);
+}
+
 LoopUser::LoopUser() {
 	qsName = QLatin1String("Loopy");
 	uiSession = 0;
 	iId = 0;
 	bMute = bDeaf = bSuppress = false;
 	bLocalMute = bSelfDeaf = false;
-	tsState = ClientUser::TalkingOff;
+	tsState = Settings::Passive;
 	cChannel = NULL;
 	qtTicker.start();
 }
@@ -252,10 +325,9 @@ void LoopUser::addFrame(const QByteArray &packet) {
 		return;
 	}
 
-	bool restart = (qtLastFetch.elapsed() > 100);
-
 	{
 		QMutexLocker l(&qmLock);
+		bool restart = (qtLastFetch.elapsed() > 100);
 
 		double time = qtTicker.elapsed();
 
@@ -282,7 +354,7 @@ void LoopUser::addFrame(const QByteArray &packet) {
 void LoopUser::fetchFrames() {
 	QMutexLocker l(&qmLock);
 
-	AudioOutputPtr ao = g.ao;
+	AudioOutputPtr ao(g.ao);
 	if (!ao || qmPackets.isEmpty()) {
 		return;
 	}
@@ -316,3 +388,97 @@ void LoopUser::fetchFrames() {
 
 	qtLastFetch.restart();
 }
+
+RecordUser::RecordUser() : LoopUser() {
+	qsName = QLatin1String("Recorder");
+}
+
+RecordUser::~RecordUser() {
+	AudioOutputPtr ao = g.ao;
+	if (ao)
+		ao->removeBuffer(this);
+}
+
+void RecordUser::addFrame(const QByteArray &packet) {
+	AudioOutputPtr ao(g.ao);
+	if (!ao)
+		return;
+
+	int iSeq;
+	PacketDataStream pds(packet.constData(), packet.size());
+
+	unsigned int msgFlags = static_cast<unsigned int>(pds.next());
+
+	pds >> iSeq;
+
+	QByteArray qba;
+	qba.reserve(pds.left() + 1);
+	qba.append(static_cast<char>(msgFlags));
+	qba.append(pds.dataBlock(pds.left()));
+
+	MessageHandler::UDPMessageType msgType = static_cast<MessageHandler::UDPMessageType>((msgFlags >> 5) & 0x7);
+
+	ao->addFrameToBuffer(this, qba, iSeq, msgType);
+}
+
+void Audio::startOutput(const QString &output) {
+	g.ao = AudioOutputRegistrar::newFromChoice(output);
+	if (g.ao)
+		g.ao->start(QThread::HighPriority);
+}
+
+void Audio::stopOutput() {
+	AudioOutputPtr ao = g.ao;
+
+	g.ao.reset();
+
+	while (ao.get() && ! ao.unique()) {
+#if QT_VERSION >= 0x040500
+		QThread::yieldCurrentThread();
+#endif
+	}
+
+	ao.reset();
+}
+
+void Audio::startInput(const QString &input) {
+	g.ai = AudioInputRegistrar::newFromChoice(input);
+	if (g.ai)
+		g.ai->start(QThread::HighestPriority);
+}
+
+void Audio::stopInput() {
+	AudioInputPtr ai = g.ai;
+
+	g.ai.reset();
+
+	while (ai.get() && ! ai.unique()) {
+#if QT_VERSION >= 0x040500
+		QThread::yieldCurrentThread();
+#endif
+	}
+
+	ai.reset();
+}
+
+void Audio::start(const QString &input, const QString &output) {
+	startInput(input);
+	startOutput(output);
+}
+
+void Audio::stop() {
+	AudioInputPtr ai = g.ai;
+	AudioOutputPtr ao = g.ao;
+
+	g.ao.reset();
+	g.ai.reset();
+
+	while ((ai.get() && ! ai.unique()) || (ao.get() && ! ao.unique())) {
+#if QT_VERSION >= 0x040500
+		QThread::yieldCurrentThread();
+#endif
+	}
+
+	ai.reset();
+	ao.reset();
+}
