@@ -55,17 +55,14 @@ void AudioInputDialog::showEvent(QShowEvent *) {
 }
 
 AudioInputDialog::AudioInputDialog(Settings &st) : ConfigWidget(st) {
-	QList<QString> keys;
-	QString key;
-
 	qtTick = new QTimer(this);
 	qtTick->setObjectName(QLatin1String("Tick"));
 
 	setupUi(this);
 
 	if (AudioInputRegistrar::qmNew) {
-		keys=AudioInputRegistrar::qmNew->keys();
-		foreach(key, keys) {
+		QList<QString> keys = AudioInputRegistrar::qmNew->keys();
+		foreach(QString key, keys) {
 			qcbSystem->addItem(key);
 		}
 	}
@@ -105,15 +102,17 @@ void AudioInputDialog::load(const Settings &r) {
 	if (i >= 0)
 		loadComboBox(qcbSystem, i);
 
+	loadCheckBox(qcbExclusive, r.bExclusiveInput);
+
 	qlePushClickPathOn->setText(r.qsPushClickOn);
 	qlePushClickPathOff->setText(r.qsPushClickOff);
 
 	loadComboBox(qcbTransmit, r.atTransmit);
 	loadSlider(qsTransmitHold, r.iVoiceHold);
-	loadSlider(qsTransmitMin, iroundf(r.fVADmin * 32767.0f));
-	loadSlider(qsTransmitMax, iroundf(r.fVADmax * 32767.0f));
+	loadSlider(qsTransmitMin, iroundf(r.fVADmin * 32767.0f + 0.5f));
+	loadSlider(qsTransmitMax, iroundf(r.fVADmax * 32767.0f + 0.5f));
 	loadSlider(qsFrames, (r.iFramesPerPacket == 1) ? 1 : (r.iFramesPerPacket/2 + 1));
-	loadSlider(qsDoublePush, iroundf(static_cast<float>(r.uiDoublePush) / 1000.f));
+	loadSlider(qsDoublePush, iroundf(static_cast<float>(r.uiDoublePush) / 1000.f + 0.5f));
 
 	if (r.vsVAD == Settings::Amplitude)
 		qrbAmplitude->setChecked(true);
@@ -158,6 +157,7 @@ void AudioInputDialog::save() const {
 	s.qsAudioInput = qcbSystem->currentText();
 	s.bEcho = qcbEcho->currentIndex() > 0;
 	s.bEchoMulti = qcbEcho->currentIndex() == 2;
+	s.bExclusiveInput = qcbExclusive->isChecked();
 
 	if (AudioInputRegistrar::qmNew) {
 		AudioInputRegistrar *air = AudioInputRegistrar::qmNew->value(qcbSystem->currentText());
@@ -355,6 +355,7 @@ void AudioInputDialog::on_qcbSystem_currentIndexChanged(int) {
 		}
 
 		qcbEcho->setEnabled(air->canEcho(s.qsAudioOutput));
+		qcbExclusive->setEnabled(air->canExclusive());
 	}
 
 	qcbDevice->setEnabled(ql.count() > 1);
@@ -370,22 +371,19 @@ void AudioInputDialog::on_Tick_timeout() {
 	abSpeech->iAbove = qsTransmitMax->value();
 
 	if (qrbAmplitude->isChecked()) {
-		abSpeech->iValue = iroundf((32767.f/96.0f) * (96.0f + ai->dPeakMic));
+		abSpeech->iValue = iroundf((32767.f/96.0f) * (96.0f + ai->dPeakCleanMic) + 0.5f);
 	} else {
-		abSpeech->iValue = iroundf(ai->fSpeechProb * 32767.0f);
+		abSpeech->iValue = iroundf(ai->fSpeechProb * 32767.0f + 0.5f);
 	}
 	abSpeech->update();
 }
 
 AudioOutputDialog::AudioOutputDialog(Settings &st) : ConfigWidget(st) {
-	QList<QString> keys;
-	QString key;
-
 	setupUi(this);
 
 	if (AudioOutputRegistrar::qmNew) {
-		keys=AudioOutputRegistrar::qmNew->keys();
-		foreach(key, keys) {
+		QList<QString> keys = AudioOutputRegistrar::qmNew->keys();
+		foreach(QString key, keys) {
 			qcbSystem->addItem(key);
 		}
 	}
@@ -418,19 +416,20 @@ void AudioOutputDialog::load(const Settings &r) {
 	if (i >= 0)
 		loadComboBox(qcbSystem, i);
 
+	loadCheckBox(qcbExclusive, r.bExclusiveOutput);
 	loadSlider(qsDelay, r.iOutputDelay);
-	loadSlider(qsVolume, iroundf(r.fVolume * 100.0f));
-	loadSlider(qsOtherVolume, iroundf((1.0f - r.fOtherVolume) * 100.0f));
+	loadSlider(qsVolume, iroundf(r.fVolume * 100.0f + 0.5f));
+	loadSlider(qsOtherVolume, iroundf((1.0f - r.fOtherVolume) * 100.0f + 0.5f));
 	loadCheckBox(qcbAttenuateOthersOnTalk, r.bAttenuateOthersOnTalk);
 	loadCheckBox(qcbAttenuateOthers, r.bAttenuateOthers);
 	loadSlider(qsJitter, r.iJitterBufferSize);
 	loadComboBox(qcbLoopback, r.lmLoopMode);
 	loadSlider(qsPacketDelay, static_cast<int>(r.dMaxPacketDelay));
-	loadSlider(qsPacketLoss, iroundf(r.dPacketLoss * 100.0f));
-	loadSlider(qsMinDistance, iroundf(r.fAudioMinDistance * 10.0f));
-	loadSlider(qsMaxDistance, iroundf(r.fAudioMaxDistance * 10.0f));
-	loadSlider(qsMaxDistVolume, iroundf(r.fAudioMaxDistVolume * 100.0f));
-	loadSlider(qsBloom, iroundf(r.fAudioBloom * 100.0f));
+	loadSlider(qsPacketLoss, iroundf(r.dPacketLoss * 100.0f + 0.5f));
+	loadSlider(qsMinDistance, iroundf(r.fAudioMinDistance * 10.0f + 0.5f));
+	loadSlider(qsMaxDistance, iroundf(r.fAudioMaxDistance * 10.0f + 0.5f));
+	loadSlider(qsMaxDistVolume, iroundf(r.fAudioMaxDistVolume * 100.0f + 0.5f));
+	loadSlider(qsBloom, iroundf(r.fAudioBloom * 100.0f + 0.5f));
 	loadCheckBox(qcbHeadphones, r.bPositionalHeadphone);
 	loadCheckBox(qcbPositional, r.bPositionalAudio);
 
@@ -455,6 +454,7 @@ void AudioOutputDialog::save() const {
 	s.fAudioBloom = static_cast<float>(qsBloom->value()) / 100.0f;
 	s.bPositionalAudio = qcbPositional->isChecked();
 	s.bPositionalHeadphone = qcbHeadphones->isChecked();
+	s.bExclusiveOutput = qcbExclusive->isChecked();
 
 
 	if (AudioOutputRegistrar::qmNew) {
@@ -495,6 +495,8 @@ void AudioOutputDialog::on_qcbSystem_currentIndexChanged(int) {
 		bool usesdelay = aor->usesOutputDelay();
 		qsDelay->setEnabled(usesdelay);
 		qlDelay->setEnabled(usesdelay);
+
+		qcbExclusive->setEnabled(aor->canExclusive());
 	}
 
 	qcbDevice->setEnabled(ql.count() > 1);
