@@ -37,6 +37,7 @@
 #include "Message.h"
 #include "Global.h"
 #include "NetworkConfig.h"
+#include "VoiceRecorder.h"
 
 // Remember that we cannot use static member classes that are not pointers, as the constructor
 // for AudioInputRegistrar() might be called before they are initialized, as the constructor
@@ -82,6 +83,9 @@ AudioInputPtr AudioInputRegistrar::newFromChoice(QString choice) {
 	return AudioInputPtr();
 }
 
+bool AudioInputRegistrar::canExclusive() const {
+	return false;
+}
 
 AudioInput::AudioInput() {
 	adjustBandwidth(g.iMaxBandwidth, iAudioQuality, iAudioFrames);
@@ -130,8 +134,6 @@ AudioInput::AudioInput() {
 	}
 	iEchoFreq = iMicFreq = iSampleRate;
 
-	mumble_drft_init(&fftTable, iFrameSize);
-
 	iFrameCounter = 0;
 	iSilentFrames = 0;
 	iHoldFrames = 0;
@@ -161,15 +163,15 @@ AudioInput::AudioInput() {
 	pfMicInput = pfEchoInput = pfOutput = NULL;
 
 	iBitrate = 0;
-	dPeakMic = dPeakSignal = dPeakSpeaker = 0.0;
+	dPeakSignal = dPeakSpeaker = dPeakMic = dPeakCleanMic = 0.0;
 
 	if (g.uiSession) {
 		setMaxBandwidth(g.iMaxBandwidth);
 	}
 
-	bRunning = false;
+	bRunning = true;
 
-	connect(this, SIGNAL(doMute()), g.mw->qaAudioMute, SLOT(trigger()), Qt::QueuedConnection);
+	connect(this, SIGNAL(doDeaf()), g.mw->qaAudioDeaf, SLOT(trigger()), Qt::QueuedConnection);
 }
 
 AudioInput::~AudioInput() {
@@ -183,8 +185,6 @@ AudioInput::~AudioInput() {
 		speex_encoder_destroy(esSpeex);
 	}
 
-	mumble_drft_clear(&fftTable);
-
 	foreach(short *buf, qlEchoFrames)
 		delete [] buf;
 
@@ -200,15 +200,11 @@ AudioInput::~AudioInput() {
 
 	delete [] psMic;
 	delete [] psClean;
-	if (psSpeaker)
-		delete [] psSpeaker;
+	delete [] psSpeaker;
 
-	if (pfMicInput)
-		delete [] pfMicInput;
-	if (pfEchoInput)
-		delete [] pfEchoInput;
-	if (pfOutput)
-		delete [] pfOutput;
+	delete [] pfMicInput;
+	delete [] pfEchoInput;
+	delete [] pfOutput;
 }
 
 bool AudioInput::isTransmitting() const {
@@ -334,12 +330,9 @@ void AudioInput::initializeMixer() {
 		speex_resampler_destroy(srsMic);
 	if (srsEcho)
 		speex_resampler_destroy(srsEcho);
-	if (pfMicInput)
-		delete [] pfMicInput;
-	if (pfEchoInput)
-		delete [] pfEchoInput;
-	if (pfOutput)
-		delete [] pfOutput;
+	delete [] pfMicInput;
+	delete [] pfEchoInput;
+	delete [] pfOutput;
 
 	if (iMicFreq != iSampleRate)
 		srsMic = speex_resampler_init(1, iMicFreq, iSampleRate, 3, &err);
@@ -424,8 +417,7 @@ void AudioInput::addMic(const void *data, unsigned int nsamp) {
 				}
 
 				if (echo) {
-					if (psSpeaker)
-						delete [] psSpeaker;
+					delete [] psSpeaker;
 					psSpeaker = echo;
 				}
 			}
@@ -535,12 +527,11 @@ void AudioInput::setMaxBandwidth(int bitspersec) {
 		ai->iAudioFrames = frames;
 		return;
 	}
-	g.ai.reset();
-	while (! ai.unique()) {}
+
 	ai.reset();
-	g.ai = AudioInputRegistrar::newFromChoice(g.s.qsAudioInput);
-	if (g.ai)
-		g.ai->start(QThread::HighestPriority);
+
+	Audio::stopInput();
+	Audio::startInput();
 }
 
 int AudioInput::getNetworkBandwidth(int bitrate, int frames) {
@@ -614,16 +605,13 @@ void AudioInput::encodeAudioFrame() {
 
 	iFrameCounter++;
 
-	if (! bRunning) {
+	if (! bRunning)
 		return;
-	}
 
 	sum=1.0f;
 	for (i=0;i<iFrameSize;i++)
 		sum += static_cast<float>(psMic[i] * psMic[i]);
-	dPeakMic=20.0f*log10f(sqrtf(sum / static_cast<float>(iFrameSize)) / 32768.0f);
-	if (dPeakMic < -96.0f)
-		dPeakMic = -96.0f;
+	dPeakMic = qMax(20.0f*log10f(sqrtf(sum / static_cast<float>(iFrameSize)) / 32768.0f), -96.0f);
 
 	max = 1;
 	for (i=0;i<iFrameSize;i++)
@@ -634,9 +622,7 @@ void AudioInput::encodeAudioFrame() {
 		sum=1.0f;
 		for (i=0;i<iFrameSize;i++)
 			sum += static_cast<float>(psSpeaker[i] * psSpeaker[i]);
-		dPeakSpeaker=20.0f*log10f(sqrtf(sum / static_cast<float>(iFrameSize)) / 32768.0f);
-		if (dPeakSpeaker < -96.0f)
-			dPeakSpeaker = -96.0f;
+		dPeakSpeaker = qMax(20.0f*log10f(sqrtf(sum / static_cast<float>(iFrameSize)) / 32768.0f), -96.0f);
 	} else {
 		dPeakSpeaker = 0.0;
 	}
@@ -664,6 +650,9 @@ void AudioInput::encodeAudioFrame() {
 		iArg = iroundf(floorf(20.0f * log10f(v)));
 		speex_preprocess_ctl(sppPreprocess, SPEEX_PREPROCESS_SET_AGC_MAX_GAIN, &iArg);
 
+		iArg = -60;
+		speex_preprocess_ctl(sppPreprocess, SPEEX_PREPROCESS_SET_AGC_DECREMENT, &iArg);
+
 		iArg = g.s.iNoiseSuppress;
 		speex_preprocess_ctl(sppPreprocess, SPEEX_PREPROCESS_SET_NOISE_SUPPRESS, &iArg);
 
@@ -681,14 +670,17 @@ void AudioInput::encodeAudioFrame() {
 		bResetProcessor = false;
 	}
 
-	int iIsSpeech;
+	speex_preprocess_ctl(sppPreprocess, SPEEX_PREPROCESS_GET_AGC_GAIN, &iArg);
+	float gainValue = static_cast<float>(iArg);
+	iArg = g.s.iNoiseSuppress - iArg;
+	speex_preprocess_ctl(sppPreprocess, SPEEX_PREPROCESS_SET_NOISE_SUPPRESS, &iArg);
 
 	if (sesEcho && psSpeaker) {
 		speex_echo_cancellation(sesEcho, psMic, psSpeaker, psClean);
-		iIsSpeech=speex_preprocess_run(sppPreprocess, psClean);
+		speex_preprocess_run(sppPreprocess, psClean);
 		psSource = psClean;
 	} else {
-		iIsSpeech=speex_preprocess_run(sppPreprocess, psMic);
+		speex_preprocess_run(sppPreprocess, psMic);
 		psSource = psMic;
 	}
 
@@ -696,43 +688,43 @@ void AudioInput::encodeAudioFrame() {
 	for (i=0;i<iFrameSize;i++)
 		sum += static_cast<float>(psSource[i] * psSource[i]);
 	float micLevel = sqrtf(sum / static_cast<float>(iFrameSize));
-	dPeakSignal=20.0f*log10f(micLevel / 32768.0f);
-	if (dPeakSignal < -96.0f)
-		dPeakSignal = -96.0f;
+	dPeakSignal = qMax(20.0f*log10f(micLevel / 32768.0f), -96.0f);
 
 	spx_int32_t prob = 0;
 	speex_preprocess_ctl(sppPreprocess, SPEEX_PREPROCESS_GET_PROB, &prob);
 	fSpeechProb = static_cast<float>(prob) / 100.0f;
 
-	float level = (g.s.vsVAD == Settings::SignalToNoise) ? fSpeechProb : (1.0f + dPeakMic / 96.0f);
+	// clean microphone level: peak of filtered signal attenuated by AGC gain
+	dPeakCleanMic = qMax(dPeakSignal - gainValue, -96.0f);
+	float level = (g.s.vsVAD == Settings::SignalToNoise) ? fSpeechProb : (1.0f + dPeakCleanMic / 96.0f);
+
+	bool bIsSpeech = false;
 
 	if (level > g.s.fVADmax)
-		iIsSpeech = 1;
+		bIsSpeech = true;
 	else if (level > g.s.fVADmin && bPreviousVoice)
-		iIsSpeech = 1;
-	else
-		iIsSpeech = 0;
+		bIsSpeech = true;
 
-	if (! iIsSpeech) {
+	if (! bIsSpeech) {
 		iHoldFrames++;
 		if (iHoldFrames < g.s.iVoiceHold)
-			iIsSpeech=1;
+			bIsSpeech = true;
 	} else {
 		iHoldFrames = 0;
 	}
 
 	if (g.s.atTransmit == Settings::Continous)
-		iIsSpeech = 1;
+		bIsSpeech = true;
 	else if (g.s.atTransmit == Settings::PushToTalk)
-		iIsSpeech = g.s.uiDoublePush && ((g.uiDoublePush < g.s.uiDoublePush) || (g.tDoublePush.elapsed() < g.s.uiDoublePush));
+		bIsSpeech = g.s.uiDoublePush && ((g.uiDoublePush < g.s.uiDoublePush) || (g.tDoublePush.elapsed() < g.s.uiDoublePush));
 
-	iIsSpeech = iIsSpeech || (g.iPushToTalk > 0);
+	bIsSpeech = bIsSpeech || (g.iPushToTalk > 0);
 
 	if (g.s.bMute || ((g.s.lmLoopMode != Settings::Local) && p && (p->bMute || p->bSuppress)) || g.bPushToMute || (g.iTarget < 0)) {
-		iIsSpeech = 0;
+		bIsSpeech = false;
 	}
 
-	if (iIsSpeech) {
+	if (bIsSpeech) {
 		iSilentFrames = 0;
 	} else {
 		iSilentFrames++;
@@ -741,28 +733,33 @@ void AudioInput::encodeAudioFrame() {
 	}
 
 	if (p) {
-		if (! iIsSpeech)
-			p->setTalking(ClientUser::TalkingOff);
+		if (! bIsSpeech)
+			p->setTalking(Settings::Passive);
 		else if (g.iTarget == 0)
-			p->setTalking(ClientUser::Talking);
+			p->setTalking(Settings::Talking);
 		else
-			p->setTalking(ClientUser::TalkingWhisperChannel);
+			p->setTalking(Settings::Shouting);
 	}
 
 	if (g.s.bPushClick && (g.s.atTransmit == Settings::PushToTalk)) {
 		AudioOutputPtr ao = g.ao;
-		if (iIsSpeech && ! bPreviousVoice && ao)
+		if (bIsSpeech && ! bPreviousVoice && ao)
 			ao->playSample(g.s.qsPushClickOn);
-		else if (ao && !iIsSpeech && bPreviousVoice && ao)
+		else if (ao && !bIsSpeech && bPreviousVoice && ao)
 			ao->playSample(g.s.qsPushClickOff);
 	}
-	if (! iIsSpeech && ! bPreviousVoice) {
+	if (! bIsSpeech && ! bPreviousVoice) {
 		iBitrate = 0;
-		if (g.s.iIdleTime && ! g.s.bMute && ((tIdle.elapsed() / 1000000ULL) > g.s.iIdleTime)) {
-			emit doMute();
+		if (g.s.iIdleTime && ! g.s.bDeaf && ((tIdle.elapsed() / 1000000ULL) > g.s.iIdleTime)) {
+			emit doDeaf();
 			tIdle.restart();
 		}
+		spx_int32_t increment = 0;
+		speex_preprocess_ctl(sppPreprocess, SPEEX_PREPROCESS_SET_AGC_INCREMENT, &increment);
 		return;
+	} else {
+		spx_int32_t increment = 12;
+		speex_preprocess_ctl(sppPreprocess, SPEEX_PREPROCESS_SET_AGC_INCREMENT, &increment);
 	}
 
 	tIdle.restart();
@@ -792,10 +789,8 @@ void AudioInput::encodeAudioFrame() {
 		}
 		if (! switchto) {
 			switchto = g.qmCodecs.value(g.bPreferAlpha ? g.iCodecAlpha : g.iCodecBeta);
-			umtType = g.bPreferAlpha ? MessageHandler::UDPVoiceCELTAlpha : MessageHandler::UDPVoiceCELTBeta;
 			if (! switchto) {
 				switchto = g.qmCodecs.value(g.bPreferAlpha ? g.iCodecBeta : g.iCodecAlpha);
-				umtType = g.bPreferAlpha ? MessageHandler::UDPVoiceCELTBeta : MessageHandler::UDPVoiceCELTAlpha;
 			}
 		}
 		if (switchto != cCodec) {
@@ -814,10 +809,22 @@ void AudioInput::encodeAudioFrame() {
 		if (! cCodec)
 			return;
 
+		if (g.uiSession && g.s.lmLoopMode != Settings::Local) {
+			int v = cCodec->bitstreamVersion();
+			if (v == g.iCodecAlpha)
+				umtType = MessageHandler::UDPVoiceCELTAlpha;
+			else if (v == g.iCodecBeta)
+				umtType = MessageHandler::UDPVoiceCELTBeta;
+			else {
+				qWarning() << "Couldn't find message type for codec version" << v;
+				return;
+			}
+		}
+
 		cCodec->celt_encoder_ctl(ceEncoder, CELT_SET_PREDICTION(0));
 
 		cCodec->celt_encoder_ctl(ceEncoder,CELT_SET_VBR_RATE(iAudioQuality));
-		len = cCodec->celt_encode(ceEncoder, psSource, NULL, buffer, qMin(iAudioQuality / 800, 127));
+		len = cCodec->encode(ceEncoder, psSource, buffer, qMin(iAudioQuality / 800, 127));
 		iBitrate = len * 100 * 8;
 	} else {
 		int vbr = 0;
@@ -836,12 +843,12 @@ void AudioInput::encodeAudioFrame() {
 		speex_bits_reset(&sbBits);
 	}
 
-	flushCheck(QByteArray(reinterpret_cast<const char *>(buffer), len), ! iIsSpeech);
+	flushCheck(QByteArray(reinterpret_cast<const char *>(buffer), len), ! bIsSpeech);
 
-	if (! iIsSpeech)
+	if (! bIsSpeech)
 		iBitrate = 0;
 
-	bPreviousVoice = iIsSpeech;
+	bPreviousVoice = bIsSpeech;
 }
 
 void AudioInput::flushCheck(const QByteArray &frame, bool terminator) {
@@ -854,7 +861,7 @@ void AudioInput::flushCheck(const QByteArray &frame, bool terminator) {
 		flags = g.iPrevTarget;
 
 	if (g.s.lmLoopMode == Settings::Server)
-		flags = 0x1f;
+		flags = 0x1f; // Server loopback
 
 	flags |= (umtType << 5);
 
@@ -882,10 +889,21 @@ void AudioInput::flushCheck(const QByteArray &frame, bool terminator) {
 		pds << g.p->fPosition[2];
 	}
 
+	ServerHandlerPtr sh = g.sh;
+	if (sh) {
+		VoiceRecorderPtr recorder(sh->recorder);
+		if (recorder)
+			recorder->getRecordUser().addFrame(QByteArray(data, pds.size() + 1));
+	}
+
 	if (g.s.lmLoopMode == Settings::Local)
 		LoopUser::lpLoopy.addFrame(QByteArray(data, pds.size() + 1));
-	else if (g.sh)
-		g.sh->sendMessage(data, pds.size() + 1);
+	else if (sh)
+		sh->sendMessage(data, pds.size() + 1);
 
 	qlFrames.clear();
 }
+
+bool AudioInput::isAlive() const {
+	return isRunning();
+}
