@@ -57,6 +57,7 @@ class AudioInputRegistrar {
 		virtual const QList<audioDevice> getDeviceChoices() = 0;
 		virtual void setDeviceChoice(const QVariant &, Settings &) = 0;
 		virtual bool canEcho(const QString &outputsys) const = 0;
+		virtual bool canExclusive() const;
 };
 
 class AudioInput : public QThread {
@@ -109,8 +110,6 @@ class AudioInput : public QThread {
 		int iAudioQuality;
 		int iAudioFrames;
 
-		drft_lookup fftTable;
-
 		short *psMic;
 		short *psSpeaker;
 		short *psClean;
@@ -138,14 +137,14 @@ class AudioInput : public QThread {
 		static bool preferCELT(int bitrate, int frames);
 		static void adjustBandwidth(int bitspersec, int &bitrate, int &frames);
 	signals:
-		void doMute();
+		void doDeaf();
 	public:
 		bool bResetProcessor;
 
 		Timer tIdle;
 
 		int iBitrate;
-		float dPeakMic, dPeakSpeaker, dPeakSignal, dMaxMic;
+		float dPeakSpeaker, dPeakSignal, dMaxMic, dPeakMic, dPeakCleanMic;
 		float fSpeechProb;
 
 		static int getNetworkBandwidth(int bitrate, int frames);
@@ -154,6 +153,7 @@ class AudioInput : public QThread {
 		AudioInput();
 		~AudioInput();
 		void run() = 0;
+		virtual bool isAlive() const;
 		bool isTransmitting() const;
 };
 
