@@ -87,6 +87,7 @@ class AudioOutputRegistrar {
 		virtual void setDeviceChoice(const QVariant &, Settings &) = 0;
 		virtual bool canMuteOthers() const;
 		virtual bool usesOutputDelay() const;
+		virtual bool canExclusive() const;
 };
 
 class AudioOutputUser : public QObject {
@@ -154,7 +155,7 @@ class AudioOutputSpeech : public AudioOutputUser {
 class SoundFile : public QObject {
 	private:
 		Q_OBJECT
-		Q_DISABLE_COPY(SoundFile);
+		Q_DISABLE_COPY(SoundFile)
 	protected:
 		SNDFILE *sfFile;
 		SF_INFO siInfo;
@@ -230,12 +231,15 @@ class AudioOutput : public QThread {
 
 		AudioOutput();
 		~AudioOutput();
+
 		void addFrameToBuffer(ClientUser *, const QByteArray &, unsigned int iSeq, MessageHandler::UDPMessageType type);
 		void removeBuffer(const ClientUser *);
 		AudioOutputSample *playSample(const QString &filename, bool loop = false);
 		void run() = 0;
+		virtual bool isAlive() const;
 		const float *getSpeakerPos(unsigned int &nspeakers);
 		static float calcGain(float dotproduct, float distance);
+		unsigned int getMixerFreq() const;
 };
 
 #else
