@@ -51,6 +51,7 @@ bool CompletablePage::isComplete() const {
 AudioWizard::AudioWizard(QWidget *p) : QWizard(p) {
 	bInit = true;
 	bLastActive = false;
+	g.bInAudioWizard = true;
 
 	ticker = new QTimer(this);
 	ticker->setObjectName(QLatin1String("Ticker"));
@@ -94,6 +95,7 @@ AudioWizard::AudioWizard(QWidget *p) : QWizard(p) {
 	}
 
 	qcbHighContrast->setChecked(g.s.bHighContrast);
+	on_qcbHighContrast_clicked(g.s.bHighContrast);
 #ifdef Q_OS_WIN
 	// On windows we can autodetect this
 	qcbHighContrast->setVisible(false);
@@ -114,11 +116,11 @@ AudioWizard::AudioWizard(QWidget *p) : QWizard(p) {
 		iMessage |= (g.s.qmMessages[i] & (Settings::LogSoundfile | Settings::LogTTS));
 	}
 
-	if (iMessage == Settings::LogTTS)
+	if (iMessage == Settings::LogTTS && g.s.bTTS)
 		qrbNotificationTTS->setChecked(true);
 	else if (iMessage == Settings::LogSoundfile)
 		qrbNotificationSounds->setChecked(true);
-	else
+	else // If we find mixed message types or only tts with main tts disable assume custom
 		qrbNotificationCustom->setChecked(true);
 
 	qrbNotificationCustom->setVisible(qrbNotificationCustom->isChecked());
@@ -155,7 +157,7 @@ AudioWizard::AudioWizard(QWidget *p) : QWizard(p) {
 	abVAD->qcInside = Qt::yellow;
 	abVAD->qcAbove = Qt::green;
 
-	qsVAD->setValue(static_cast<int>(g.s.fVADmax * 32767.f));
+	qsVAD->setValue(iroundf(g.s.fVADmax * 32767.f + 0.5f));
 
 	// Positional
 	qcbHeadphone->setChecked(g.s.bPositionalHeadphone);
@@ -243,12 +245,7 @@ void AudioWizard::on_qcbInputDevice_activated(int) {
 	if (! AudioInputRegistrar::qmNew)
 		return;
 
-	AudioInputPtr ai = g.ai;
-	g.ai.reset();
-
-	while (! ai.unique()) {
-	}
-	ai.reset();
+	Audio::stopInput();
 
 	AudioInputRegistrar *air = AudioInputRegistrar::qmNew->value(qcbInput->currentText());
 	int idx = qcbInputDevice->currentIndex();
@@ -289,12 +286,7 @@ void AudioWizard::on_qcbOutputDevice_activated(int) {
 	if (! AudioOutputRegistrar::qmNew)
 		return;
 
-	AudioOutputPtr ao = g.ao;
-	g.ao.reset();
-
-	while (! ao.unique()) {
-	}
-	ao.reset();
+	Audio::stopOutput();
 
 	AudioOutputRegistrar *aor = AudioOutputRegistrar::qmNew->value(qcbOutput->currentText());
 	int idx = qcbOutputDevice->currentIndex();
@@ -385,32 +377,14 @@ void AudioWizard::playChord() {
 }
 
 void AudioWizard::restartAudio() {
-	AudioInputPtr ai = g.ai;
-	AudioOutputPtr ao = g.ao;
-
 	aosSource = NULL;
 
-	g.ai.reset();
-	g.ao.reset();
-
-	while (! ai.unique() || ! ao.unique()) {
-#if QT_VERSION >= 0x040500
-		QThread::yieldCurrentThread();
-#endif
-	}
-
-	ai.reset();
-	ao.reset();
+	Audio::stop();
 
 	g.s.qsAudioInput = qcbInput->currentText();
 	g.s.qsAudioOutput = qcbOutput->currentText();
 
-	g.ai = AudioInputRegistrar::newFromChoice(g.s.qsAudioInput);
-	if (g.ai)
-		g.ai->start(QThread::HighestPriority);
-	g.ao = AudioOutputRegistrar::newFromChoice(g.s.qsAudioOutput);
-	if (g.ao)
-		g.ao->start(QThread::HighPriority);
+	Audio::start();
 
 	if (qgsScene) {
 		delete qgsScene;
@@ -432,6 +406,7 @@ void AudioWizard::reject() {
 	if (ao)
 		ao->wipe();
 	aosSource = NULL;
+	g.bInAudioWizard = false;
 
 	QWizard::reject();
 }
@@ -449,15 +424,24 @@ void AudioWizard::accept() {
 	g.s.lmLoopMode = Settings::None;
 
 	// Switch TTS<->Sounds according to user selection
-	Settings::MessageLog mlReplace = qrbNotificationTTS->isChecked() ? Settings::LogSoundfile : Settings::LogTTS;
-	for (int i = Log::firstMsgType;i <= Log::lastMsgType; ++i) {
-		if (g.s.qmMessages[i] & mlReplace)
-			g.s.qmMessages[i] ^= Settings::LogSoundfile | Settings::LogTTS;
+	if (!qrbNotificationCustom->isChecked()) {
+		Settings::MessageLog mlReplace = qrbNotificationTTS->isChecked() ? Settings::LogSoundfile : Settings::LogTTS;
+
+		for (int i = Log::firstMsgType;i <= Log::lastMsgType; ++i) {
+			if (g.s.qmMessages[i] & mlReplace)
+				g.s.qmMessages[i] ^= Settings::LogSoundfile | Settings::LogTTS;
+		}
+
+		if (qrbNotificationTTS->isChecked()) {
+			g.s.bTTS = true;
+			g.mw->qaAudioTTS->setChecked(true);
+		}
 	}
 
 	g.s.bUsage = qcbUsage->isChecked();
 	g.bPosTest = false;
 	restartAudio();
+	g.bInAudioWizard = false;
 	QWizard::accept();
 }
 
@@ -489,13 +473,13 @@ void AudioWizard::on_Ticker_timeout() {
 	abAmplify->iPeak = iMaxPeak;
 	abAmplify->update();
 
-	abVAD->iBelow = static_cast<int>(g.s.fVADmin * 32767.0f);
-	abVAD->iAbove = static_cast<int>(g.s.fVADmax * 32767.0f);
+	abVAD->iBelow = iroundf(g.s.fVADmin * 32767.0f + 0.5f);
+	abVAD->iAbove = iroundf(g.s.fVADmax * 32767.0f + 0.5f);
 
 	if (g.s.vsVAD == Settings::Amplitude) {
-		abVAD->iValue = iroundf((32767.f/96.0f) * (96.0f + ai->dPeakMic));
+		abVAD->iValue = iroundf((32767.f/96.0f) * (96.0f + ai->dPeakCleanMic) + 0.5f);
 	} else {
-		abVAD->iValue = static_cast<int>(ai->fSpeechProb * 32767.0f);
+		abVAD->iValue = iroundf(ai->fSpeechProb * 32767.0f + 0.5f);
 	}
 	abVAD->update();
 
@@ -577,7 +561,7 @@ void AudioWizard::on_skwPTT_keySet(bool valid, bool last) {
 	if (valid)
 		qrPTT->setChecked(true);
 	else if (qrPTT->isChecked())
-		qrSNR->setChecked(true);
+		qrAmplitude->setChecked(true);
 	updateTriggerWidgets(valid);
 	bTransmitChanged = true;
 
@@ -635,8 +619,17 @@ void AudioWizard::on_qcbAttenuateOthers_clicked(bool checked) {
 	g.s.bAttenuateOthers = checked;
 }
 
-void AudioWizard::on_qcbHighContrast_clicked(bool checked) {
-	g.s.bHighContrast = checked;
+void AudioWizard::on_qcbHighContrast_clicked(bool on) {
+	g.s.bHighContrast = on;
+
+	qliAmpTuningText->setVisible(!g.s.bHighContrast);
+	qliAmpTuningTextHC->setVisible(g.s.bHighContrast);
+
+	qliVolumeTuningText->setVisible(!g.s.bHighContrast);
+	qliVolumeTuningTextHC->setVisible(g.s.bHighContrast);
+
+	qliVadTuningText->setVisible(!g.s.bHighContrast);
+	qliVadTuningTextHC->setVisible(g.s.bHighContrast);
 }
 
 void AudioWizard::on_qrbQualityLow_clicked() {
