@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -31,6 +31,9 @@
 #include "ConfigDialog.h"
 #include "AudioInput.h"
 #include "AudioOutput.h"
+#ifndef COMPAT_CLIENT
+#include "Overlay.h"
+#endif
 #include "Global.h"
 
 ConfigDialog::ConfigDialog(QWidget *p) : QDialog(p) {
@@ -38,17 +41,14 @@ ConfigDialog::ConfigDialog(QWidget *p) : QDialog(p) {
 
 	s = g.s;
 
-	QWidget *w;
-	while ((w = qtwWidgets->widget(0)))
-		delete w;
-
 	unsigned int idx = 0;
 	ConfigWidgetNew cwn;
 	foreach(cwn, *ConfigRegistrar::c_qmNew) {
 		addPage(cwn(s), ++idx);
 	}
 
-	updateExpert(g.s.bExpert);
+	qcbExpert->setChecked(g.s.bExpert);
+	on_qcbExpert_clicked(g.s.bExpert);
 
 	QPushButton *okButton = dialogButtonBox->button(QDialogButtonBox::Ok);
 	okButton->setToolTip(tr("Accept changes"));
@@ -70,12 +70,16 @@ ConfigDialog::ConfigDialog(QWidget *p) : QDialog(p) {
 
 	QPushButton *restoreButton = pageButtonBox->button(QDialogButtonBox::RestoreDefaults);
 	restoreButton->setToolTip(tr("Restore defaults for current page"));
-	restoreButton->setWhatsThis(tr("This button will restore the settings for the current page only to their defaults. Other pages will not be changed.<br />"
+	restoreButton->setWhatsThis(tr("This button will restore the defaults for the settings on the current page. Other pages will not be changed.<br />"
 	                               "To restore all settings to their defaults, you will have to use this button on every page."
 	                              ));
 
-	if (! g.s.qbaConfigGeometry.isEmpty())
-		restoreGeometry(g.s.qbaConfigGeometry);
+	if (! g.s.qbaConfigGeometry.isEmpty()) {
+#ifndef COMPAT_CLIENT
+		if (! g.ocIntercept)
+#endif
+			restoreGeometry(g.s.qbaConfigGeometry);
+	}
 }
 
 void ConfigDialog::addPage(ConfigWidget *cw, unsigned int idx) {
@@ -99,11 +103,14 @@ void ConfigDialog::addPage(ConfigWidget *cw, unsigned int idx) {
 	ms.rheight() += 192;
 	if ((ms.width() > w) || (ms.height() > h)) {
 		QScrollArea *qsa=new QScrollArea();
+		qsa->setFrameShape(QFrame::NoFrame);
 		qsa->setWidgetResizable(true);
 		qsa->setWidget(cw);
 		qhPages.insert(cw, qsa);
+		qswPages->addWidget(qsa);
 	} else {
 		qhPages.insert(cw, cw);
+		qswPages->addWidget(cw);
 	}
 	qmWidgets.insert(idx, cw);
 	cw->load(g.s);
@@ -115,17 +122,22 @@ ConfigDialog::~ConfigDialog() {
 }
 
 void ConfigDialog::on_pageButtonBox_clicked(QAbstractButton *b) {
-	ConfigWidget *conf = qobject_cast<ConfigWidget *>(qtwWidgets->currentWidget());
+	ConfigWidget *conf = qobject_cast<ConfigWidget *>(qswPages->currentWidget());
+	if (! conf) {
+		QScrollArea *qsa = qobject_cast<QScrollArea *>(qswPages->currentWidget());
+		if (qsa)
+			conf = qobject_cast<ConfigWidget *>(qsa->widget());
+	}
+	if (! conf)
+		return;
 	switch (pageButtonBox->standardButton(b)) {
 		case QDialogButtonBox::RestoreDefaults: {
 				Settings def;
-				if (conf)
-					conf->load(def);
+				conf->load(def);
 				break;
 			}
 		case QDialogButtonBox::Reset: {
-				if (conf)
-					conf->load(g.s);
+				conf->load(g.s);
 				break;
 			}
 		default:
@@ -144,59 +156,83 @@ void ConfigDialog::on_dialogButtonBox_clicked(QAbstractButton *b) {
 	}
 }
 
-void ConfigDialog::updateExpert(bool b) {
-	QWidget *ccw = qtwWidgets->currentWidget();
+void ConfigDialog::on_qlwIcons_currentItemChanged(QListWidgetItem *current, QListWidgetItem *previous) {
+	if (!current)
+		current = previous;
+
+	if (current) {
+		QWidget *w = qhPages.value(qmIconWidgets.value(current));
+		if (w)
+			qswPages->setCurrentWidget(w);
+	}
+}
+
+void ConfigDialog::on_qcbExpert_clicked(bool b) {
+	QWidget *ccw = qmIconWidgets.value(qlwIcons->currentItem());
+	QListWidgetItem *sel = NULL;
+
+	qmIconWidgets.clear();
+	qlwIcons->clear();
+
+	QFontMetrics qfm(qlwIcons->font());
+	int width = 0;
 
-	bool found = false;
-	qtwWidgets->clear();
 	foreach(ConfigWidget *cw, qmWidgets) {
 		bool showit = cw->expert(b);
+		width = qMax(width, qfm.width(cw->title()));
 		if (showit || b)  {
-			QWidget *w = qhPages.value(cw);
-			qtwWidgets->addTab(w, cw->icon(), cw->title());
+			QListWidgetItem *i = new QListWidgetItem(qlwIcons);
+			i->setIcon(cw->icon());
+			i->setText(cw->title());
+			i->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
 
-			if (w == ccw)
-				found = true;
+
+			qmIconWidgets.insert(i, cw);
+			if (cw == ccw)
+				sel = i;
 		}
 	}
-	if (found)
-		qtwWidgets->setCurrentWidget(ccw);
+
+	// Add space for icon and some padding.
+	width += qlwIcons->iconSize().width() + 25;
+
+	qlwIcons->setMinimumWidth(width);
+	qlwIcons->setMaximumWidth(width);
+
+	if (sel)
+		qlwIcons->setCurrentItem(sel);
+	else
+		qlwIcons->setCurrentRow(0);
 }
 
 void ConfigDialog::apply() {
+	Audio::stop();
+
 	foreach(ConfigWidget *cw, qmWidgets)
 		cw->save();
 
-	AudioInputPtr ai = g.ai;
-	AudioOutputPtr ao = g.ao;
-
-	g.ai.reset();
-	g.ao.reset();
-
-	while (! ai.unique() || ! ao.unique()) {
-		// Where is QThread::yield() ?
-	}
-	ai.reset();
-	ao.reset();
-
 	g.s = s;
 
 	foreach(ConfigWidget *cw, qmWidgets)
 		cw->accept();
 
-	g.ai = AudioInputRegistrar::newFromChoice(g.s.qsAudioInput);
-	if (g.ai)
-		g.ai->start(QThread::HighestPriority);
-	g.ao = AudioOutputRegistrar::newFromChoice(g.s.qsAudioOutput);
-	if (g.ao)
-		g.ao->start(QThread::HighPriority);
+	if (!g.s.bAttenuateOthersOnTalk)
+		g.bAttenuateOthers = false;
 
 	// They might have changed their keys.
 	g.iPushToTalk = 0;
+	g.s.bExpert = qcbExpert->isChecked();
+
+	Audio::start();
 }
 
 void ConfigDialog::accept() {
 	apply();
-	g.s.qbaConfigGeometry=saveGeometry();
+
+#ifndef COMPAT_CLIENT
+	if (! g.ocIntercept)
+#endif
+		g.s.qbaConfigGeometry=saveGeometry();
+
 	QDialog::accept();
 }
