@@ -56,7 +56,7 @@ struct PublicInfo {
 
 struct PingStats {
 private:
-	Q_DISABLE_COPY(PingStats);
+	Q_DISABLE_COPY(PingStats)
 protected:
 	void init();
 public:
@@ -83,8 +83,8 @@ public:
 class ServerItem;
 
 class ServerView : public QTreeWidget {
-		Q_OBJECT;
-		Q_DISABLE_COPY(ServerView);
+		Q_OBJECT
+		Q_DISABLE_COPY(ServerView)
 	public:
 		ServerItem *siFavorite, *siLAN, *siPublic;
 		QMap<QString, QString> qmContinentNames;
@@ -108,7 +108,7 @@ class ServerView : public QTreeWidget {
 #include "ui_ConnectDialogEdit.h"
 
 class ServerItem : public QTreeWidgetItem, public PingStats {
-		Q_DISABLE_COPY(ServerItem);
+		Q_DISABLE_COPY(ServerItem)
 	protected:
 		void init();
 	public:
@@ -144,7 +144,7 @@ class ServerItem : public QTreeWidgetItem, public PingStats {
 
 		ServerItem(const FavoriteServer &fs);
 		ServerItem(const PublicInfo &pi);
-		ServerItem(const QString &name, const QString &host, unsigned short port, const QString &uname);
+		ServerItem(const QString &name, const QString &host, unsigned short port, const QString &uname, const QString &password = QString());
 		ServerItem(const BonjourRecord &br);
 		ServerItem(const QString &name, ItemType itype, const QString &continent = QString(), const QString &country = QString());
 		ServerItem(const ServerItem *si);
@@ -175,18 +175,18 @@ class ServerItem : public QTreeWidgetItem, public PingStats {
 class ConnectDialogEdit : public QDialog, protected Ui::ConnectDialogEdit {
 	private:
 		Q_OBJECT
-		Q_DISABLE_COPY(ConnectDialogEdit);
+		Q_DISABLE_COPY(ConnectDialogEdit)
 	protected:
 		bool bOk;
 	public slots:
 		void validate();
 		void accept();
 
-		void on_qleServer_textChanged(const QString&);
+		void on_qcbShowPassword_toggled(bool);
 	public:
-		QString qsName, qsHostname, qsUsername;
+		QString qsName, qsHostname, qsUsername, qsPassword;
 		unsigned short usPort;
-		ConnectDialogEdit(QWidget *parent, const QString &name = QString(), const QString &host = QString(), const QString &user = QString(), unsigned short port = 64738);
+		ConnectDialogEdit(QWidget *parent, const QString &name = QString(), const QString &host = QString(), const QString &user = QString(), unsigned short port = 64738, const QString &password = QString(), bool add = false);
 };
 
 class ConnectDialog : public QDialog, public Ui::ConnectDialog {
@@ -203,6 +203,7 @@ class ConnectDialog : public QDialog, public Ui::ConnectDialog {
 		QActionGroup *qagFilters;
 
 		bool bPublicInit;
+		bool bAutoConnect;
 
 		Timer tPing;
 		Timer tCurrent, tHover, tRestart;
@@ -211,6 +212,8 @@ class ConnectDialog : public QDialog, public Ui::ConnectDialog {
 		QTimer *qtPingTick;
 		QList<ServerItem *> qlItems;
 
+		ServerItem *siAutoConnect;
+
 		QList<QString> qlDNSLookup;
 		QSet<QString> qsDNSActive;
 		QHash<QString, QSet<ServerItem *> > qhDNSWait;
@@ -260,7 +263,7 @@ class ConnectDialog : public QDialog, public Ui::ConnectDialog {
 	public:
 		QString qsServer, qsUsername, qsPassword;
 		unsigned short usPort;
-		ConnectDialog(QWidget *parent);
+		ConnectDialog(QWidget *parent, bool autoconnect);
 		~ConnectDialog();
 
 #ifdef USE_BONJOUR
