@@ -33,17 +33,22 @@
 
 #include "mumble_pch.hpp"
 
-class ChatbarLineEdit : public QLineEdit {
+class ChatbarTextEdit : public QTextEdit {
 	private:
 		Q_OBJECT
-		Q_DISABLE_COPY(ChatbarLineEdit)
+		Q_DISABLE_COPY(ChatbarTextEdit)
+		void inFocus(bool);
 	protected:
 		QString qsDefaultText;
 		bool bDefaultVisible;
 		void focusInEvent(QFocusEvent *);
 		void focusOutEvent(QFocusEvent *);
 		void contextMenuEvent(QContextMenuEvent *);
+		void dropEvent(QDropEvent *);
 		bool event(QEvent *);
+		QSize minimumSizeHint() const;
+		QSize sizeHint() const;
+		void resizeEvent(QResizeEvent *e);
 	public:
 		void setDefaultText(const QString &, bool = false);
 		unsigned int completeAtCursor();
@@ -52,11 +57,13 @@ class ChatbarLineEdit : public QLineEdit {
 		void ctrlSpacePressed(void);
 	public slots:
 		void pasteAndSend_triggered();
+		void doResize();
+		void doScrollbar();
 	public:
-		ChatbarLineEdit(QWidget *p = NULL);
+		ChatbarTextEdit(QWidget *p = NULL);
 };
 
-class DockTitleBar : public QWidget {
+class DockTitleBar : public QLabel {
 	private:
 		Q_OBJECT
 		Q_DISABLE_COPY(DockTitleBar)
