@@ -45,7 +45,7 @@ class GlobalShortcut : public QObject {
 		Q_OBJECT
 		Q_DISABLE_COPY(GlobalShortcut)
 	protected:
-		bool bActive;
+		QList<QVariant> qlActive;
 	signals:
 		void down(QVariant);
 		void triggered(bool, QVariant);
@@ -61,7 +61,7 @@ class GlobalShortcut : public QObject {
 		~GlobalShortcut();
 
 		bool active() const {
-			return bActive;
+			return ! qlActive.isEmpty();
 		};
 };
 
@@ -113,7 +113,7 @@ class ShortcutToggleWidget : public QComboBox {
 class ShortcutTargetDialog : public QDialog, public Ui::GlobalShortcutTarget {
 	private:
 		Q_OBJECT
-		Q_DISABLE_COPY(ShortcutTargetDialog);
+		Q_DISABLE_COPY(ShortcutTargetDialog)
 	protected:
 		QMap<QString, QString> qmHashNames;
 		ShortcutTarget stTarget;
@@ -219,6 +219,8 @@ class GlobalShortcutEngine : public QThread {
 		static QString buttonText(const QList<QVariant> &);
 		virtual QString buttonName(const QVariant &) = 0;
 		virtual bool canSuppress();
+
+		virtual void prepareInput();
 	signals:
 		void buttonPressed(bool last);
 };
