@@ -34,8 +34,10 @@
 #include "MainWindow.h"
 #include "Channel.h"
 #include "ServerHandler.h"
+#include "NetworkConfig.h"
 #include "Global.h"
 
+
 #ifdef Q_OS_MAC
 extern bool qt_mac_execute_apple_script(const QString &script, AEDesc *ret);
 
@@ -223,6 +225,7 @@ const char *Log::msgNames[] = {
 	QT_TRANSLATE_NOOP("Log", "Server Disconnected"),
 	QT_TRANSLATE_NOOP("Log", "User Joined Server"),
 	QT_TRANSLATE_NOOP("Log", "User Left Server"),
+	QT_TRANSLATE_NOOP("Log", "User recording state changed"),
 	QT_TRANSLATE_NOOP("Log", "User kicked (you or by you)"),
 	QT_TRANSLATE_NOOP("Log", "User kicked"),
 	QT_TRANSLATE_NOOP("Log", "You self-muted/deafened"),
@@ -255,6 +258,8 @@ const QStringList Log::allowedSchemes() {
 	qslAllowedSchemeNames << QLatin1String("clientid");
 	qslAllowedSchemeNames << QLatin1String("channelid");
 	qslAllowedSchemeNames << QLatin1String("spotify");
+	qslAllowedSchemeNames << QLatin1String("steam");
+	qslAllowedSchemeNames << QLatin1String("irc");
 	return qslAllowedSchemeNames;
 }
 
@@ -385,7 +390,7 @@ QString Log::validHtml(const QString &html, bool allowReplacement, QTextCursor *
 			if (qcf.isImageFormat()) {
 				QTextImageFormat qtif = qcf.toImageFormat();
 				QUrl url(qtif.name());
-				if (! url.isValid())
+				if (! qtif.name().isEmpty() && ! url.isValid())
 					valid = false;
 			}
 		}
@@ -420,7 +425,7 @@ QString Log::validHtml(const QString &html, bool allowReplacement, QTextCursor *
 	}
 }
 
-void Log::log(MsgType mt, const QString &console, const QString &terse) {
+void Log::log(MsgType mt, const QString &console, const QString &terse, bool console_only) {
 	QDateTime dt = QDateTime::currentDateTime();
 
 	int ignore = qmIgnore.value(mt);
@@ -462,6 +467,9 @@ void Log::log(MsgType mt, const QString &console, const QString &terse) {
 		g.mw->qteLog->ensureCursorVisible();
 	}
 
+	if (console_only)
+		return;
+
 	// Message notification with balloon tooltips
 	if ((flags & Settings::LogBalloon) && !(g.mw->isActiveWindow() && g.mw->qdwLog->isVisible()))  {
 		QString qsIcon;
@@ -565,19 +573,25 @@ void Log::log(MsgType mt, const QString &console, const QString &terse) {
 		return;
 
 	// Apply simplifications to spoken text
-	QRegExp identifyURL(QLatin1String("\\b(([\\w-]+://?|www[.])[^\\s()<>]+(?:\\([\\w\\d]+\\)|([^!\"#$%&'()*+,-./:;<=>?@\\[\\\\\\]^_`{|}~\\s]|/)))"));
+	QRegExp identifyURL(QLatin1String("[a-z-]+://[^ <$]*"),
+	                    Qt::CaseInsensitive,
+	                    QRegExp::RegExp2);
+
 	QStringList qslAllowed = allowedSchemes();
+
 	int pos = 0;
 	while ((pos = identifyURL.indexIn(plain, pos)) != -1) {
-		QUrl url(identifyURL.cap(0));
+		QUrl url(identifyURL.cap(0).toLower());
 		int len = identifyURL.matchedLength();
 		if (url.isValid() && qslAllowed.contains(url.scheme())) {
 			// Replace it appropriatly
 			QString replacement;
+			QString host = url.host().replace(QRegExp(QLatin1String("^www.")), QString());
+
 			if (url.scheme() == QLatin1String("http") || url.scheme() == QLatin1String("https"))
-				replacement = tr("link to %1").arg(url.host());
+				replacement = tr("link to %1").arg(host);
 			else if (url.scheme() == QLatin1String("ftp"))
-				replacement = tr("ftp link to %1").arg(url.host());
+				replacement = tr("ftp link to %1").arg(host);
 			else if (url.scheme() == QLatin1String("clientid"))
 				replacement = tr("player link");
 			else if (url.scheme() == QLatin1String("channelid"))
@@ -645,8 +659,7 @@ QVariant LogDocument::loadResource(int type, const QUrl &url) {
 	if ((url.scheme() != QLatin1String("http")) && (url.scheme() != QLatin1String("https")))
 		return qi;
 
-	QNetworkRequest req(url);
-	QNetworkReply *rep = g.nam->get(req);
+	QNetworkReply *rep = Network::get(url);
 	connect(rep, SIGNAL(metaDataChanged()), this, SLOT(receivedHead()));
 	connect(rep, SIGNAL(finished()), this, SLOT(finished()));
 	return qi;
