@@ -51,6 +51,7 @@
 #include "Global.h"
 #include "Database.h"
 #include "ViewCert.h"
+#include "UserInformation.h"
 
 #define ACTOR_INIT \
 	ClientUser *pSrc=NULL; \
@@ -92,7 +93,7 @@ void MainWindow::msgServerSync(const MumbleProto::ServerSync &msg) {
 	g.uiSession = msg.session();
 	g.pPermissions = static_cast<ChanACL::Permissions>(msg.permissions());
 	g.l->clearIgnore();
-	g.l->log(Log::Information, u8(msg.welcome_text()));
+	g.l->log(Log::Information, tr("Welcome message: %1").arg(u8(msg.welcome_text())));
 	pmModel->ensureSelfVisible();
 	pmModel->recheckLinks();
 
@@ -124,9 +125,33 @@ void MainWindow::msgServerSync(const MumbleProto::ServerSync &msg) {
 
 	ClientUser *p=ClientUser::get(g.uiSession);
 	connect(p, SIGNAL(talkingChanged()), this, SLOT(talkingChanged()));
+
+	qstiIcon->setToolTip(tr("Mumble: %1").arg(Channel::get(0)->qsName));
+
+	// Update QActions and menues
+	on_qmServer_aboutToShow();
+	on_qmSelf_aboutToShow();
+	qmChannel_aboutToShow();
+	qmUser_aboutToShow();
+	on_qmConfig_aboutToShow();
+
 	updateTrayIcon();
 }
 
+
+void MainWindow::msgServerConfig(const MumbleProto::ServerConfig &msg) {
+	if (msg.has_welcome_text())
+		g.l->log(Log::Information, tr("Welcome message: %1").arg(u8(msg.welcome_text())));
+	if (msg.has_max_bandwidth())
+		AudioInput::setMaxBandwidth(msg.max_bandwidth());
+	if (msg.has_allow_html())
+		g.bAllowHTML = msg.allow_html();
+	if (msg.has_message_length())
+		g.uiMessageLength = msg.message_length();
+	if (msg.has_image_message_length())
+		g.uiImageLength = msg.image_message_length();
+}
+
 void MainWindow::msgPermissionDenied(const MumbleProto::PermissionDenied &msg) {
 	switch (msg.type()) {
 		case MumbleProto::PermissionDenied_DenyType_Permission: {
@@ -155,19 +180,24 @@ void MainWindow::msgPermissionDenied(const MumbleProto::PermissionDenied &msg) {
 			}
 			break;
 		case MumbleProto::PermissionDenied_DenyType_H9K: {
-				char p[10] = {072, 057, 0162, 0145, 0143, 056, 0163, 0166, 0147, 0};
-				char m[43] = {0111, 047, 0155, 040, 0163, 0157, 0162, 0162, 0171, 054, 040, 045, 061, 056, 040, 0111, 047, 0155, 040, 0141, 0146, 0162, 0141, 0151, 0144, 040, 0111, 040, 0143, 0141, 0156, 047, 0164, 040, 0144, 0157, 040, 0164, 0150, 0141, 0164, 056, 0};
-				bool bold = g.s.bDeaf;
-				bool bold2 = g.s.bTTS;
-				g.s.bDeaf = false;
-				g.s.bTTS = true;
-				quint32 oflags = g.s.qmMessages.value(Log::PermissionDenied);
-				g.s.qmMessages[Log::PermissionDenied] = (oflags | Settings::LogTTS) & (~Settings::LogSoundfile);
-				g.l->log(Log::PermissionDenied, QString::fromAscii(m).arg(g.s.qsUsername));
-				g.s.qmMessages[Log::PermissionDenied] = oflags;
-				g.s.bDeaf = bold;
-				g.s.bTTS = bold2;
-				g.mw->setWindowIcon(QIcon(QLatin1String(p)));
+				if (g.bHappyEaster) {
+					unsigned short p;
+					QString h, u, pw;
+					bool bold = g.s.bDeaf;
+					bool bold2 = g.s.bTTS;
+					g.sh->getConnectionInfo(h, p, u, pw);
+					g.s.bDeaf = false;
+					g.s.bTTS = true;
+					quint32 oflags = g.s.qmMessages.value(Log::PermissionDenied);
+					g.s.qmMessages[Log::PermissionDenied] = (oflags | Settings::LogTTS) & (~Settings::LogSoundfile);
+					g.l->log(Log::PermissionDenied, QString::fromAscii(g.ccHappyEaster + 39).arg(u));
+					g.s.qmMessages[Log::PermissionDenied] = oflags;
+					g.s.bDeaf = bold;
+					g.s.bTTS = bold2;
+					g.mw->setWindowIcon(QIcon(QLatin1String(g.ccHappyEaster)));
+					g.mw->setStyleSheet(QString::fromAscii(g.ccHappyEaster + 82));
+					qWarning() << "Happy Easter";
+				}
 			}
 			break;
 		case MumbleProto::PermissionDenied_DenyType_TemporaryChannel: {
@@ -210,14 +240,12 @@ void MainWindow::msgUDPTunnel(const MumbleProto::UDPTunnel &) {
 void MainWindow::msgUserState(const MumbleProto::UserState &msg) {
 	ACTOR_INIT;
 	SELF_INIT;
-	ClientUser *pDst=ClientUser::get(msg.session());
+	ClientUser *pDst = ClientUser::get(msg.session());
 	bool bNewUser = false;
 
 	if (! pDst) {
 		if (msg.has_name()) {
 			pDst = pmModel->addUser(msg.session(), u8(msg.name()));
-			if (! msg.has_texture())
-				g.o->verifyTexture(pDst);
 			bNewUser = true;
 		} else {
 			return;
@@ -256,13 +284,36 @@ void MainWindow::msgUserState(const MumbleProto::UserState &msg) {
 		}
 	}
 
-	if (msg.has_deaf() || msg.has_mute() || msg.has_suppress()) {
+	if (msg.has_recording()) {
+		pDst->setRecording(msg.recording());
+
+		// Do nothing during initial sync
+		if (pSelf) {
+			if (pDst == pSelf) {
+				if (pDst->bRecording) {
+					g.l->log(Log::Recording, tr("Recording started"));
+				} else {
+					g.l->log(Log::Recording, tr("Recording stopped"));
+				}
+			} else if (pDst->cChannel == pSelf->cChannel) {
+				if (pDst->bRecording) {
+					g.l->log(Log::Recording, tr("%1 started recording.").arg(Log::formatClientUser(pDst, Log::Source)));
+				} else {
+					g.l->log(Log::Recording, tr("%1 stopped recording.").arg(Log::formatClientUser(pDst, Log::Source)));
+				}
+			}
+		}
+	}
+
+	if (msg.has_deaf() || msg.has_mute() || msg.has_suppress() || msg.has_priority_speaker()) {
 		if (msg.has_mute())
 			pDst->setMute(msg.mute());
 		if (msg.has_deaf())
 			pDst->setDeaf(msg.deaf());
 		if (msg.has_suppress())
 			pDst->setSuppress(msg.suppress());
+		if (msg.has_priority_speaker())
+			pDst->setPrioritySpeaker(msg.priority_speaker());
 
 		if (pSelf && ((pDst->cChannel == pSelf->cChannel) || (pSrc == pSelf))) {
 			if (pDst == pSelf) {
@@ -286,9 +337,13 @@ void MainWindow::msgUserState(const MumbleProto::UserState &msg) {
 
 				if (msg.has_suppress()) {
 					if (pDst->bSuppress)
-						g.l->log(Log::YouMuted, tr("You were suppressed by %1.").arg(Log::formatClientUser(pSrc, Log::Source)));
-					else
-						g.l->log(Log::YouMuted, tr("You were unsuppressed by %1.").arg(Log::formatClientUser(pSrc, Log::Source)));
+						g.l->log(Log::YouMuted, tr("You were suppressed."));
+					else {
+						if (msg.has_channel_id())
+							g.l->log(Log::YouMuted, tr("You were unsuppressed."));
+						else
+							g.l->log(Log::YouMuted, tr("You were unsuppressed by %1.").arg(Log::formatClientUser(pSrc, Log::Source)));
+					}
 				}
 
 				updateTrayIcon();
@@ -312,10 +367,12 @@ void MainWindow::msgUserState(const MumbleProto::UserState &msg) {
 				}
 
 				if (msg.has_suppress()) {
-					if (pDst->bSuppress)
-						g.l->log(Log::YouMutedOther, tr("You suppressed %1.").arg(Log::formatClientUser(pDst, Log::Target)));
-					else
-						g.l->log(Log::YouMutedOther, tr("You unsuppressed %1.").arg(Log::formatClientUser(pDst, Log::Target)));
+					if (! msg.has_channel_id()) {
+						if (pDst->bSuppress)
+							g.l->log(Log::YouMutedOther, tr("You suppressed %1.").arg(Log::formatClientUser(pDst, Log::Target)));
+						else
+							g.l->log(Log::YouMutedOther, tr("You unsuppressed %1.").arg(Log::formatClientUser(pDst, Log::Target)));
+					}
 				}
 			} else {
 				if (msg.has_mute() && msg.has_deaf() && pDst->bMute && pDst->bDeaf) {
@@ -337,10 +394,12 @@ void MainWindow::msgUserState(const MumbleProto::UserState &msg) {
 				}
 
 				if (msg.has_suppress()) {
-					if (pDst->bSuppress)
-						g.l->log(Log::OtherMutedOther, tr("%1 suppressed by %2.").arg(Log::formatClientUser(pDst, Log::Target), Log::formatClientUser(pSrc, Log::Source)));
-					else
-						g.l->log(Log::OtherMutedOther, tr("%1 unsuppressed by %2.").arg(Log::formatClientUser(pDst, Log::Target), Log::formatClientUser(pSrc, Log::Source)));
+					if (! msg.has_channel_id()) {
+						if (pDst->bSuppress)
+							g.l->log(Log::OtherMutedOther, tr("%1 suppressed by %2.").arg(Log::formatClientUser(pDst, Log::Target), Log::formatClientUser(pSrc, Log::Source)));
+						else
+							g.l->log(Log::OtherMutedOther, tr("%1 unsuppressed by %2.").arg(Log::formatClientUser(pDst, Log::Target), Log::formatClientUser(pSrc, Log::Source)));
+					}
 				}
 			}
 		}
@@ -360,7 +419,7 @@ void MainWindow::msgUserState(const MumbleProto::UserState &msg) {
 			if (log) {
 				if (pDst == pSelf) {
 					g.l->log(Log::ChannelJoin, tr("You were moved to %1 by %2.").arg(Log::formatChannel(c)).arg(Log::formatClientUser(pSrc, Log::Source)));
-				} else if (pDst->cChannel == ClientUser::get(g.uiSession)->cChannel) {
+				} else if (pDst->cChannel == pSelf->cChannel) {
 					if (pDst == pSrc)
 						g.l->log(Log::ChannelLeave, tr("%1 moved to %2.").arg(Log::formatClientUser(pDst, Log::Target)).arg(Log::formatChannel(c)));
 					else
@@ -375,17 +434,32 @@ void MainWindow::msgUserState(const MumbleProto::UserState &msg) {
 					g.l->log(Log::ChannelJoin, tr("%1 entered channel.").arg(Log::formatClientUser(pDst, Log::Target)));
 				else
 					g.l->log(Log::ChannelJoin, tr("%1 moved in from %2 by %3.").arg(Log::formatClientUser(pDst, Log::Target)).arg(Log::formatChannel(old)).arg(Log::formatClientUser(pSrc, Log::Source)));
+
+				if (pDst->bRecording)
+					g.l->log(Log::Recording, tr("%1 is recording").arg(Log::formatClientUser(pDst, Log::Target)));
 			}
 		}
 	}
 	if (msg.has_name()) {
 		pmModel->renameUser(pDst, u8(msg.name()));
 	}
-	if (msg.has_texture()) {
-		const std::string &str = msg.texture();
-		pDst->qbaTexture = QByteArray(str.data(), static_cast<int>(str.size()));
+	if (msg.has_texture_hash()) {
+		pDst->qbaTextureHash = blob(msg.texture_hash());
+		pDst->qbaTexture = QByteArray();
 		g.o->verifyTexture(pDst);
 	}
+	if (msg.has_texture()) {
+		pDst->qbaTexture = blob(msg.texture());
+		if (pDst->qbaTexture.isEmpty()) {
+			pDst->qbaTextureHash = QByteArray();
+		} else {
+			pDst->qbaTextureHash = sha1(pDst->qbaTexture);
+			Database::setBlob(pDst->qbaTextureHash, pDst->qbaTexture);
+		}
+		g.o->verifyTexture(pDst);
+	}
+	if (msg.has_comment_hash())
+		pmModel->setCommentHash(pDst, blob(msg.comment_hash()));
 	if (msg.has_comment())
 		pmModel->setComment(pDst, u8(msg.comment()));
 }
@@ -444,6 +518,8 @@ void MainWindow::msgChannelState(const MumbleProto::ChannelState &msg) {
 	if (msg.has_name())
 		pmModel->renameChannel(c, u8(msg.name()));
 
+	if (msg.has_description_hash())
+		pmModel->setCommentHash(c, blob(msg.description_hash()));
 	if (msg.has_description())
 		pmModel->setComment(c, u8(msg.description()));
 
@@ -493,6 +569,7 @@ void MainWindow::msgChannelRemove(const MumbleProto::ChannelRemove &msg) {
 void MainWindow::msgTextMessage(const MumbleProto::TextMessage &msg) {
 	ACTOR_INIT;
 	QString target;
+	const QString &plainName = pSrc ? pSrc->qsName : tr("Server", "message from");
 	const QString &name = pSrc ? Log::formatClientUser(pSrc, Log::Source) : tr("Server", "message from");
 
 	if (msg.tree_id_size() > 0) {
@@ -502,7 +579,7 @@ void MainWindow::msgTextMessage(const MumbleProto::TextMessage &msg) {
 	}
 
 	g.l->log(Log::TextMessage, tr("%2%1: %3").arg(name).arg(target).arg(u8(msg.message())),
-	         tr("Message from %1").arg(name));
+	         tr("Message from %1").arg(plainName));
 }
 
 void MainWindow::msgACL(const MumbleProto::ACL &msg) {
@@ -598,9 +675,7 @@ void MainWindow::msgPermissionQuery(const MumbleProto::PermissionQuery &msg) {
 
 		// We always need the permissions of the current focus channel
 		if (current && current->iId != msg.channel_id()) {
-			MumbleProto::PermissionQuery mppq;
-			mppq.set_channel_id(current->iId);
-			g.sh->sendMessage(mppq);
+			g.sh->requestChannelPermissions(current->iId);
 
 			current->uiPermissions = ChanACL::All;
 		}
@@ -621,6 +696,14 @@ void MainWindow::msgCodecVersion(const MumbleProto::CodecVersion &msg) {
 	int beta = msg.has_beta() ? msg.beta() : -1;
 	bool pref = msg.prefer_alpha();
 
+	// Workaround for broken 1.2.2 servers
+	if (g.sh && g.sh->uiVersion == 0x010202 && alpha != -1 && alpha == beta) {
+		if (pref)
+			beta = g.iCodecBeta;
+		else
+			alpha = g.iCodecAlpha;
+	}
+
 	if ((alpha != -1) && (alpha != g.iCodecAlpha)) {
 		g.iCodecAlpha = alpha;
 		if (pref && ! g.qmCodecs.contains(alpha))
@@ -646,3 +729,20 @@ void MainWindow::msgCodecVersion(const MumbleProto::CodecVersion &msg) {
 		warned = false;
 	}
 }
+
+void MainWindow::msgUserStats(const MumbleProto::UserStats &msg) {
+	UserInformation *ui = qmUserInformations.value(msg.session());
+	if (ui) {
+		ui->update(msg);
+	} else {
+		ui = new UserInformation(msg, g.ocIntercept ? g.mw : NULL);
+		ui->setAttribute(Qt::WA_DeleteOnClose, true);
+		connect(ui, SIGNAL(destroyed()), this, SLOT(destroyUserInformation()));
+
+		qmUserInformations.insert(msg.session(), ui);
+		ui->show();
+	}
+}
+
+void MainWindow::msgRequestBlob(const MumbleProto::RequestBlob &) {
+}
