@@ -1,5 +1,6 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
    Copyright (C) 2008-2009, Mikkel Krautz <mikkel@krautz.dk>
+   Copyright (C) 2009, Stefan Hacker <dd0t@users.sourceforge.net>
 
    All rights reserved.
 
@@ -32,6 +33,7 @@
 #include "NetworkConfig.h"
 #include "Global.h"
 #include "MainWindow.h"
+#include "OSInfo.h"
 
 static ConfigWidget *NetworkConfigNew(Settings &st) {
 	return new NetworkConfig(st);
@@ -41,6 +43,10 @@ static ConfigRegistrar registrar(1300, NetworkConfigNew);
 
 NetworkConfig::NetworkConfig(Settings &st) : ConfigWidget(st) {
 	setupUi(this);
+#ifdef NO_UPDATE_CHECK
+	qcbAutoUpdate->hide();
+	qcbPluginUpdate->hide();
+#endif
 }
 
 QString NetworkConfig::title() const {
@@ -54,7 +60,9 @@ QIcon NetworkConfig::icon() const {
 void NetworkConfig::load(const Settings &r) {
 
 	loadCheckBox(qcbTcpMode, s.bTCPCompat);
+	loadCheckBox(qcbQoS, s.bQoS);
 	loadCheckBox(qcbAutoReconnect, s.bReconnect);
+	loadCheckBox(qcbAutoConnect, s.bAutoConnect);
 	loadCheckBox(qcbSuppressIdentity, s.bSuppressIdentity);
 	loadComboBox(qcbType, s.ptProxyType);
 
@@ -70,14 +78,23 @@ void NetworkConfig::load(const Settings &r) {
 	qleUsername->setText(r.qsProxyUsername);
 	qlePassword->setText(r.qsProxyPassword);
 
+	loadCheckBox(qcbImageDownload, r.iMaxImageSize <= 0);
+
 	loadCheckBox(qcbAutoUpdate, r.bUpdateCheck);
-	loadCheckBox(qcbPluginUpdate, r.bPluginCheck);
+	loadCheckBox(qcbPluginUpdate, r.bPluginOverlayCheck);
 	loadCheckBox(qcbUsage, r.bUsage);
+
+#if defined(SNAPSHOT_BUILD) && defined(QT_NO_DEBUG)
+	qcbAutoUpdate->setEnabled(false);
+	qcbAutoUpdate->setToolTip(tr("Updates are mandatory when using snapshot releases."));
+#endif
 }
 
 void NetworkConfig::save() const {
 	s.bTCPCompat = qcbTcpMode->isChecked();
+	s.bQoS = qcbQoS->isChecked();
 	s.bReconnect = qcbAutoReconnect->isChecked();
+	s.bAutoConnect = qcbAutoConnect->isChecked();
 	s.bSuppressIdentity = qcbSuppressIdentity->isChecked();
 
 	s.ptProxyType = static_cast<Settings::ProxyType>(qcbType->currentIndex());
@@ -86,8 +103,14 @@ void NetworkConfig::save() const {
 	s.qsProxyUsername = qleUsername->text();
 	s.qsProxyPassword = qlePassword->text();
 
+	if (qcbImageDownload->isChecked()) {
+		s.iMaxImageSize = 0;
+	} else if (s.iMaxImageSize <= 0) {
+		s.iMaxImageSize = s.ciDefaultMaxImageSize;
+	}
+
 	s.bUpdateCheck=qcbAutoUpdate->isChecked();
-	s.bPluginCheck=qcbPluginUpdate->isChecked();
+	s.bPluginOverlayCheck=qcbPluginUpdate->isChecked();
 	s.bUsage=qcbUsage->isChecked();
 }
 
@@ -139,8 +162,14 @@ void NetworkConfig::accept() const {
 
 bool NetworkConfig::expert(bool b) {
 	qcbTcpMode->setVisible(b);
+	qcbQoS->setVisible(b);
 	qgbProxy->setVisible(b);
 	qcbUsage->setVisible(b);
+
+	qgbMisc->setVisible(b); // For now Misc only contains elements visible in expert mode
+	qcbImageDownload->setVisible(b);
+	qcbSuppressIdentity->setVisible(b);
+
 	return true;
 }
 
@@ -155,3 +184,16 @@ void NetworkConfig::on_qcbType_currentIndexChanged(int v) {
 
 	s.ptProxyType = pt;
 }
+
+QNetworkReply *Network::get(const QUrl &url) {
+	QNetworkRequest req(url);
+	prepareRequest(req);
+	return g.nam->get(req);
+}
+
+void Network::prepareRequest(QNetworkRequest &req) {
+#if QT_VERSION >= 0x040600
+	req.setAttribute(QNetworkRequest::HttpPipeliningAllowedAttribute, true);
+#endif
+	req.setRawHeader(QString::fromLatin1("User-Agent").toUtf8(), QString::fromLatin1("Mozilla/5.0 (%1; %2) Mumble/%3 %4").arg(OSInfo::getOS(), OSInfo::getOSVersion(), QLatin1String(MUMTEXT(MUMBLE_VERSION_STRING)), QLatin1String(MUMBLE_RELEASE)).toUtf8());
+}
