@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
    Copyright (C) 2008-2009, Mikkel Krautz <mikkel@krautz.dk>
 
    All rights reserved.
@@ -54,6 +54,11 @@ class NetworkConfig : public ConfigWidget, Ui::NetworkConfig {
 		void on_qcbType_currentIndexChanged(int v);
 };
 
+namespace Network {
+	void prepareRequest(QNetworkRequest &);
+	QNetworkReply *get(const QUrl &);
+}
+
 #else
 class NetworkConfig;
 #endif
