@@ -103,3 +103,34 @@ void Overlay::platformInit() {
 void Overlay::setActive(bool act) {
 	static_cast<OverlayPrivateWin *>(d)->setActive(act);
 }
+
+bool OverlayConfig::supportsInstallableOverlay() {
+	return false;
+}
+
+bool OverlayConfig::isInstalled() {
+	return true;
+}
+
+bool OverlayConfig::needsUpgrade() {
+	return false;
+}
+
+bool OverlayConfig::installFiles() {
+	return false;
+}
+
+bool OverlayConfig::uninstallFiles() {
+	return false;
+}
+
+bool OverlayConfig::supportsCertificates() {
+	return false;
+}
+
+bool OverlayConfig::installerIsValid() {
+	return false;
+}
+
+void OverlayConfig::showCertificates() {
+}
