@@ -33,6 +33,7 @@
 #include "Message.h"
 #include "ServerHandler.h"
 #include "MainWindow.h"
+#include "NetworkConfig.h"
 #include "../../plugins/mumble_plugin.h"
 #include "Global.h"
 
@@ -50,6 +51,7 @@ struct PluginInfo {
 	QString description;
 	QString shortname;
 	MumblePlugin *p;
+	MumblePlugin2 *p2;
 	PluginInfo();
 };
 
@@ -57,6 +59,7 @@ PluginInfo::PluginInfo() {
 	locked = false;
 	enabled = false;
 	p = NULL;
+	p2 = NULL;
 }
 
 PluginConfig::PluginConfig(Settings &st) : ConfigWidget(st) {
@@ -81,28 +84,46 @@ void PluginConfig::load(const Settings &r) {
 }
 
 void PluginConfig::save() const {
+	QReadLocker lock(&g.p->qrwlPlugins);
+
 	s.bTransmitPosition = qcbTransmit->isChecked();
-	QMutexLocker lock(&g.p->qmPlugins);
-
 	s.qmPositionalAudioPlugins.clear();
-	foreach(int k, qhInfos.keys()) {
-		PluginInfo *pi = qhInfos[k];
-		bool enabled = qtwPlugins->topLevelItem(k)->checkState(1) == Qt::Checked;
-		s.qmPositionalAudioPlugins[QFileInfo(pi->filename).fileName()] = enabled;
-		pi->enabled = enabled;
-	}
 
+	QList<QTreeWidgetItem *> list = qtwPlugins->findItems(QString(), Qt::MatchContains);
+	foreach(QTreeWidgetItem *i, list) {
+		bool enabled = (i->checkState(1) == Qt::Checked);
+
+		PluginInfo *pi = pluginForItem(i);
+		if (pi) {
+			s.qmPositionalAudioPlugins.insert(pi->filename, enabled);
+			pi->enabled = enabled;
+		}
+	}
 }
 
 bool PluginConfig::expert(bool) {
 	return false;
 }
 
+PluginInfo *PluginConfig::pluginForItem(QTreeWidgetItem *i) const {
+	if (i) {
+		foreach(PluginInfo *pi, g.p->qlPlugins) {
+			if (pi->filename == i->data(0, Qt::UserRole).toString())
+				return pi;
+		}
+	}
+	return NULL;
+}
+
 void PluginConfig::on_qpbConfig_clicked() {
-	QModelIndex idx = qtwPlugins->currentIndex();
-	if (!idx.isValid())
+	PluginInfo *pi;
+	{
+		QReadLocker lock(&g.p->qrwlPlugins);
+		pi = pluginForItem(qtwPlugins->currentItem());
+	}
+
+	if (! pi)
 		return;
-	PluginInfo *pi=qhInfos[idx.row()];
 
 	if (pi->p->config)
 		pi->p->config(winId());
@@ -111,10 +132,14 @@ void PluginConfig::on_qpbConfig_clicked() {
 }
 
 void PluginConfig::on_qpbAbout_clicked() {
-	QModelIndex idx = qtwPlugins->currentIndex();
-	if (!idx.isValid())
+	PluginInfo *pi;
+	{
+		QReadLocker lock(&g.p->qrwlPlugins);
+		pi = pluginForItem(qtwPlugins->currentItem());
+	}
+
+	if (! pi)
 		return;
-	PluginInfo *pi=qhInfos[idx.row()];
 
 	if (pi->p->about)
 		pi->p->about(winId());
@@ -128,23 +153,26 @@ void PluginConfig::on_qpbReload_clicked() {
 }
 
 void PluginConfig::refillPluginList() {
-	QMutexLocker lock(&g.p->qmPlugins);
+	QReadLocker lock(&g.p->qrwlPlugins);
 	qtwPlugins->clear();
-	qhInfos.clear();
-	PluginInfo *pi;
-	foreach(pi, g.p->qlPlugins) {
+
+	foreach(PluginInfo *pi, g.p->qlPlugins) {
 		QTreeWidgetItem *i = new QTreeWidgetItem(qtwPlugins);
 		i->setFlags(Qt::ItemIsUserCheckable | Qt::ItemIsEnabled | Qt::ItemIsSelectable);
 		i->setCheckState(1, pi->enabled ? Qt::Checked : Qt::Unchecked);
 		i->setText(0, pi->description);
-		qhInfos[qtwPlugins->indexOfTopLevelItem(i)] = pi;
+		if (pi->p->longdesc)
+			i->setToolTip(0, QString::fromStdWString(pi->p->longdesc()));
+		i->setData(0, Qt::UserRole, pi->filename);
 	}
 	qtwPlugins->setCurrentItem(qtwPlugins->topLevelItem(0));
 	on_qtwPlugins_currentItemChanged(qtwPlugins->topLevelItem(0), NULL);
 }
 
 void PluginConfig::on_qtwPlugins_currentItemChanged(QTreeWidgetItem *current, QTreeWidgetItem *) {
-	PluginInfo *pi = qhInfos.value(qtwPlugins->indexOfTopLevelItem(current));
+	QReadLocker lock(&g.p->qrwlPlugins);
+
+	PluginInfo *pi=pluginForItem(current);
 	if (pi) {
 		qpbAbout->setEnabled(pi->p->about != NULL);
 		qpbConfig->setEnabled(pi->p->config != NULL);
@@ -165,6 +193,7 @@ Plugins::Plugins(QObject *p) : QObject(p) {
 		fPosition[i]=fFront[i]=fTop[i]= 0.0;
 	QMetaObject::connectSlotsByName(this);
 
+#ifdef QT_NO_DEBUG
 #ifndef PLUGIN_PATH
 #ifndef Q_OS_MAC
 	qsSystemPlugins=QString::fromLatin1("%1/plugins").arg(qApp->applicationDirPath());
@@ -176,6 +205,10 @@ Plugins::Plugins(QObject *p) : QObject(p) {
 #endif
 
 	qsUserPlugins = g.qdBasePath.absolutePath() + QLatin1String("/Plugins");
+#else
+	qsSystemPlugins = QString::fromLatin1("%1/plugins").arg(qApp->applicationDirPath());
+	qsUserPlugins = QString();
+#endif
 
 #ifdef Q_OS_WIN
 	// According to MS KB Q131065, we need this to OpenProcess()
@@ -213,7 +246,7 @@ Plugins::~Plugins() {
 }
 
 void Plugins::clearPlugins() {
-	QMutexLocker lock(&qmPlugins);
+	QWriteLocker lock(&g.p->qrwlPlugins);
 	foreach(PluginInfo *pi, qlPlugins) {
 		if (pi->locked)
 			pi->p->unlock();
@@ -226,21 +259,26 @@ void Plugins::clearPlugins() {
 void Plugins::rescanPlugins() {
 	clearPlugins();
 
-	QMutexLocker lock(&qmPlugins);
+	QWriteLocker lock(&g.p->qrwlPlugins);
 	prevlocked = locked = NULL;
 	bValid = false;
 
-	QDir qud(qsUserPlugins, QString(), QDir::Name, QDir::Files | QDir::Readable);
 	QDir qd(qsSystemPlugins, QString(), QDir::Name, QDir::Files | QDir::Readable);
+#ifdef QT_NO_DEBUG
+	QDir qud(qsUserPlugins, QString(), QDir::Name, QDir::Files | QDir::Readable);
 	QFileInfoList libs = qud.entryInfoList() + qd.entryInfoList();
+#else
+	QFileInfoList libs = qd.entryInfoList();
+#endif
+
 	QSet<QString> loaded;
 	foreach(const QFileInfo &libinfo, libs) {
 		QString fname = libinfo.fileName();
 		QString libname = libinfo.absoluteFilePath();
 		if (!loaded.contains(fname) && QLibrary::isLibrary(libname)) {
 			PluginInfo *pi = new PluginInfo();
-			pi->filename = libname;
-			pi->lib.setFileName(pi->filename);
+			pi->lib.setFileName(libname);
+			pi->filename = fname;
 			if (pi->lib.load()) {
 				mumblePluginFunc mpf = reinterpret_cast<mumblePluginFunc>(pi->lib.resolve("getMumblePlugin"));
 				if (mpf) {
@@ -248,9 +286,18 @@ void Plugins::rescanPlugins() {
 					if (pi->p && (pi->p->magic == MUMBLE_PLUGIN_MAGIC)) {
 						pi->description = QString::fromStdWString(pi->p->description);
 						pi->shortname = QString::fromStdWString(pi->p->shortname);
+						pi->enabled = g.s.qmPositionalAudioPlugins.value(pi->filename, true);
+
+						mumblePlugin2Func mpf2 = reinterpret_cast<mumblePlugin2Func>(pi->lib.resolve("getMumblePlugin2"));
+						if (mpf2) {
+							pi->p2 = mpf2();
+							if (pi->p2->magic != MUMBLE_PLUGIN_MAGIC_2) {
+								pi->p2 = NULL;
+							}
+						}
+
 						qlPlugins << pi;
 						loaded.insert(fname);
-						pi->enabled = g.s.qmPositionalAudioPlugins.contains(fname) ? g.s.qmPositionalAudioPlugins[fname] : true;
 						continue;
 					}
 				}
@@ -288,7 +335,7 @@ bool Plugins::fetch() {
 		return bValid;
 	}
 
-	QMutexLocker lock(&qmPlugins);
+	QReadLocker lock(&qrwlPlugins);
 	if (! locked) {
 		bValid = false;
 		return bValid;
@@ -297,14 +344,23 @@ bool Plugins::fetch() {
 	if (!locked->enabled)
 		bUnlink = true;
 
-	bool ok = locked->p->fetch(fPosition, fFront, fTop, fCameraPosition, fCameraFront, fCameraTop, ssContext, swsIdentity);
+	bool ok;
+	{
+		QMutexLocker mlock(&qmPluginStrings);
+		ok = locked->p->fetch(fPosition, fFront, fTop, fCameraPosition, fCameraFront, fCameraTop, ssContext, swsIdentity);
+	}
 	if (! ok || bUnlink) {
-		locked->p->unlock();
-		locked->locked = false;
-		prevlocked = locked;
-		locked = NULL;
-		for (int i=0;i<3;i++)
-			fPosition[i]=fFront[i]=fTop[i]=fCameraPosition[i]=fCameraFront[i]=fCameraTop[i] = 0.0f;
+		lock.unlock();
+		QWriteLocker wlock(&qrwlPlugins);
+
+		if (locked) {
+			locked->p->unlock();
+			locked->locked = false;
+			prevlocked = locked;
+			locked = NULL;
+			for (int i=0;i<3;i++)
+				fPosition[i]=fFront[i]=fTop[i]=fCameraPosition[i]=fCameraFront[i]=fCameraTop[i] = 0.0f;
+		}
 	}
 	bValid = ok;
 	return bValid;
@@ -313,37 +369,43 @@ bool Plugins::fetch() {
 void Plugins::on_Timer_timeout() {
 	fetch();
 
-	QMutexLocker lock(&qmPlugins);
+	QReadLocker lock(&qrwlPlugins);
 
 	if (prevlocked) {
 		g.l->log(Log::Information, tr("%1 lost link.").arg(prevlocked->shortname));
 		prevlocked = NULL;
 	}
 
-	if (! locked) {
-		ssContext.clear();
-		swsIdentity.clear();
-	}
 
-	std::string context;
-	if (locked)
-		context.assign(u8(QString::fromStdWString(locked->p->shortname)) + static_cast<char>(0) + ssContext);
+	{
+		QMutexLocker mlock(&qmPluginStrings);
 
-	if (! g.uiSession) {
-		ssContextSent.clear();
-		swsIdentitySent.clear();
-	} else if ((context != ssContextSent) || (swsIdentity != swsIdentitySent)) {
-		MumbleProto::UserState mpus;
-		mpus.set_session(g.uiSession);
-		if (context != ssContextSent) {
-			ssContextSent.assign(context);
-			mpus.set_plugin_context(context);
+		if (! locked) {
+			ssContext.clear();
+			swsIdentity.clear();
 		}
-		if (swsIdentity != swsIdentitySent) {
-			swsIdentitySent.assign(swsIdentity);
-			mpus.set_plugin_identity(u8(QString::fromStdWString(swsIdentitySent)));
+
+		std::string context;
+		if (locked)
+			context.assign(u8(QString::fromStdWString(locked->p->shortname)) + static_cast<char>(0) + ssContext);
+
+		if (! g.uiSession) {
+			ssContextSent.clear();
+			swsIdentitySent.clear();
+		} else if ((context != ssContextSent) || (swsIdentity != swsIdentitySent)) {
+			MumbleProto::UserState mpus;
+			mpus.set_session(g.uiSession);
+			if (context != ssContextSent) {
+				ssContextSent.assign(context);
+				mpus.set_plugin_context(context);
+			}
+			if (swsIdentity != swsIdentitySent) {
+				swsIdentitySent.assign(swsIdentity);
+				mpus.set_plugin_identity(u8(QString::fromStdWString(swsIdentitySent)));
+			}
+			if (g.sh)
+				g.sh->sendMessage(mpus);
 		}
-		g.sh->sendMessage(mpus);
 	}
 
 	if (locked) {
@@ -353,6 +415,9 @@ void Plugins::on_Timer_timeout() {
 	if (! g.s.bTransmitPosition)
 		return;
 
+	lock.unlock();
+	QWriteLocker wlock(&qrwlPlugins);
+
 	if (qlPlugins.isEmpty())
 		return;
 
@@ -360,40 +425,67 @@ void Plugins::on_Timer_timeout() {
 	if (iPluginTry >= qlPlugins.count())
 		iPluginTry = 0;
 
+	std::multimap<std::wstring, unsigned long long int> pids;
+#ifdef Q_OS_WIN
+	PROCESSENTRY32 pe;
+
+	pe.dwSize = sizeof(pe);
+	HANDLE hSnap = CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);
+	if (hSnap != INVALID_HANDLE_VALUE) {
+		BOOL ok = Process32First(hSnap, &pe);
+
+		while (ok) {
+			pids.insert(std::pair<std::wstring, unsigned long long int>(std::wstring(pe.szExeFile), pe.th32ProcessID));
+			ok = Process32Next(hSnap, &pe);
+		}
+		CloseHandle(hSnap);
+	}
+#endif
+
 	PluginInfo *pi = qlPlugins.at(iPluginTry);
-	if (pi->enabled && pi->p->trylock()) {
-		pi->shortname = QString::fromStdWString(pi->p->shortname);
-		g.l->log(Log::Information, tr("%1 linked.").arg(pi->shortname));
-		pi->locked = true;
-		bUnlink = false;
-		locked = pi;
+	if (pi->enabled) {
+		if (pi->p2 ? pi->p2->trylock(pids) : pi->p->trylock()) {
+			pi->shortname = QString::fromStdWString(pi->p->shortname);
+			g.l->log(Log::Information, tr("%1 linked.").arg(pi->shortname));
+			pi->locked = true;
+			bUnlink = false;
+			locked = pi;
+		}
 	}
 }
 
 void Plugins::checkUpdates() {
-	QUrl url(QLatin1String("http://mumble.info/plugins.php"));
+	QUrl url;
+	url.setScheme(QLatin1String("http"));
+	url.setHost(g.qsRegionalHost);
+	url.setPath(QLatin1String("/plugins.php"));
 
 	url.addQueryItem(QLatin1String("ver"), QLatin1String(QUrl::toPercentEncoding(QLatin1String(MUMBLE_RELEASE))));
 #if defined(Q_OS_WIN)
 	url.addQueryItem(QLatin1String("os"), QLatin1String("Win32"));
+	url.addQueryItem(QLatin1String("abi"), QLatin1String(MUMTEXT(_MSC_VER)));
 #elif defined(Q_OS_MAC)
 	url.addQueryItem(QLatin1String("os"), QLatin1String("MacOSX"));
 #else
 	url.addQueryItem(QLatin1String("os"), QLatin1String("Unix"));
 #endif
 
-	QNetworkRequest req(url);
-	QNetworkReply *rep = g.nam->get(req);
+#ifdef QT_NO_DEBUG
+	QNetworkReply *rep = Network::get(url);
 	connect(rep, SIGNAL(finished()), this, SLOT(finished()));
+#else
+	g.mw->msgBox(tr("Skipping plugin update in debug mode."));
+#endif
 }
 
 void Plugins::finished() {
 	QNetworkReply *rep = qobject_cast<QNetworkReply *>(sender());
+	QUrl url = rep->request().url();
 
 	bool rescan = false;
 
 	if (rep->error() == QNetworkReply::NoError) {
-		const QString &path = rep->url().path();
+		const QString &path = url.path();
 		if (path == QLatin1String("/plugins.php")) {
 			qmPluginHash.clear();
 			QDomDocument doc;
@@ -477,11 +569,12 @@ void Plugins::finished() {
 			QMap<QString, QString>::const_iterator i;
 			for (i = qmPluginHash.constBegin(); i != qmPluginHash.constEnd(); ++i) {
 				if (! i.value().isEmpty()) {
-					QUrl url(QLatin1String("http://mumble.info/"));
+					QUrl url;
+					url.setScheme(QLatin1String("http"));
+					url.setHost(g.qsRegionalHost);
 					url.setPath(QString::fromLatin1("plugins/%1").arg(i.key()));
 
-					QNetworkRequest req(url);
-					QNetworkReply *r = g.nam->get(req);
+					QNetworkReply *r = Network::get(url);
 					connect(r, SIGNAL(finished()), this, SLOT(finished()));
 				}
 			}
@@ -521,11 +614,11 @@ void Plugins::finished() {
 
 						static GUID guid = WINTRUST_ACTION_GENERIC_VERIFY_V2;
 
-						HRESULT hr = WinVerifyTrust(0, &guid , &data);
+						LONG ts = WinVerifyTrust(0, &guid , &data);
 
 						QFile deltemp(tempname);
 						deltemp.remove();
-						verified = (hr == 0);
+						verified = (ts == 0);
 					}
 #endif
 					if (verified) {
@@ -553,7 +646,12 @@ void Plugins::finished() {
 				}
 			}
 		}
-
+	} else {
+		if (url.host() == g.qsRegionalHost) {
+			url.setHost(QLatin1String("mumble.info"));
+			QNetworkReply *nrep = Network::get(url);
+			connect(nrep, SIGNAL(finished()), this, SLOT(finished()));
+		}
 	}
 
 	if (rescan)
