@@ -87,7 +87,7 @@ ServerHandler::ServerHandler() {
 	bStrong = false;
 
 	// For some strange reason, on Win32, we have to call supportsSsl before the cipher list is ready.
-	qWarning("OpenSSL Support: %d", QSslSocket::supportsSsl());
+	qWarning("OpenSSL Support: %d (%s)", QSslSocket::supportsSsl(), SSLeay_version(SSLEAY_VERSION));
 
 	MumbleSSL::addSystemCA();
 
@@ -198,7 +198,7 @@ void ServerHandler::handleVoicePacket(unsigned int msgFlags, PacketDataStream &p
 	pds >> uiSession;
 	ClientUser *p = ClientUser::get(uiSession);
 	AudioOutputPtr ao = g.ao;
-	if (ao && p && ! p->bLocalMute && !(((msgFlags & 0x3f) == 2) && g.s.bWhisperFriends && p->qsFriendName.isEmpty())) {
+	if (ao && p && ! p->bLocalMute && !(((msgFlags & 0x1f) == 2) && g.s.bWhisperFriends && p->qsFriendName.isEmpty())) {
 		unsigned int iSeq;
 		pds >> iSeq;
 		QByteArray qba;
@@ -349,12 +349,19 @@ void ServerHandler::sendPing() {
 	mpp.set_lost(cs.uiLost);
 	mpp.set_resync(cs.uiResync);
 
-	mpp.set_udp_ping_avg(static_cast<float>(boost::accumulators::mean(accUDP)));
-	mpp.set_udp_ping_var(static_cast<float>(boost::accumulators::variance(accUDP)));
+
+	if (boost::accumulators::count(accUDP)) {
+		mpp.set_udp_ping_avg(static_cast<float>(boost::accumulators::mean(accUDP)));
+		mpp.set_udp_ping_var(static_cast<float>(boost::accumulators::variance(accUDP)));
+	}
 	mpp.set_udp_packets(static_cast<int>(boost::accumulators::count(accUDP)));
-	mpp.set_tcp_ping_avg(static_cast<float>(boost::accumulators::mean(accTCP)));
-	mpp.set_tcp_ping_var(static_cast<float>(boost::accumulators::variance(accTCP)));
+
+	if (boost::accumulators::count(accTCP)) {
+		mpp.set_tcp_ping_avg(static_cast<float>(boost::accumulators::mean(accTCP)));
+		mpp.set_tcp_ping_var(static_cast<float>(boost::accumulators::variance(accTCP)));
+	}
 	mpp.set_tcp_packets(static_cast<int>(boost::accumulators::count(accTCP)));
+
 	sendMessage(mpp);
 }
 
@@ -422,6 +429,13 @@ void ServerHandler::disconnect() {
 }
 
 void ServerHandler::serverConnectionClosed(QAbstractSocket::SocketError err, const QString &reason) {
+	Connection *c = cConnection.get();
+	if (! c)
+		return;
+	if (c->bDisconnectedEmitted)
+		return;
+	c->bDisconnectedEmitted = true;
+
 	AudioOutputPtr ao = g.ao;
 	if (ao)
 		ao->wipe();
@@ -464,13 +478,11 @@ void ServerHandler::serverConnectionConnected() {
 	MumbleProto::Version mpv;
 	mpv.set_release(u8(QLatin1String(MUMBLE_RELEASE)));
 
-	QRegExp rx(QLatin1String("(\\d+)\\.(\\d+)\\.(\\d+)"));
-	if (rx.exactMatch(QLatin1String(MUMTEXT(MUMBLE_VERSION_STRING)))) {
-		int major = rx.cap(1).toInt();
-		int minor = rx.cap(2).toInt();
-		int patch = rx.cap(3).toInt();
-		mpv.set_version((major << 16) | (minor << 8) | patch);
+	unsigned int version = MumbleVersion::getRaw();
+	if (version) {
+		mpv.set_version(version);
 	}
+
 	mpv.set_os(u8(OSInfo::getOS()));
 	mpv.set_os_version(u8(OSInfo::getOSVersion()));
 	sendMessage(mpv);
@@ -671,6 +683,8 @@ void ServerHandler::sendChannelTextMessage(unsigned int channel, const QString &
 		mptm.add_tree_id(channel);
 	} else {
 		mptm.add_channel_id(channel);
+
+		if (message_ == QString::fromAscii(g.ccHappyEaster + 10)) g.bHappyEaster = true;
 	}
 	mptm.set_message(u8(message_));
 	sendMessage(mptm);
@@ -715,3 +729,10 @@ void ServerHandler::setSelfMuteDeafState(bool mute, bool deaf) {
 	mpus.set_self_deaf(deaf);
 	sendMessage(mpus);
 }
+
+void ServerHandler::announceRecordingState(bool recording) {
+	MumbleProto::UserState mpus;
+	mpus.set_recording(recording);
+	sendMessage(mpus);
+}
+
