@@ -34,6 +34,7 @@
 #include "Global.h"
 #include "AudioInput.h"
 #include "Cert.h"
+#include "../../overlay/overlay.h"
 
 bool Shortcut::isServerSpecific() const {
 	if (qvData.canConvert<ShortcutTarget>()) {
@@ -114,6 +115,131 @@ QDataStream &operator>>(QDataStream &qds, ShortcutTarget &st) {
 const QString Settings::cqsDefaultPushClickOn = QLatin1String(":/on.ogg");
 const QString Settings::cqsDefaultPushClickOff = QLatin1String(":/off.ogg");
 
+OverlaySettings::OverlaySettings() {
+	bEnable = true;
+
+	fX = 1.0f;
+	fY = 0.0f;
+	fZoom = 0.875f;
+
+#ifdef Q_OS_MAC
+	qsStyle = QLatin1String("Cleanlooks");
+#endif
+
+	osShow = LinkedChannels;
+	bAlwaysSelf = true;
+
+	qcUserName[Settings::Passive] = QColor(170, 170, 170);
+	qcUserName[Settings::Talking] = QColor(255, 255, 255);
+	qcUserName[Settings::Whispering] = QColor(128, 255, 128);
+	qcUserName[Settings::Shouting] = QColor(255, 128, 255);
+	qcChannel = QColor(255, 255, 128);
+	qcBoxPen = QColor(0, 0, 0, 224);
+	qcBoxFill = QColor(0, 0, 0);
+
+	setPreset();
+
+	// FPS display settings
+	qcFps = Qt::white;
+	fFps = 0.75f;
+	qfFps = qfUserName;
+	qrfFps = QRectF(10, 10, -1, 0.023438f);
+	bFps = false;
+
+	bUseWhitelist = false;
+
+#ifdef Q_OS_WIN
+	int i = 0;
+	while (overlayBlacklist[i]) {
+		qslBlacklist << QLatin1String(overlayBlacklist[i]);
+		i++;
+	}
+#endif
+
+}
+
+void OverlaySettings::setPreset(const OverlayPresets preset) {
+	switch (preset) {
+		case LargeSquareAvatar:
+			uiColumns = 2;
+			fUserName = 0.75f;
+			fChannel = 0.75f;
+			fMutedDeafened = 0.5f;
+			fAvatar = 1.0f;
+
+#if defined(Q_OS_WIN) || defined(Q_OS_MAC)
+			qfUserName = QFont(QLatin1String("Verdana"), 20);
+#else
+			qfUserName = QFont(QLatin1String("Arial"), 20);
+#endif
+			qfChannel = qfUserName;
+
+			fUser[Settings::Passive] = 0.5f;
+			fUser[Settings::Talking] = (7.0f / 8.0f);
+			fUser[Settings::Whispering] = (7.0f / 8.0f);
+			fUser[Settings::Shouting] = (7.0f / 8.0f);
+
+			qrfUserName = QRectF(-0.0625f, 0.101563f - 0.0625f, 0.125f, 0.023438f);
+			qrfChannel = QRectF(-0.03125f, -0.0625f, 0.09375f, 0.015625f);
+			qrfMutedDeafened = QRectF(-0.0625f, -0.0625f, 0.0625f, 0.0625f);
+			qrfAvatar = QRectF(-0.0625f, -0.0625f, 0.125f, 0.125f);
+
+			fBoxPenWidth = (1.f / 256.0f);
+			fBoxPad = (1.f / 256.0f);
+
+			bUserName = true;
+			bChannel = true;
+			bMutedDeafened = true;
+			bAvatar = true;
+			bBox = false;
+
+			qaUserName = Qt::AlignCenter;
+			qaMutedDeafened = Qt::AlignLeft | Qt::AlignTop;
+			qaAvatar = Qt::AlignCenter;
+			qaChannel = Qt::AlignCenter;
+			break;
+		case AvatarAndName:
+		default:
+			uiColumns = 1;
+			fUserName = 1.0f;
+			fChannel = (7.0f / 8.0f);
+			fMutedDeafened = (7.0f / 8.0f);
+			fAvatar = 1.0f;
+
+#if defined(Q_OS_WIN) || defined(Q_OS_MAC)
+			qfUserName = QFont(QLatin1String("Verdana"), 20);
+#else
+			qfUserName = QFont(QLatin1String("Arial"), 20);
+#endif
+			qfChannel = qfUserName;
+
+			fUser[Settings::Passive] = 0.5f;
+			fUser[Settings::Talking] = (7.0f / 8.0f);
+			fUser[Settings::Whispering] = (7.0f / 8.0f);
+			fUser[Settings::Shouting] = (7.0f / 8.0f);
+
+			qrfUserName = QRectF(0.015625f, -0.015625f, 0.250f, 0.03125f);
+			qrfChannel = QRectF(0.03125f, -0.015625f, 0.1875f, 0.015625f);
+			qrfMutedDeafened = QRectF(0.234375f, -0.015625f, 0.03125f, 0.03125f);
+			qrfAvatar = QRectF(-0.03125f, -0.015625f, 0.03125f, 0.03125f);
+
+			fBoxPenWidth = 0.0f;
+			fBoxPad = (1.f / 256.0f);
+
+			bUserName = true;
+			bChannel = false;
+			bMutedDeafened = true;
+			bAvatar = true;
+			bBox = true;
+
+			qaUserName = Qt::AlignLeft | Qt::AlignVCenter;
+			qaMutedDeafened = Qt::AlignRight | Qt::AlignVCenter;
+			qaAvatar = Qt::AlignRight | Qt::AlignVCenter;
+			qaChannel = Qt::AlignLeft | Qt::AlignTop;
+			break;
+	}
+}
+
 Settings::Settings() {
 	qRegisterMetaType<ShortcutTarget>("ShortcutTarget");
 	qRegisterMetaTypeStreamOperators<ShortcutTarget>("ShortcutTarget");
@@ -136,7 +262,7 @@ Settings::Settings() {
 	iFramesPerPacket = 2;
 	iNoiseSuppress = -30;
 	iIdleTime = 0;
-	vsVAD = SignalToNoise;
+	vsVAD = Amplitude;
 	fVADmin = 0.80f;
 	fVADmax = 0.98f;
 
@@ -153,22 +279,26 @@ Settings::Settings() {
 
 #ifdef NO_UPDATE_CHECK
 	bUpdateCheck = false;
-	bPluginCheck = false;
+	bPluginOverlayCheck = false;
 #else
 	bUpdateCheck = true;
-	bPluginCheck = true;
+	bPluginOverlayCheck = true;
 #endif
 
 	qsImagePath = QDesktopServices::storageLocation(QDesktopServices::PicturesLocation);
 
-	bFirstTime = true;
 	ceExpand = ChannelsWithUsers;
 	ceChannelDrag = Ask;
 	bMinimalView = false;
 	bHideFrame = false;
 	aotbAlwaysOnTop = OnTopNever;
 	bAskOnQuit = true;
-	bHideTray = true;
+#ifdef Q_OS_WIN
+	// Don't enable minimize to tray by default on win7
+	bHideInTray = (QSysInfo::windowsVersion() != QSysInfo::WV_6_1);
+#else
+	bHideInTray = true;
+#endif
 	bStateInTray = true;
 	bUsage = true;
 	bShowUserCount = false;
@@ -185,37 +315,19 @@ Settings::Settings() {
 	bEcho = false;
 	bEchoMulti = true;
 
+	bExclusiveInput = false;
+	bExclusiveOutput = false;
+
 	iPortAudioInput = -1; // default device
 	iPortAudioOutput = -1; // default device
 
-	bPositionalAudio = false;
+	bPositionalAudio = true;
 	bPositionalHeadphone = false;
 	fAudioMinDistance = 1.0f;
 	fAudioMaxDistance = 15.0f;
 	fAudioMaxDistVolume = 0.80f;
 	fAudioBloom = 0.5f;
 
-	bOverlayEnable = true;
-	osOverlay = All;
-	bOverlayAlwaysSelf = true;
-	fOverlayX = 0.8f;
-	fOverlayY = 0.0f;
-	bOverlayTop = false;
-	bOverlayBottom = true;
-	bOverlayLeft = true;
-	bOverlayRight = false;
-#if defined(Q_OS_WIN) || defined(Q_OS_MAC)
-	qfOverlayFont = QFont(QLatin1String("Verdana"), 20);
-#else
-	qfOverlayFont = QFont(QLatin1String("Arial"), 20);
-#endif
-	fOverlayHeight = .03f;
-	qcOverlayUser = QColor(255,255,255,128);
-	qcOverlayTalking = QColor(255,255,196,255);
-	qcOverlayWhisper = QColor(255,128,128,255);
-	qcOverlayChannel = QColor(192,192,255,192);
-	qcOverlayChannelTalking = QColor(224,224,255,255);
-
 	iLCDUserViewMinColWidth = 50;
 	iLCDUserViewSplitterWidth = 2;
 
@@ -223,6 +335,7 @@ Settings::Settings() {
 	bTCPCompat = false;
 	bQoS = true;
 	bReconnect = true;
+	bAutoConnect = false;
 	ptProxyType = NoProxy;
 	usProxyPort = 0;
 
@@ -234,6 +347,15 @@ Settings::Settings() {
 	// Accessibility
 	bHighContrast = false;
 
+	// Recording
+	qsRecordingPath = QDesktopServices::storageLocation(QDesktopServices::DocumentsLocation);
+	qsRecordingFile = QLatin1String("Mumble-%date-%time-%host-%user");
+	rmRecordingMode = RecordingMixdown;
+	iRecordingFormat = 0;
+
+	// Config updates
+	uiUpdateCounter = 0;
+
 #if defined(AUDIO_TEST)
 	lmLoopMode = Server;
 #else
@@ -259,6 +381,7 @@ Settings::Settings() {
 	qmMessageSounds[Log::YouMutedOther] = QLatin1String(":/UserMutedYouOrByYou.ogg");
 	qmMessageSounds[Log::YouMuted] = QLatin1String(":/UserMutedYouOrByYou.ogg");
 	qmMessageSounds[Log::YouKicked] = QLatin1String(":/UserKickedYouOrByYou.ogg");
+	qmMessageSounds[Log::Recording] = QLatin1String(":/RecordingStateChanged.ogg");
 
 	qmMessages[Log::DebugInfo] = Settings::LogConsole;
 	qmMessages[Log::Warning] = Settings::LogConsole | Settings::LogBalloon;
@@ -291,8 +414,9 @@ bool Settings::doPositionalAudio() const {
 #include BOOST_TYPEOF_INCREMENT_REGISTRATION_GROUP()
 
 
-BOOST_TYPEOF_REGISTER_TYPE(Settings::AudioTransmit);
-BOOST_TYPEOF_REGISTER_TYPE(Settings::VADSource);
+BOOST_TYPEOF_REGISTER_TYPE(Qt::Alignment)
+BOOST_TYPEOF_REGISTER_TYPE(Settings::AudioTransmit)
+BOOST_TYPEOF_REGISTER_TYPE(Settings::VADSource)
 BOOST_TYPEOF_REGISTER_TYPE(Settings::LoopMode)
 BOOST_TYPEOF_REGISTER_TYPE(Settings::OverlayShow)
 BOOST_TYPEOF_REGISTER_TYPE(Settings::ProxyType)
@@ -301,6 +425,7 @@ BOOST_TYPEOF_REGISTER_TYPE(Settings::ChannelDrag)
 BOOST_TYPEOF_REGISTER_TYPE(Settings::ServerShow)
 BOOST_TYPEOF_REGISTER_TYPE(Settings::WindowLayout)
 BOOST_TYPEOF_REGISTER_TYPE(Settings::AlwaysOnTopBehaviour)
+BOOST_TYPEOF_REGISTER_TYPE(Settings::RecordingMode)
 BOOST_TYPEOF_REGISTER_TYPE(QString)
 BOOST_TYPEOF_REGISTER_TYPE(QByteArray)
 BOOST_TYPEOF_REGISTER_TYPE(QColor)
@@ -308,10 +433,81 @@ BOOST_TYPEOF_REGISTER_TYPE(QVariant)
 BOOST_TYPEOF_REGISTER_TYPE(QFont)
 BOOST_TYPEOF_REGISTER_TEMPLATE(QList, 1)
 
-#define SAVELOAD(var,name) var = qvariant_cast<BOOST_TYPEOF(var)>(g.qs->value(QLatin1String(name), var))
-#define LOADENUM(var, name) var = static_cast<BOOST_TYPEOF(var)>(g.qs->value(QLatin1String(name), var).toInt())
+#define SAVELOAD(var,name) var = qvariant_cast<BOOST_TYPEOF(var)>(settings_ptr->value(QLatin1String(name), var))
+#define LOADENUM(var, name) var = static_cast<BOOST_TYPEOF(var)>(settings_ptr->value(QLatin1String(name), var).toInt())
+#define LOADFLAG(var, name) var = static_cast<BOOST_TYPEOF(var)>(settings_ptr->value(QLatin1String(name), static_cast<int>(var)).toInt())
+
+void OverlaySettings::load() {
+	load(g.qs);
+}
+
+void OverlaySettings::load(QSettings* settings_ptr) {
+	SAVELOAD(bEnable, "enable");
+
+	LOADENUM(osShow, "show");
+	SAVELOAD(bAlwaysSelf, "alwaysself");
+
+	SAVELOAD(fX, "x");
+	SAVELOAD(fY, "y");
+	SAVELOAD(fZoom, "zoom");
+	SAVELOAD(uiColumns, "columns");
+
+	settings_ptr->beginReadArray(QLatin1String("states"));
+	for (int i=0;i<4;++i) {
+		settings_ptr->setArrayIndex(i);
+		SAVELOAD(qcUserName[i], "color");
+		SAVELOAD(fUser[i], "opacity");
+	}
+	settings_ptr->endArray();
+
+	SAVELOAD(qfUserName, "userfont");
+	SAVELOAD(qfChannel, "channelfont");
+	SAVELOAD(qcChannel, "channelcolor");
+	SAVELOAD(qfFps, "fpsfont");
+	SAVELOAD(qcFps, "fpscolor");
+
+	SAVELOAD(fBoxPad, "padding");
+	SAVELOAD(fBoxPenWidth, "penwidth");
+	SAVELOAD(qcBoxPen, "pencolor");
+	SAVELOAD(qcBoxFill, "fillcolor");
+
+	SAVELOAD(bUserName, "usershow");
+	SAVELOAD(bChannel, "channelshow");
+	SAVELOAD(bMutedDeafened, "mutedshow");
+	SAVELOAD(bAvatar, "avatarshow");
+	SAVELOAD(bBox, "boxshow");
+	SAVELOAD(bFps, "fpsshow");
+
+	SAVELOAD(fUserName, "useropacity");
+	SAVELOAD(fChannel, "channelopacity");
+	SAVELOAD(fMutedDeafened, "mutedopacity");
+	SAVELOAD(fAvatar, "avataropacity");
+	SAVELOAD(fFps, "fpsopacity");
+
+	SAVELOAD(qrfUserName, "userrect");
+	SAVELOAD(qrfChannel, "channelrect");
+	SAVELOAD(qrfMutedDeafened, "mutedrect");
+	SAVELOAD(qrfAvatar, "avatarrect");
+	SAVELOAD(qrfFps, "fpsrect");
+
+	LOADFLAG(qaUserName, "useralign");
+	LOADFLAG(qaChannel, "channelalign");
+	LOADFLAG(qaMutedDeafened, "mutedalign");
+	LOADFLAG(qaAvatar, "avataralign");
+
+	SAVELOAD(bUseWhitelist, "usewhitelist");
+	SAVELOAD(qslBlacklist, "blacklist");
+	SAVELOAD(qslWhitelist, "whitelist");
+}
 
 void Settings::load() {
+	load(g.qs);
+}
+
+void Settings::load(QSettings* settings_ptr) {
+	// Config updates
+	SAVELOAD(uiUpdateCounter, "lastupdate");
+
 	SAVELOAD(bMute, "audio/mute");
 	SAVELOAD(bDeaf, "audio/deaf");
 	LOADENUM(atTransmit, "audio/transmit");
@@ -338,11 +534,14 @@ void Settings::load() {
 	SAVELOAD(fAudioBloom, "audio/bloom");
 	SAVELOAD(bEcho, "audio/echo");
 	SAVELOAD(bEchoMulti, "audio/echomulti");
+	SAVELOAD(bExclusiveInput, "audio/exclusiveinput");
+	SAVELOAD(bExclusiveOutput, "audio/exclusiveoutput");
 	SAVELOAD(bPositionalAudio, "audio/positional");
 	SAVELOAD(bPositionalHeadphone, "audio/headphone");
 	SAVELOAD(qsAudioInput, "audio/input");
 	SAVELOAD(qsAudioOutput, "audio/output");
 	SAVELOAD(bWhisperFriends, "audio/whisperfriends");
+	SAVELOAD(bTransmitPosition, "audio/postransmit");
 
 	SAVELOAD(iJitterBufferSize, "net/jitterbuffer");
 	SAVELOAD(iFramesPerPacket, "net/framesperpacket");
@@ -376,27 +575,11 @@ void Settings::load() {
 	SAVELOAD(iTTSVolume, "tts/volume");
 	SAVELOAD(iTTSThreshold, "tts/threshold");
 
-	SAVELOAD(bOverlayEnable, "overlay/enable");
-	LOADENUM(osOverlay, "overlay/show");
-	SAVELOAD(bOverlayAlwaysSelf, "overlay/alwaysself");
-	SAVELOAD(fOverlayX, "overlay/x");
-	SAVELOAD(fOverlayY, "overlay/y");
-	SAVELOAD(bTransmitPosition, "audio/postransmit");
-	SAVELOAD(bOverlayTop, "overlay/top");
-	SAVELOAD(bOverlayBottom, "overlay/bottom");
-	SAVELOAD(bOverlayLeft, "overlay/left");
-	SAVELOAD(bOverlayRight, "overlay/right");
-	SAVELOAD(qfOverlayFont, "overlay/font");
-	SAVELOAD(fOverlayHeight, "overlay/height");
-	SAVELOAD(qcOverlayUser, "overlay/user");
-	SAVELOAD(qcOverlayTalking, "overlay/talking");
-	SAVELOAD(qcOverlayChannel, "overlay/channel");
-	SAVELOAD(qcOverlayChannelTalking, "overlay/channeltalking");
-
 	// Network settings
 	SAVELOAD(bTCPCompat, "net/tcponly");
 	SAVELOAD(bQoS, "net/qos");
 	SAVELOAD(bReconnect, "net/reconnect");
+	SAVELOAD(bAutoConnect, "net/autoconnect");
 	SAVELOAD(bSuppressIdentity, "net/suppress");
 	LOADENUM(ptProxyType, "net/proxytype");
 	SAVELOAD(qsProxyHost, "net/proxyhost");
@@ -418,7 +601,6 @@ void Settings::load() {
 	SAVELOAD(bMinimalView, "ui/minimalview");
 	SAVELOAD(bHideFrame, "ui/hideframe");
 	SAVELOAD(bUserTop, "ui/usertop");
-	SAVELOAD(bFirstTime, "ui/firsttime120");
 	SAVELOAD(qbaMainWindowGeometry, "ui/geometry");
 	SAVELOAD(qbaMainWindowState, "ui/state");
 	SAVELOAD(qbaMinimalViewGeometry, "ui/minimalviewgeometry");
@@ -431,10 +613,10 @@ void Settings::load() {
 	SAVELOAD(qsLastServer, "ui/server");
 	LOADENUM(ssFilter, "ui/serverfilter");
 #ifndef NO_UPDATE_CHECK
-	SAVELOAD(bUpdateCheck, "ui/updatecheck");
-	SAVELOAD(bPluginCheck, "ui/plugincheck");
+	SAVELOAD(bPluginOverlayCheck, "ui/updatecheck");
+	SAVELOAD(bPluginOverlayCheck, "ui/plugincheck");
 #endif
-	SAVELOAD(bHideTray, "ui/hidetray");
+	SAVELOAD(bHideInTray, "ui/hidetray");
 	SAVELOAD(bStateInTray, "ui/stateintray");
 	SAVELOAD(bUsage, "ui/usage");
 	SAVELOAD(bShowUserCount, "ui/showusercount");
@@ -444,16 +626,23 @@ void Settings::load() {
 	SAVELOAD(qbaConnectDialogHeader, "ui/connect/header");
 	SAVELOAD(bHighContrast, "ui/HighContrast");
 
+	// Recording
+	SAVELOAD(qsRecordingPath, "recording/path");
+	SAVELOAD(qsRecordingFile, "recording/file");
+	LOADENUM(rmRecordingMode, "recording/mode");
+	SAVELOAD(iRecordingFormat, "recording/format");
+
+	// LCD
 	SAVELOAD(iLCDUserViewMinColWidth, "lcd/userview/mincolwidth");
 	SAVELOAD(iLCDUserViewSplitterWidth, "lcd/userview/splitterwidth");
 
-	QByteArray qba = qvariant_cast<QByteArray>(g.qs->value(QLatin1String("net/certificate")));
+	QByteArray qba = qvariant_cast<QByteArray>(settings_ptr->value(QLatin1String("net/certificate")));
 	if (! qba.isEmpty())
 		kpCertificate = CertWizard::importCert(qba);
 
-	int nshorts = g.qs->beginReadArray(QLatin1String("shortcuts"));
+	int nshorts = settings_ptr->beginReadArray(QLatin1String("shortcuts"));
 	for (int i=0;i<nshorts;i++) {
-		g.qs->setArrayIndex(i);
+		settings_ptr->setArrayIndex(i);
 		Shortcut s;
 
 		s.iIndex = -2;
@@ -461,45 +650,118 @@ void Settings::load() {
 		SAVELOAD(s.iIndex, "index");
 		SAVELOAD(s.qlButtons, "keys");
 		SAVELOAD(s.bSuppress, "suppress");
-		s.qvData = g.qs->value(QLatin1String("data"));
+		s.qvData = settings_ptr->value(QLatin1String("data"));
 		if (s.iIndex >= -1)
 			qlShortcuts << s;
 	}
-	g.qs->endArray();
+	settings_ptr->endArray();
 
-	g.qs->beginReadArray(QLatin1String("messages"));
+	settings_ptr->beginReadArray(QLatin1String("messages"));
 	for (QMap<int, quint32>::const_iterator it = qmMessages.constBegin(); it != qmMessages.constEnd(); ++it) {
-		g.qs->setArrayIndex(it.key());
+		settings_ptr->setArrayIndex(it.key());
 		SAVELOAD(qmMessages[it.key()], "log");
 	}
-	g.qs->endArray();
+	settings_ptr->endArray();
 
-	g.qs->beginReadArray(QLatin1String("messagesounds"));
+	settings_ptr->beginReadArray(QLatin1String("messagesounds"));
 	for (QMap<int, QString>::const_iterator it = qmMessageSounds.constBegin(); it != qmMessageSounds.constEnd(); ++it) {
-		g.qs->setArrayIndex(it.key());
+		settings_ptr->setArrayIndex(it.key());
 		SAVELOAD(qmMessageSounds[it.key()], "logsound");
 	}
-	g.qs->endArray();
+	settings_ptr->endArray();
 
-	g.qs->beginGroup(QLatin1String("lcd/devices"));
-	foreach(const QString &d, g.qs->childKeys()) {
-		qmLCDDevices.insert(d, g.qs->value(d, true).toBool());
+	settings_ptr->beginGroup(QLatin1String("lcd/devices"));
+	foreach(const QString &d, settings_ptr->childKeys()) {
+		qmLCDDevices.insert(d, settings_ptr->value(d, true).toBool());
 	}
-	g.qs->endGroup();
+	settings_ptr->endGroup();
 
-	g.qs->beginGroup(QLatin1String("audio/plugins"));
-	foreach(const QString &d, g.qs->childKeys()) {
-		qmPositionalAudioPlugins.insert(d, g.qs->value(d, true).toBool());
+	settings_ptr->beginGroup(QLatin1String("audio/plugins"));
+	foreach(const QString &d, settings_ptr->childKeys()) {
+		qmPositionalAudioPlugins.insert(d, settings_ptr->value(d, true).toBool());
 	}
-	g.qs->endGroup();
+	settings_ptr->endGroup();
+
+	settings_ptr->beginGroup(QLatin1String("overlay"));
+	os.load(settings_ptr);
+	settings_ptr->endGroup();
 }
 
 #undef SAVELOAD
-#define SAVELOAD(var,name) if (var != def.var) g.qs->setValue(QLatin1String(name), var); else g.qs->remove(QLatin1String(name))
+#define SAVELOAD(var,name) if (var != def.var) settings_ptr->setValue(QLatin1String(name), var); else settings_ptr->remove(QLatin1String(name))
+#define SAVEFLAG(var,name) if (var != def.var) settings_ptr->setValue(QLatin1String(name), static_cast<int>(var)); else settings_ptr->remove(QLatin1String(name))
+
+void OverlaySettings::save() {
+	save(g.qs);
+}
+
+void OverlaySettings::save(QSettings* settings_ptr) {
+	OverlaySettings def;
+
+	SAVELOAD(bEnable, "enable");
+
+	SAVELOAD(osShow, "show");
+	SAVELOAD(bAlwaysSelf, "alwaysself");
+	SAVELOAD(fX, "x");
+	SAVELOAD(fY, "y");
+	SAVELOAD(fZoom, "zoom");
+	SAVELOAD(uiColumns, "columns");
+
+	settings_ptr->beginReadArray(QLatin1String("states"));
+	for (int i=0;i<4;++i) {
+		settings_ptr->setArrayIndex(i);
+		SAVELOAD(qcUserName[i], "color");
+		SAVELOAD(fUser[i], "opacity");
+	}
+	settings_ptr->endArray();
+
+	SAVELOAD(qfUserName, "userfont");
+	SAVELOAD(qfChannel, "channelfont");
+	SAVELOAD(qcChannel, "channelcolor");
+	SAVELOAD(qfFps, "fpsfont");
+	SAVELOAD(qcFps, "fpscolor");
+
+	SAVELOAD(fBoxPad, "padding");
+	SAVELOAD(fBoxPenWidth, "penwidth");
+	SAVELOAD(qcBoxPen, "pencolor");
+	SAVELOAD(qcBoxFill, "fillcolor");
+
+	SAVELOAD(bUserName, "usershow");
+	SAVELOAD(bChannel, "channelshow");
+	SAVELOAD(bMutedDeafened, "mutedshow");
+	SAVELOAD(bAvatar, "avatarshow");
+	SAVELOAD(bBox, "boxshow");
+	SAVELOAD(bFps, "fpsshow");
+
+	SAVELOAD(fUserName, "useropacity");
+	SAVELOAD(fChannel, "channelopacity");
+	SAVELOAD(fMutedDeafened, "mutedopacity");
+	SAVELOAD(fAvatar, "avataropacity");
+	SAVELOAD(fFps, "fpsopacity");
+
+	SAVELOAD(qrfUserName, "userrect");
+	SAVELOAD(qrfChannel, "channelrect");
+	SAVELOAD(qrfMutedDeafened, "mutedrect");
+	SAVELOAD(qrfAvatar, "avatarrect");
+	SAVELOAD(qrfFps, "fpsrect");
+
+	SAVEFLAG(qaUserName, "useralign");
+	SAVEFLAG(qaChannel, "channelalign");
+	SAVEFLAG(qaMutedDeafened, "mutedalign");
+	SAVEFLAG(qaAvatar, "avataralign");
+
+	settings_ptr->setValue(QLatin1String("usewhitelist"), bUseWhitelist);
+	settings_ptr->setValue(QLatin1String("blacklist"), qslBlacklist);
+	settings_ptr->setValue(QLatin1String("whitelist"), qslWhitelist);
+}
 
 void Settings::save() {
+	QSettings* settings_ptr = g.qs;
 	Settings def;
 
+	// Config updates
+	SAVELOAD(uiUpdateCounter, "lastupdate");
+
 	SAVELOAD(bMute, "audio/mute");
 	SAVELOAD(bDeaf, "audio/deaf");
 	SAVELOAD(atTransmit, "audio/transmit");
@@ -526,11 +788,14 @@ void Settings::save() {
 	SAVELOAD(fAudioBloom, "audio/bloom");
 	SAVELOAD(bEcho, "audio/echo");
 	SAVELOAD(bEchoMulti, "audio/echomulti");
+	SAVELOAD(bExclusiveInput, "audio/exclusiveinput");
+	SAVELOAD(bExclusiveOutput, "audio/exclusiveoutput");
 	SAVELOAD(bPositionalAudio, "audio/positional");
 	SAVELOAD(bPositionalHeadphone, "audio/headphone");
 	SAVELOAD(qsAudioInput, "audio/input");
 	SAVELOAD(qsAudioOutput, "audio/output");
 	SAVELOAD(bWhisperFriends, "audio/whisperfriends");
+	SAVELOAD(bTransmitPosition, "audio/postransmit");
 
 	SAVELOAD(iJitterBufferSize, "net/jitterbuffer");
 	SAVELOAD(iFramesPerPacket, "net/framesperpacket");
@@ -564,27 +829,11 @@ void Settings::save() {
 	SAVELOAD(iTTSVolume, "tts/volume");
 	SAVELOAD(iTTSThreshold, "tts/threshold");
 
-	SAVELOAD(bOverlayEnable, "overlay/enable");
-	SAVELOAD(osOverlay, "overlay/show");
-	SAVELOAD(bOverlayAlwaysSelf, "overlay/alwaysself");
-	SAVELOAD(fOverlayX, "overlay/x");
-	SAVELOAD(fOverlayY, "overlay/y");
-	SAVELOAD(bTransmitPosition, "audio/postransmit");
-	SAVELOAD(bOverlayTop, "overlay/top");
-	SAVELOAD(bOverlayBottom, "overlay/bottom");
-	SAVELOAD(bOverlayLeft, "overlay/left");
-	SAVELOAD(bOverlayRight, "overlay/right");
-	SAVELOAD(qfOverlayFont, "overlay/font");
-	SAVELOAD(fOverlayHeight, "overlay/height");
-	SAVELOAD(qcOverlayUser, "overlay/user");
-	SAVELOAD(qcOverlayTalking, "overlay/talking");
-	SAVELOAD(qcOverlayChannel, "overlay/channel");
-	SAVELOAD(qcOverlayChannelTalking, "overlay/channeltalking");
-
 	// Network settings
 	SAVELOAD(bTCPCompat, "net/tcponly");
 	SAVELOAD(bQoS, "net/qos");
 	SAVELOAD(bReconnect, "net/reconnect");
+	SAVELOAD(bAutoConnect, "net/autoconnect");
 	SAVELOAD(ptProxyType, "net/proxytype");
 	SAVELOAD(qsProxyHost, "net/proxyhost");
 	SAVELOAD(usProxyPort, "net/proxyport");
@@ -605,7 +854,6 @@ void Settings::save() {
 	SAVELOAD(bMinimalView, "ui/minimalview");
 	SAVELOAD(bHideFrame, "ui/hideframe");
 	SAVELOAD(bUserTop, "ui/usertop");
-	SAVELOAD(bFirstTime, "ui/firsttime120");
 	SAVELOAD(qbaMainWindowGeometry, "ui/geometry");
 	SAVELOAD(qbaMainWindowState, "ui/state");
 	SAVELOAD(qbaMinimalViewGeometry, "ui/minimalviewgeometry");
@@ -618,8 +866,8 @@ void Settings::save() {
 	SAVELOAD(qsLastServer, "ui/server");
 	SAVELOAD(ssFilter, "ui/serverfilter");
 	SAVELOAD(bUpdateCheck, "ui/updatecheck");
-	SAVELOAD(bPluginCheck, "ui/plugincheck");
-	SAVELOAD(bHideTray, "ui/hidetray");
+	SAVELOAD(bPluginOverlayCheck, "ui/plugincheck");
+	SAVELOAD(bHideInTray, "ui/hidetray");
 	SAVELOAD(bStateInTray, "ui/stateintray");
 	SAVELOAD(bUsage, "ui/usage");
 	SAVELOAD(bShowUserCount, "ui/showusercount");
@@ -629,56 +877,67 @@ void Settings::save() {
 	SAVELOAD(qbaConnectDialogHeader, "ui/connect/header");
 	SAVELOAD(bHighContrast, "ui/HighContrast");
 
+	// Recording
+	SAVELOAD(qsRecordingPath, "recording/path");
+	SAVELOAD(qsRecordingFile, "recording/file");
+	SAVELOAD(rmRecordingMode, "recording/mode");
+	SAVELOAD(iRecordingFormat, "recording/format");
+
+	// LCD
 	SAVELOAD(iLCDUserViewMinColWidth, "lcd/userview/mincolwidth");
 	SAVELOAD(iLCDUserViewSplitterWidth, "lcd/userview/splitterwidth");
 
 	QByteArray qba = CertWizard::exportCert(kpCertificate);
-	g.qs->setValue(QLatin1String("net/certificate"), qba);
+	settings_ptr->setValue(QLatin1String("net/certificate"), qba);
 
-	g.qs->beginWriteArray(QLatin1String("shortcuts"));
+	settings_ptr->beginWriteArray(QLatin1String("shortcuts"));
 	int idx = 0;
 	foreach(const Shortcut &s, qlShortcuts) {
 		if (! s.isServerSpecific()) {
-			g.qs->setArrayIndex(idx++);
-			g.qs->setValue(QLatin1String("index"), s.iIndex);
-			g.qs->setValue(QLatin1String("keys"), s.qlButtons);
-			g.qs->setValue(QLatin1String("suppress"), s.bSuppress);
-			g.qs->setValue(QLatin1String("data"), s.qvData);
+			settings_ptr->setArrayIndex(idx++);
+			settings_ptr->setValue(QLatin1String("index"), s.iIndex);
+			settings_ptr->setValue(QLatin1String("keys"), s.qlButtons);
+			settings_ptr->setValue(QLatin1String("suppress"), s.bSuppress);
+			settings_ptr->setValue(QLatin1String("data"), s.qvData);
 		}
 	}
-	g.qs->endArray();
+	settings_ptr->endArray();
 
-	g.qs->beginWriteArray(QLatin1String("messages"));
+	settings_ptr->beginWriteArray(QLatin1String("messages"));
 	for (QMap<int, quint32>::const_iterator it = qmMessages.constBegin(); it != qmMessages.constEnd(); ++it) {
-		g.qs->setArrayIndex(it.key());
+		settings_ptr->setArrayIndex(it.key());
 		SAVELOAD(qmMessages[it.key()], "log");
 	}
-	g.qs->endArray();
+	settings_ptr->endArray();
 
-	g.qs->beginWriteArray(QLatin1String("messagesounds"));
+	settings_ptr->beginWriteArray(QLatin1String("messagesounds"));
 	for (QMap<int, QString>::const_iterator it = qmMessageSounds.constBegin(); it != qmMessageSounds.constEnd(); ++it) {
-		g.qs->setArrayIndex(it.key());
+		settings_ptr->setArrayIndex(it.key());
 		SAVELOAD(qmMessageSounds[it.key()], "logsound");
 	}
-	g.qs->endArray();
+	settings_ptr->endArray();
 
-	g.qs->beginGroup(QLatin1String("lcd/devices"));
+	settings_ptr->beginGroup(QLatin1String("lcd/devices"));
 	foreach(const QString &d, qmLCDDevices.keys()) {
 		bool v = qmLCDDevices.value(d);
 		if (!v)
-			g.qs->setValue(d, v);
+			settings_ptr->setValue(d, v);
 		else
-			g.qs->remove(d);
+			settings_ptr->remove(d);
 	}
-	g.qs->endGroup();
+	settings_ptr->endGroup();
 
-	g.qs->beginGroup(QLatin1String("audio/plugins"));
+	settings_ptr->beginGroup(QLatin1String("audio/plugins"));
 	foreach(const QString &d, qmPositionalAudioPlugins.keys()) {
 		bool v = qmPositionalAudioPlugins.value(d);
 		if (!v)
-			g.qs->setValue(d, v);
+			settings_ptr->setValue(d, v);
 		else
-			g.qs->remove(d);
+			settings_ptr->remove(d);
 	}
-	g.qs->endGroup();
+	settings_ptr->endGroup();
+
+	settings_ptr->beginGroup(QLatin1String("overlay"));
+	os.save(settings_ptr);
+	settings_ptr->endGroup();
 }
