@@ -36,6 +36,7 @@
 #include "OSInfo.h"
 #include "LCD.h"
 #include "ClientUser.h"
+#include "NetworkConfig.h"
 
 Usage::Usage(QObject *p) : QObject(p) {
 	qbReport.open(QBuffer::ReadWrite);
@@ -48,6 +49,7 @@ Usage::Usage(QObject *p) : QObject(p) {
 	QTimer::singleShot(60 * 10 * 1000, this, SLOT(reportJitter()));
 }
 
+#ifdef REPORT_JITTER
 void Usage::addJitter(ClientUser *cu) {
 	QMutexLocker qml(& cu->qmTiming);
 	if (! cu->qlTiming.isEmpty()) {
@@ -55,9 +57,10 @@ void Usage::addJitter(ClientUser *cu) {
 		qdsReport << cu->qlTiming;
 	}
 }
+#endif
 
 void Usage::registerUsage() {
-	if (! g.s.bUsage || g.s.bFirstTime)
+	if (! g.s.bUsage || g.s.uiUpdateCounter == 0) // Only register usage if allowed by the user and first wizard run has finished
 		return;
 
 	QDomDocument doc;
@@ -89,6 +92,7 @@ void Usage::registerUsage() {
 	qb->open(QIODevice::ReadOnly);
 
 	QNetworkRequest req(QUrl(QLatin1String("http://mumble.info/usage.cgi")));
+	Network::prepareRequest(req);
 	req.setHeader(QNetworkRequest::ContentTypeHeader, QLatin1String("text/xml"));
 
 	QNetworkReply *rep = g.nam->post(req, qb);
@@ -98,12 +102,14 @@ void Usage::registerUsage() {
 }
 
 void Usage::reportJitter() {
+#ifdef REPORT_JITTER
 	QTimer::singleShot(60 * 10 * 1000, this, SLOT(reportJitter()));
 
 	if (qbReport.size() < 1024)
 		return;
 
 	QNetworkRequest req(QUrl(QLatin1String("http://mumble.info/jitter.cgi")));
+	Network::prepareRequest(req);
 	req.setHeader(QNetworkRequest::ContentTypeHeader, QLatin1String("application/octet-stream"));
 
 	QNetworkReply *rep = g.nam->post(req, qCompress(qbReport.buffer(), 9));
@@ -116,4 +122,5 @@ void Usage::reportJitter() {
 	qdsReport.setDevice(&qbReport);
 	qdsReport.setVersion(QDataStream::Qt_4_4);
 	qdsReport << static_cast<unsigned int>(2);
+#endif
 }
