@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
    Copyright (C) 2009, Stefan Hacker <dd0t@users.sourceforge.net>
 
    All rights reserved.
@@ -169,7 +169,7 @@ int ModelItem::insertIndex(Channel *c) const {
 }
 
 int ModelItem::insertIndex(ClientUser *p) const {
-	QList<QString> qls;
+	QList<ClientUser*> qlclientuser;
 	ModelItem *item;
 
 	int ocount = 0;
@@ -177,15 +177,15 @@ int ModelItem::insertIndex(ClientUser *p) const {
 	foreach(item, qlChildren) {
 		if (item->pUser) {
 			if (item->pUser != p)
-				qls << item->pUser->qsName;
+				qlclientuser << item->pUser;
 		} else
 			ocount++;
 	}
 
-	qls << p->qsName;
-	qSort(qls);
+	qlclientuser << p;
+	qSort(qlclientuser.begin(), qlclientuser.end(), ClientUser::lessThan);
 
-	return qls.indexOf(p->qsName) + (bUsersTop ? 0 : ocount);
+	return qlclientuser.indexOf(p) + (bUsersTop ? 0 : ocount);
 }
 
 QString ModelItem::hash() const {
@@ -193,7 +193,7 @@ QString ModelItem::hash() const {
 		if (! pUser->qsHash.isEmpty())
 			return pUser->qsHash;
 		else
-			return QLatin1String(QCryptographicHash::hash(pUser->qsName.toUtf8(), QCryptographicHash::Sha1).toHex());
+			return QLatin1String(sha1(pUser->qsName).toHex());
 	} else {
 		QCryptographicHash chash(QCryptographicHash::Sha1);
 
@@ -213,8 +213,10 @@ QString ModelItem::hash() const {
 UserModel::UserModel(QObject *p) : QAbstractItemModel(p) {
 	qiTalkingOff=QIcon(QLatin1String("skin:talking_off.svg"));
 	qiTalkingOn=QIcon(QLatin1String("skin:talking_on.svg"));
-	qiTalkingWhisperChannel=QIcon(QLatin1String("skin:talking_alt.svg"));
+	qiTalkingShout=QIcon(QLatin1String("skin:talking_alt.svg"));
 	qiTalkingWhisper=QIcon(QLatin1String("skin:talking_whisper.svg"));
+	qiPrioritySpeaker=QIcon(QLatin1String("skin:actions/audio-input-microphone.svg"));
+	qiRecording=QIcon(QLatin1String("skin:actions/media-record.svg"));
 	qiMutedSelf=QIcon(QLatin1String("skin:muted_self.svg"));
 	qiMutedServer=QIcon(QLatin1String("skin:muted_server.svg"));
 	qiMutedLocal=QIcon(QLatin1String("skin:muted_local.svg"));
@@ -225,12 +227,16 @@ UserModel::UserModel(QObject *p) : QAbstractItemModel(p) {
 	qiChannel=QIcon(QLatin1String("skin:channel.svg"));
 	qiActiveChannel=QIcon(QLatin1String("skin:channel_active.svg"));
 	qiLinkedChannel=QIcon(QLatin1String("skin:channel_linked.svg"));
-	qiFriend=QIcon(QLatin1String(":/emblems/emblem-favorite.svg"));
+	qiFriend=QIcon(QLatin1String("skin:emblems/emblem-favorite.svg"));
 	qiComment=QIcon(QLatin1String("skin:comment.svg"));
 	qiCommentSeen=QIcon(QLatin1String("skin:comment_seen.svg"));
 
 	ModelItem::bUsersTop = g.s.bUserTop;
 
+	uiSessionComment = 0;
+	iChannelDescription = -1;
+	bClicked = false;
+
 	miRoot = new ModelItem(Channel::get(0));
 }
 
@@ -363,13 +369,13 @@ QVariant UserModel::data(const QModelIndex &idx, int role) const {
 			case Qt::DecorationRole:
 				if (idx.column() == 0)
 					switch (p->tsState) {
-						case ClientUser::Talking:
+						case Settings::Talking:
 							return qiTalkingOn;
-						case ClientUser::TalkingWhisper:
+						case Settings::Whispering:
 							return qiTalkingWhisper;
-						case ClientUser::TalkingWhisperChannel:
-							return qiTalkingWhisperChannel;
-						case ClientUser::TalkingOff:
+						case Settings::Shouting:
+							return qiTalkingShout;
+						case Settings::Passive:
 						default:
 							return qiTalkingOff;
 					}
@@ -388,8 +394,12 @@ QVariant UserModel::data(const QModelIndex &idx, int role) const {
 					else
 						return p->qsName;
 				}
-				if (! p->qsComment.isEmpty())
+				if (! p->qbaCommentHash.isEmpty())
 					l << (item->bCommentSeen ? qiCommentSeen : qiComment);
+				if (p->bPrioritySpeaker)
+					l << qiPrioritySpeaker;
+				if (p->bRecording)
+					l << qiRecording;
 				if (p->bMute)
 					l << qiMutedServer;
 				if (p->bSuppress)
@@ -414,7 +424,7 @@ QVariant UserModel::data(const QModelIndex &idx, int role) const {
 		switch (role) {
 			case Qt::DecorationRole:
 				if (idx.column() == 0) {
-					if (qsLinked.contains(c)) {
+					if (g.uiSession && qsLinked.contains(c)) {
 						if (ClientUser::get(g.uiSession)->cChannel == c)
 							return qiActiveChannel;
 						else
@@ -429,7 +439,7 @@ QVariant UserModel::data(const QModelIndex &idx, int role) const {
 
 					return QString::fromLatin1("%1 (%2)").arg(c->qsName).arg(item->iUsers);
 				}
-				if (! c->qsDesc.isEmpty())
+				if (! c->qbaDescHash.isEmpty())
 					l << (item->bCommentSeen ? qiCommentSeen : qiComment);
 				return l;
 			case Qt::FontRole:
@@ -446,6 +456,13 @@ QVariant UserModel::data(const QModelIndex &idx, int role) const {
 					}
 				}
 				break;
+			case Qt::BackgroundRole:
+				if ((c->iId == 0) && g.sh && g.sh->isStrong()) {
+					QColor qc(Qt::green);
+					qc.setAlpha(32);
+					return qc;
+				}
+				break;
 			default:
 				break;
 		}
@@ -472,37 +489,77 @@ QVariant UserModel::otherRoles(const QModelIndex &idx, int role) const {
 
 	switch (role) {
 		case Qt::ToolTipRole:
+			const_cast<UserModel *>(this)->uiSessionComment = 0;
+			const_cast<UserModel *>(this)->iChannelDescription = -1;
+			const_cast<UserModel *>(this)->bClicked = false;
 			switch (section) {
 				case 0: {
 						if (isUser) {
 							QString qsImage;
-							if (! p->qbaTexture.isEmpty() && (p->iTextureWidth > 0)) {
-								// On-the-fly generate a suitably sized compressed image and return its HTML
-
-								QImage img(p->iTextureWidth, 60, QImage::Format_ARGB32);
-								for (int i=0;i<60;++i) {
-									memcpy(img.scanLine(i), p->qbaTexture.constData() + i * 600 * 4, p->iTextureWidth * 4);
+							if (! p->qbaTextureHash.isEmpty()) {
+								if (p->qbaTexture.isEmpty()) {
+									p->qbaTexture = Database::blob(p->qbaTextureHash);
+									if (p->qbaTexture.isEmpty()) {
+										MumbleProto::RequestBlob mprb;
+										mprb.add_session_texture(p->uiSession);
+										g.sh->sendMessage(mprb);
+									} else {
+										g.o->verifyTexture(p);
+									}
+								}
+								if (! p->qbaTexture.isEmpty()) {
+									QBuffer qb(&p->qbaTexture);
+									qb.open(QIODevice::ReadOnly);
+									QImageReader qir(&qb, p->qbaTextureFormat);
+									QSize sz = qir.size();
+									if (sz.width() > 128) {
+										int targ = sz.width() / ((sz.width()+127)/ 128);
+										qsImage = QString::fromLatin1("<img src=\"memoryblob://avatar/%1/texture.%2\" width=\"%3\" />").arg(p->uiSession).arg(QString::fromLatin1(p->qbaTextureFormat).toLower()).arg(targ);
+									} else if (sz.width() > 0) {
+										qsImage = QString::fromLatin1("<img src=\"memoryblob://avatar/%1/texture.%2\" />").arg(p->uiSession).arg(QString::fromLatin1(p->qbaTextureFormat).toLower());
+									}
 								}
-
-								qsImage = Log::imageToImg(img);
 							}
 
-							if (p->qsComment.isEmpty()) {
+							if (p->qbaCommentHash.isEmpty()) {
 								if (! qsImage.isEmpty())
 									return qsImage;
 								else
 									return p->qsName;
 							} else {
+								if (p->qsComment.isEmpty()) {
+									p->qsComment = QString::fromUtf8(Database::blob(p->qbaCommentHash));
+									if (p->qsComment.isEmpty()) {
+										const_cast<UserModel *>(this)->uiSessionComment = p->uiSession;
+
+										MumbleProto::RequestBlob mprb;
+										mprb.add_session_comment(p->uiSession);
+										g.sh->sendMessage(mprb);
+										return QVariant();
+									}
+								}
 								const_cast<UserModel *>(this)->seenComment(idx);
 								QString base = Log::validHtml(p->qsComment);
 								if (! qsImage.isEmpty())
-									base += QLatin1String("<br />")+qsImage;
+									return QString::fromLatin1("<table><tr><td valign=\"top\">%1</td><td>%2</td></tr></table>").arg(qsImage,base);
 								return base;
 							}
 						} else {
-							if (c->qsDesc.isEmpty()) {
+							if (c->qbaDescHash.isEmpty()) {
 								return c->qsName;
 							} else {
+								if (c->qsDesc.isEmpty()) {
+									c->qsDesc = QString::fromUtf8(Database::blob(c->qbaDescHash));
+									if (c->qsDesc.isEmpty()) {
+										const_cast<UserModel *>(this)->iChannelDescription = c->iId;
+
+										MumbleProto::RequestBlob mprb;
+										mprb.add_channel_description(c->iId);
+										g.sh->sendMessage(mprb);
+										return QVariant();
+									}
+								}
+
 								const_cast<UserModel *>(this)->seenComment(idx);
 								return Log::validHtml(c->qsDesc);
 							}
@@ -525,7 +582,7 @@ QVariant UserModel::otherRoles(const QModelIndex &idx, int role) const {
 						                           "<tr><td><img src=\"skin:talking_off.svg\" width=64 /></td><td valign=\"middle\">%5</td></tr>"
 						                           "</table>").arg(tr("This is a user connected to the server. The icon to the left of the user indicates whether or not they are talking:"),
 						                                           tr("Talking to your channel."),
-						                                           tr("Whispering directly to your channel."),
+						                                           tr("Shouting directly to your channel."),
 						                                           tr("Whispering directly to you."),
 						                                           tr("Not talking.")
 						                                          );
@@ -544,7 +601,7 @@ QVariant UserModel::otherRoles(const QModelIndex &idx, int role) const {
 					if (isUser)
 						return QString::fromLatin1("%1"
 						                           "<table>"
-						                           "<tr><td><img src=\":/emblems/emblem-favorite.svg\" width=64 /></td><td valign=\"middle\">%2</td></tr>"
+						                           "<tr><td><img src=\"skin:emblems/emblem-favorite.svg\" width=64 /></td><td valign=\"middle\">%2</td></tr>"
 						                           "<tr><td><img src=\"skin:authenticated.svg\" width=64 /></td><td valign=\"middle\">%3</td></tr>"
 						                           "<tr><td><img src=\"skin:muted_self.svg\" width=64 /></td><td valign=\"middle\">%4</td></tr>"
 						                           "<tr><td><img src=\"skin:muted_server.svg\" width=64 /></td><td valign=\"middle\">%5</td></tr>"
@@ -797,8 +854,7 @@ ClientUser *UserModel::addUser(unsigned int id, const QString &name) {
 		citem = citem->parent;
 	}
 
-	if (g.uiSession && (p->cChannel == ClientUser::get(g.uiSession)->cChannel))
-		updateOverlay();
+	updateOverlay();
 
 	return p;
 }
@@ -830,10 +886,11 @@ void UserModel::removeUser(ClientUser *p) {
 	if (g.s.ceExpand == Settings::ChannelsWithUsers)
 		collapseEmpty(c);
 
-	if (g.uiSession && (c == ClientUser::get(g.uiSession)->cChannel))
-		updateOverlay();
+	updateOverlay();
 
+#ifdef REPORT_JITTER
 	g.mw->uUsage.addJitter(p);
+#endif
 
 	delete p;
 	delete item;
@@ -866,11 +923,7 @@ void UserModel::moveUser(ClientUser *p, Channel *np) {
 		collapseEmpty(oc);
 	}
 
-	if (g.uiSession) {
-		Channel *home = ClientUser::get(g.uiSession)->cChannel;
-		if (home==np || home == oc)
-			updateOverlay();
-	}
+	updateOverlay();
 }
 
 void UserModel::renameUser(ClientUser *p, const QString &name) {
@@ -881,8 +934,7 @@ void UserModel::renameUser(ClientUser *p, const QString &name) {
 	ModelItem *item = ModelItem::c_qhUsers.value(p);
 	moveItem(pi, pi, item);
 
-	if (g.uiSession && (p->cChannel == ClientUser::get(g.uiSession)->cChannel))
-		updateOverlay();
+	updateOverlay();
 }
 
 void UserModel::setUserId(ClientUser *p, int id) {
@@ -906,19 +958,40 @@ void UserModel::setFriendName(ClientUser *p, const QString &name) {
 }
 
 void UserModel::setComment(ClientUser *cu, const QString &comment) {
+	cu->qbaCommentHash = comment.isEmpty() ? QByteArray() : sha1(comment);
+
 	if (comment != cu->qsComment) {
 		ModelItem *item = ModelItem::c_qhUsers.value(cu);
-		int oldstate = cu->qsComment.isEmpty() ? 0 : (item->bCommentSeen ? 2 : 1);
-		int newstate;
+		int oldstate = (cu->qsComment.isEmpty() && cu->qbaCommentHash.isEmpty()) ? 0 : (item->bCommentSeen ? 2 : 1);
+		int newstate = 0;
 
 		cu->qsComment = comment;
 
 		if (! comment.isEmpty()) {
-			item->bCommentSeen = Database::seenComment(item->hash(), comment);
-			newstate = item->bCommentSeen ? 2 : 1;
+			Database::setBlob(cu->qbaCommentHash, cu->qsComment.toUtf8());
+			if (cu->uiSession == uiSessionComment) {
+				uiSessionComment = 0;
+				item->bCommentSeen = false;
+				if (bClicked) {
+					QRect r = g.mw->qtvUsers->visualRect(index(cu));
+					QWhatsThis::showText(g.mw->qtvUsers->viewport()->mapToGlobal(r.bottomRight()), data(index(cu, 0), Qt::ToolTipRole).toString(), g.mw->qtvUsers);
+				} else {
+					QToolTip::showText(QCursor::pos(), data(index(cu, 0), Qt::ToolTipRole).toString(), g.mw->qtvUsers);
+				}
+			} else if (cu->uiSession == ~uiSessionComment) {
+				uiSessionComment = 0;
+				if (cu->uiSession == g.uiSession) {
+					QTimer::singleShot(0, g.mw, SLOT(on_qaSelfComment_triggered()));
+				} else {
+					g.mw->cuContextUser = cu;
+					QTimer::singleShot(0, g.mw, SLOT(on_qaUserCommentView_triggered()));
+				}
+			} else {
+				item->bCommentSeen = Database::seenComment(item->hash(), cu->qbaCommentHash);
+				newstate = item->bCommentSeen ? 2 : 1;
+			}
 		} else {
 			item->bCommentSeen = true;
-			newstate = 0;
 		}
 
 		if (oldstate != newstate) {
@@ -928,20 +1001,53 @@ void UserModel::setComment(ClientUser *cu, const QString &comment) {
 	}
 }
 
+void UserModel::setCommentHash(ClientUser *cu, const QByteArray &hash) {
+	if (hash != cu->qbaCommentHash) {
+		ModelItem *item = ModelItem::c_qhUsers.value(cu);
+		int oldstate = (cu->qsComment.isEmpty() && cu->qbaCommentHash.isEmpty()) ? 0 : (item->bCommentSeen ? 2 : 1);
+		int newstate;
+
+		cu->qsComment = QString();
+		cu->qbaCommentHash = hash;
+
+		item->bCommentSeen = Database::seenComment(item->hash(), cu->qbaCommentHash);
+		newstate = item->bCommentSeen ? 2 : 1;
+
+		if (oldstate != newstate) {
+			QModelIndex idx = index(cu, 0);
+			emit dataChanged(idx, idx);
+		}
+	}
+}
+
 void UserModel::setComment(Channel *c, const QString &comment) {
+	c->qbaDescHash = comment.isEmpty() ? QByteArray() : sha1(comment);
+
 	if (comment != c->qsDesc) {
 		ModelItem *item = ModelItem::c_qhChannels.value(c);
 		int oldstate = c->qsDesc.isEmpty() ? 0 : (item->bCommentSeen ? 2 : 1);
-		int newstate;
+		int newstate = 0;
 
 		c->qsDesc = comment;
 
 		if (! comment.isEmpty()) {
-			item->bCommentSeen = Database::seenComment(item->hash(), comment);
-			newstate = item->bCommentSeen ? 2 : 1;
+			Database::setBlob(c->qbaDescHash, c->qsDesc.toUtf8());
+
+			if (c->iId == iChannelDescription) {
+				iChannelDescription = -1;
+				item->bCommentSeen = false;
+				if (bClicked) {
+					QRect r = g.mw->qtvUsers->visualRect(index(c));
+					QWhatsThis::showText(g.mw->qtvUsers->viewport()->mapToGlobal(r.bottomRight()), data(index(c, 0), Qt::ToolTipRole).toString(), g.mw->qtvUsers);
+				} else {
+					QToolTip::showText(QCursor::pos(), data(index(c, 0), Qt::ToolTipRole).toString(), g.mw->qtvUsers);
+				}
+			} else {
+				item->bCommentSeen = Database::seenComment(item->hash(), c->qbaDescHash);
+				newstate = item->bCommentSeen ? 2 : 1;
+			}
 		} else {
 			item->bCommentSeen = true;
-			newstate = 0;
 		}
 
 		if (oldstate != newstate) {
@@ -951,6 +1057,25 @@ void UserModel::setComment(Channel *c, const QString &comment) {
 	}
 }
 
+void UserModel::setCommentHash(Channel *c, const QByteArray &hash) {
+	if (hash != c->qbaDescHash) {
+		ModelItem *item = ModelItem::c_qhChannels.value(c);
+		int oldstate = (c->qsDesc.isEmpty() && c->qbaDescHash.isEmpty()) ? 0 : (item->bCommentSeen ? 2 : 1);
+		int newstate;
+
+		c->qsDesc = QString();
+		c->qbaDescHash = hash;
+
+		item->bCommentSeen = Database::seenComment(item->hash(), hash);
+		newstate = item->bCommentSeen ? 2 : 1;
+
+		if (oldstate != newstate) {
+			QModelIndex idx = index(c, 0);
+			emit dataChanged(idx, idx);
+		}
+	}
+}
+
 void UserModel::seenComment(const QModelIndex &idx) {
 	ModelItem *item;
 	item = static_cast<ModelItem *>(idx.internalPointer());
@@ -963,9 +1088,9 @@ void UserModel::seenComment(const QModelIndex &idx) {
 	emit dataChanged(idx, idx);
 
 	if (item->pUser)
-		Database::setSeenComment(item->hash(), item->pUser->qsComment);
+		Database::setSeenComment(item->hash(), item->pUser->qbaCommentHash);
 	else
-		Database::setSeenComment(item->hash(), item->cChan->qsDesc);
+		Database::setSeenComment(item->hash(), item->cChan->qbaDescHash);
 }
 
 void UserModel::renameChannel(Channel *c, const QString &name) {
@@ -1099,6 +1224,10 @@ void UserModel::removeAll() {
 	ModelItem *item = miRoot;
 	ModelItem *i;
 
+	uiSessionComment = 0;
+	iChannelDescription = -1;
+	bClicked = false;
+
 	foreach(i, item->qlChildren) {
 		if (i->pUser)
 			removeUser(i->pUser);
@@ -1166,12 +1295,12 @@ void UserModel::userMuteDeafChanged() {
 	ClientUser *p=static_cast<ClientUser *>(sender());
 	QModelIndex idx = index(p);
 	emit dataChanged(idx, idx);
-	if (g.uiSession && (p->cChannel == ClientUser::get(g.uiSession)->cChannel))
-		updateOverlay();
+
+	updateOverlay();
 }
 
 Qt::DropActions UserModel::supportedDropActions() const {
-	return Qt::CopyAction;
+	return Qt::MoveAction;
 }
 
 QStringList UserModel::mimeTypes() const {
@@ -1243,7 +1372,7 @@ bool UserModel::dropMimeData(const QMimeData *md, Qt::DropAction, int row, int c
 		int ret;
 		switch (g.s.ceChannelDrag) {
 			case Settings::Ask:
-				ret=QMessageBox::question(g.mw, tr("Mumble"), tr("Are you sure you want to drag this channel?"), QMessageBox::Yes, QMessageBox::No);
+				ret=QMessageBox::question(g.mw, QLatin1String("Mumble"), tr("Are you sure you want to drag this channel?"), QMessageBox::Yes, QMessageBox::No);
 
 				if (ret == QMessageBox::No)
 					return false;
@@ -1262,6 +1391,9 @@ bool UserModel::dropMimeData(const QMimeData *md, Qt::DropAction, int row, int c
 		long long inewpos = 0;
 		Channel *dropped = Channel::c_qhChannels.value(iId);
 
+		if (! dropped)
+			return false;
+
 		if (p.isValid()) {
 			ModelItem *pi = static_cast<ModelItem *>(p.internalPointer());
 			if (pi->pUser)
@@ -1333,7 +1465,7 @@ bool UserModel::dropMimeData(const QMimeData *md, Qt::DropAction, int row, int c
 					} else {
 						// Not enough space, other channels have to be moved
 						if (static_cast<long long>(pi->channelAt(ilast)->iPosition) + 40 > INT_MAX) {
-							QMessageBox::critical(g.mw, tr("Mumble"), tr("Cannot perform this movement automatically, please reset the numeric sorting indicators or adjust it manually."));
+							QMessageBox::critical(g.mw, QLatin1String("Mumble"), tr("Cannot perform this movement automatically, please reset the numeric sorting indicators or adjust it manually."));
 							return false;
 						}
 						for (int i = row; i <= ilast; i++) {
@@ -1353,7 +1485,7 @@ bool UserModel::dropMimeData(const QMimeData *md, Qt::DropAction, int row, int c
 		}
 
 		if (inewpos > INT_MAX || inewpos < INT_MIN) {
-			QMessageBox::critical(g.mw, tr("Mumble"), tr("Cannot perform this movement automatically, please reset the numeric sorting indicators or adjust it manually."));
+			QMessageBox::critical(g.mw, QLatin1String("Mumble"), tr("Cannot perform this movement automatically, please reset the numeric sorting indicators or adjust it manually."));
 			return false;
 		}
 
