@@ -41,7 +41,7 @@ struct ModelItem {
 	friend class UserModel;
 
 private:
-	Q_DISABLE_COPY(ModelItem);
+	Q_DISABLE_COPY(ModelItem)
 public:
 	Channel *cChan;
 	ClientUser *pUser;
@@ -83,8 +83,10 @@ class UserModel : public QAbstractItemModel {
 		Q_OBJECT
 		Q_DISABLE_COPY(UserModel)
 	protected:
-		QIcon qiTalkingOn, qiTalkingWhisper, qiTalkingWhisperChannel, qiTalkingOff;
+		QIcon qiTalkingOn, qiTalkingWhisper, qiTalkingShout, qiTalkingOff;
 		QIcon qiMutedSelf, qiMutedServer, qiMutedLocal, qiMutedSuppressed;
+		QIcon qiPrioritySpeaker;
+		QIcon qiRecording;
 		QIcon qiDeafenedSelf, qiDeafenedServer;
 		QIcon qiAuthenticated, qiChannel, qiLinkedChannel, qiActiveChannel;
 		QIcon qiFriend;
