@@ -50,6 +50,14 @@
   on user/channel flags (e.g. showing the comment)
 */
 
+/*!
+  \fn void UserView::activated(const QModelIndex &idx)
+  Depending on whether idx points to a channel or user this function
+  either moves the player to the channel or opens a message window.
+  This Slot connected to the objects activated signal. The activated
+  signal could, for example, be triggered by doubleclick.
+*/
+
 /*!
   \fn void UserView::keyboardSearch(const QString &search)
   This implementation provides a recursive realtime search over
@@ -136,8 +144,8 @@ UserView::UserView(QWidget *p) : QTreeView(p) {
 	qtSearch->setInterval(QApplication::keyboardInputInterval());
 	qtSearch->setSingleShot(true);
 
-	connect(this, SIGNAL(customContextMenuRequested(const QPoint &)), this, SLOT(contextMenu(const QPoint &)));
-	connect(this, SIGNAL(doubleClicked(const QModelIndex &)), this, SLOT(doubleClick(const QModelIndex &)));
+	connect(this, SIGNAL(doubleClicked(const QModelIndex &)), this, SLOT(nodeActivated(const QModelIndex &)));
+
 	connect(qtSearch, SIGNAL(timeout()), this, SLOT(selectSearchResult()));
 }
 
@@ -167,6 +175,10 @@ void UserView::mouseReleaseEvent(QMouseEvent *evt) {
 
 			if (cu) {
 				// Calculate pixel offset of comment flag
+				if (cu->bRecording)
+					offset += 18;
+				if (cu->bPrioritySpeaker)
+					offset += 18;
 				if (cu->bMute)
 					offset += 18;
 				if (cu->bSuppress)
@@ -202,33 +214,23 @@ void UserView::mouseReleaseEvent(QMouseEvent *evt) {
 	QTreeView::mouseReleaseEvent(evt);
 }
 
-void UserView::contextMenu(const QPoint &mpos) {
-	UserModel *um = static_cast<UserModel *>(model());
-	QModelIndex idx = indexAt(mpos);
-	if (! idx.isValid())
-		idx = currentIndex();
-	else
-		setCurrentIndex(idx);
-	User *p = um->getUser(idx);
-
-	if (p) {
-		g.mw->qmUser->popup(mapToGlobal(mpos), g.mw->qaUserMute);
-	} else {
-		g.mw->qmChannel->popup(mapToGlobal(mpos), NULL);
-	}
+void UserView::keyPressEvent(QKeyEvent *ev) {
+	if (ev->key() == Qt::Key_Return || ev->key() == Qt::Key_Enter)
+		UserView::nodeActivated(currentIndex());
+	QTreeView::keyPressEvent(ev);
 }
 
-void UserView::doubleClick(const QModelIndex &idx) {
+void UserView::nodeActivated(const QModelIndex &idx) {
 	UserModel *um = static_cast<UserModel *>(model());
-	User *p = um->getUser(idx);
+	ClientUser *p = um->getUser(idx);
 	if (p) {
-		g.mw->on_qaUserTextMessage_triggered();
+		g.mw->openTextMessageDialog(p);
 		return;
 	}
 
 	Channel *c = um->getChannel(idx);
 	if (c) {
-		// if a channel is double clicked join it
+		// if a channel is activated join it
 		g.sh->joinChannel(c->iId);
 	}
 }
