@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -143,12 +143,12 @@ class MurmurDBus : public QDBusAbstractAdaptor {
 		static void registerTypes();
 	public slots:
 		// These have the result ref as the first parameter, so won't be converted to DBus
-		void authenticateSlot(int &res, QString &uname, const QString &pw);
-		void registerUserSlot(int &res, const QMap<QString, QString> &);
+		void authenticateSlot(int &res, QString &uname, const QList<QSslCertificate> &certs, const QString &certhash, bool strong, const QString &pw);
+		void registerUserSlot(int &res, const QMap<int, QString> &);
 		void unregisterUserSlot(int &res, int id);
-		void getRegisteredPlayersSlot(const QString &filter, QMap<int, QString> &res);
-		void getRegistrationSlot(int &, int, QMap<QString, QString> &);
-		void setInfoSlot(int &, int, const QMap<QString, QString> &);
+		void getRegisteredUsersSlot(const QString &filter, QMap<int, QString> &res);
+		void getRegistrationSlot(int &, int, QMap<int, QString> &);
+		void setInfoSlot(int &, int, const QMap<int, QString> &);
 		void setTextureSlot(int &res, int id, const QByteArray &texture);
 		void nameToIdSlot(int &res, const QString &name);
 		void idToNameSlot(QString &res, int id);
