@@ -51,6 +51,7 @@ MetaParams::MetaParams() {
 	iMaxImageMessageLength = 131072;
 	bAllowHTML = true;
 	iDefaultChan = 0;
+	bRememberChan = true;
 	qsWelcomeText = QString("Welcome to this server");
 	qsDatabase = QString();
 	iDBPort = 0;
@@ -79,8 +80,7 @@ MetaParams::MetaParams() {
 }
 
 MetaParams::~MetaParams() {
-	if (qsSettings)
-		delete qsSettings;
+	delete qsSettings;
 }
 
 void MetaParams::read(QString fname) {
@@ -100,12 +100,16 @@ void MetaParams::read(QString fname) {
 
 			datapaths << appdir.absolutePath() + QLatin1String("/Mumble");
 		}
-#elif defined(Q_OS_MAC)
-		datapaths << QDir::homePath() + QLatin1String("/Library/Preferences/Mumble/");
 #else
 		datapaths << QDir::homePath() + QLatin1String("/.murmurd");
 		datapaths << QDir::homePath() + QLatin1String("/.config/Mumble");
 #endif
+
+#if defined(Q_OS_MAC)
+		/* Old Mac data path. */
+		datapaths << QDir::homePath() + QLatin1String("/Library/Preferences/Mumble/");
+#endif
+
 		datapaths << QDir::homePath();
 		datapaths << QDir::currentPath();
 		datapaths << QCoreApplication::instance()->applicationDirPath();
@@ -220,6 +224,7 @@ void MetaParams::read(QString fname) {
 	bAllowHTML = qsSettings->value("allowhtml", bAllowHTML).toBool();
 	iMaxBandwidth = qsSettings->value("bandwidth", iMaxBandwidth).toInt();
 	iDefaultChan = qsSettings->value("defaultchannel", iDefaultChan).toInt();
+	bRememberChan = qsSettings->value("rememberchannel", bRememberChan).toBool();
 	iMaxUsers = qsSettings->value("users", iMaxUsers).toInt();
 	iMaxUsersPerChannel = qsSettings->value("usersperchannel", iMaxUsersPerChannel).toInt();
 	qsWelcomeText = qsSettings->value("welcometext", qsWelcomeText).toString();
@@ -236,7 +241,9 @@ void MetaParams::read(QString fname) {
 	iDBPort = qsSettings->value("dbPort", iDBPort).toInt();
 
 	qsIceEndpoint = qsSettings->value("ice", qsIceEndpoint).toString();
-	qsIceSecret = qsSettings->value("icesecret", qsIceSecret).toString();
+	qsIceSecretRead = qsSettings->value("icesecret", qsIceSecretRead).toString();
+	qsIceSecretRead = qsSettings->value("icesecretread", qsIceSecretRead).toString();
+	qsIceSecretWrite = qsSettings->value("icesecretwrite", qsIceSecretRead).toString();
 
 	iLogDays = qsSettings->value("logdays", iLogDays).toInt();
 
@@ -248,6 +255,7 @@ void MetaParams::read(QString fname) {
 	qsRegName = qsSettings->value("registerName", qsRegName).toString();
 	qsRegPassword = qsSettings->value("registerPassword", qsRegPassword).toString();
 	qsRegHost = qsSettings->value("registerHostname", qsRegHost).toString();
+	qsRegLocation = qsSettings->value("registerLocation", qsRegLocation).toString();
 	qurlRegWeb = QUrl(qsSettings->value("registerUrl", qurlRegWeb.toString()).toString());
 	bBonjour = qsSettings->value("bonjour", bBonjour).toBool();
 
@@ -299,7 +307,7 @@ void MetaParams::read(QString fname) {
 				QSslSocket::addDefaultCaCertificates(ql);
 			}
 		} else {
-			qCritical("Failed to read %s", qPrintable(qsSSLCert));
+			qCritical("Failed to read %s", qPrintable(qsSSLCA));
 		}
 	}
 
@@ -373,6 +381,8 @@ void MetaParams::read(QString fname) {
 		qFatal("No SSL ciphers of at least 128 bit found");
 	QSslSocket::setDefaultCiphers(pref);
 
+	qWarning("OpenSSL: %s", SSLeay_version(SSLEAY_VERSION));
+
 	qmConfig.clear();
 	QStringList hosts;
 	foreach(const QHostAddress &qha, qlBind) {
@@ -387,10 +397,12 @@ void MetaParams::read(QString fname) {
 	qmConfig.insert(QLatin1String("bandwidth"),QString::number(iMaxBandwidth));
 	qmConfig.insert(QLatin1String("users"),QString::number(iMaxUsers));
 	qmConfig.insert(QLatin1String("defaultchannel"),QString::number(iDefaultChan));
+	qmConfig.insert(QLatin1String("rememberchannel"),bRememberChan ? QLatin1String("true") : QLatin1String("false"));
 	qmConfig.insert(QLatin1String("welcometext"),qsWelcomeText);
 	qmConfig.insert(QLatin1String("registername"),qsRegName);
 	qmConfig.insert(QLatin1String("registerpassword"),qsRegPassword);
 	qmConfig.insert(QLatin1String("registerhostname"),qsRegHost);
+	qmConfig.insert(QLatin1String("registerlocation"), qsRegLocation);
 	qmConfig.insert(QLatin1String("registerurl"),qurlRegWeb.toString());
 	qmConfig.insert(QLatin1String("bonjour"), bBonjour ? QLatin1String("true") : QLatin1String("false"));
 	qmConfig.insert(QLatin1String("certificate"),qscCert.toPem());
