@@ -30,6 +30,7 @@
 
 
 #include <Ice/Ice.h>
+#include <Ice/SliceChecksums.h>
 #include <IceUtil/IceUtil.h>
 #include "Meta.h"
 #include "Server.h"
@@ -57,8 +58,7 @@ void IceStart() {
 }
 
 void IceStop() {
-	if (mi)
-		delete mi;
+	delete mi;
 	mi = NULL;
 }
 
@@ -74,6 +74,8 @@ static void userToUser(const ::User *p, Murmur::User &mp) {
 	mp.mute = p->bMute;
 	mp.deaf = p->bDeaf;
 	mp.suppress = p->bSuppress;
+	mp.recording = p->bRecording;
+	mp.prioritySpeaker = p->bPrioritySpeaker;
 	mp.selfMute = p->bSelfMute;
 	mp.selfDeaf = p->bSelfDeaf;
 	mp.channel = p->cChannel->iId;
@@ -142,7 +144,7 @@ static void banToBan(const ::Ban &b, Murmur::Ban &mb) {
 	mb.name = u8(b.qsUsername);
 	mb.hash = u8(b.qsHash);
 	mb.reason = u8(b.qsReason);
-	mb.start = b.qdtStart.toTime_t();
+	mb.start = b.qdtStart.toLocalTime().toTime_t();
 	mb.duration = b.iDuration;
 }
 
@@ -157,7 +159,7 @@ static void banToBan(const ::Murmur::Ban &mb, ::Ban &b) {
 	b.qsUsername = u8(mb.name);
 	b.qsHash = u8(mb.hash);
 	b.qsReason = u8(mb.reason);
-	b.qdtStart = QDateTime::fromTime_t(static_cast<quint32>(mb.start));
+	b.qdtStart = QDateTime::fromTime_t(static_cast<quint32>(mb.start)).toUTC();
 	b.iDuration = mb.duration;
 }
 
@@ -206,10 +208,10 @@ MurmurIce::MurmurIce() {
 
 	try {
 		communicator = Ice::initialize(idd);
-		if (! meta->mp.qsIceSecret.isEmpty()) {
+		if (! meta->mp.qsIceSecretWrite.isEmpty()) {
 			::Ice::ImplicitContextPtr impl = communicator->getImplicitContext();
 			if (impl)
-				impl->put("secret", u8(meta->mp.qsIceSecret));
+				impl->put("secret", u8(meta->mp.qsIceSecretWrite));
 		}
 		adapter = communicator->createObjectAdapterWithEndpoints("Murmur", qPrintable(meta->mp.qsIceEndpoint));
 		MetaPtr m = new MetaI;
@@ -683,6 +685,7 @@ void ServerI::ice_ping(const Ice::Current &current) const {
 		throw ::Ice::ObjectNotExistException(__FILE__, __LINE__);
 }
 
+#define ACCESS_Server_isRunning_READ
 static void impl_Server_isRunning(const ::Murmur::AMD_Server_isRunningPtr cb, int server_id) {
 	NEED_SERVER_EXISTS;
 	cb->ice_response(server != NULL);
@@ -750,7 +753,7 @@ static void impl_Server_setAuthenticator(const ::Murmur::AMD_Server_setAuthentic
 	::Murmur::ServerAuthenticatorPrx prx;
 
 	try {
-		prx = ::Murmur::ServerAuthenticatorPrx::checkedCast(aptr->ice_connectionCached(false));
+		prx = ::Murmur::ServerAuthenticatorPrx::checkedCast(aptr->ice_connectionCached(false)->ice_timeout(5000));
 		const ::Murmur::ServerUpdatingAuthenticatorPrx uprx = ::Murmur::ServerUpdatingAuthenticatorPrx::checkedCast(prx);
 
 		mi->qmServerAuthenticator[server_id] = prx;
@@ -769,16 +772,19 @@ static void impl_Server_setAuthenticator(const ::Murmur::AMD_Server_setAuthentic
 	server->log(QString("Registered Ice Authenticator %1").arg(QString::fromStdString(mi->communicator->proxyToString(aptr))));
 }
 
+#define ACCESS_Server_id_READ
 static void impl_Server_id(const ::Murmur::AMD_Server_idPtr cb, int server_id) {
 	NEED_SERVER_EXISTS;
 	cb->ice_response(server_id);
 }
 
+#define ACCESS_Server_getConf_READ
 static void impl_Server_getConf(const ::Murmur::AMD_Server_getConfPtr cb, int server_id,  const ::std::string& key) {
 	NEED_SERVER_EXISTS;
 	cb->ice_response(u8(ServerDB::getConf(server_id, u8(key)).toString()));
 }
 
+#define ACCESS_Server_getAllConf_READ
 static void impl_Server_getAllConf(const ::Murmur::AMD_Server_getAllConfPtr cb, int server_id) {
 	NEED_SERVER_EXISTS;
 
@@ -808,6 +814,7 @@ static void impl_Server_setSuperuserPassword(const ::Murmur::AMD_Server_setSuper
 	cb->ice_response();
 }
 
+#define ACCESS_Server_getLog_READ
 static void impl_Server_getLog(const ::Murmur::AMD_Server_getLogPtr cb, int server_id,  ::Ice::Int min,  ::Ice::Int max) {
 	NEED_SERVER_EXISTS;
 
@@ -822,6 +829,14 @@ static void impl_Server_getLog(const ::Murmur::AMD_Server_getLogPtr cb, int serv
 	cb->ice_response(ll);
 }
 
+static void impl_Server_getLogLen(const ::Murmur::AMD_Server_getLogLenPtr cb, int server_id) {
+	NEED_SERVER_EXISTS;
+
+	int len = ServerDB::getLogLen(server_id);
+	cb->ice_response(len);
+}
+
+#define ACCESS_Server_getUsers_READ
 static void impl_Server_getUsers(const ::Murmur::AMD_Server_getUsersPtr cb, int server_id) {
 	NEED_SERVER;
 	::Murmur::UserMap pm;
@@ -835,6 +850,7 @@ static void impl_Server_getUsers(const ::Murmur::AMD_Server_getUsersPtr cb, int
 	cb->ice_response(pm);
 }
 
+#define ACCESS_Server_getChannels_READ
 static void impl_Server_getChannels(const ::Murmur::AMD_Server_getChannelsPtr cb, int server_id) {
 	NEED_SERVER;
 	::Murmur::ChannelMap cm;
@@ -847,7 +863,7 @@ static void impl_Server_getChannels(const ::Murmur::AMD_Server_getChannelsPtr cb
 }
 
 static bool userSort(const ::User *a, const ::User *b) {
-	return a->qsName < b->qsName;
+	return ::User::lessThan(a, b);
 }
 
 static bool channelSort(const ::Channel *a, const ::Channel *b) {
@@ -876,11 +892,13 @@ TreePtr recurseTree(const ::Channel *c) {
 	return t;
 }
 
+#define ACCESS_Server_getTree_READ
 static void impl_Server_getTree(const ::Murmur::AMD_Server_getTreePtr cb, int server_id) {
 	NEED_SERVER;
 	cb->ice_response(recurseTree(server->qhChannels.value(0)));
 }
 
+#define ACCESS_Server_getCertificateList_READ
 static void impl_Server_getCertificateList(const ::Murmur::AMD_Server_getCertificateListPtr cb, int server_id, ::Ice::Int session) {
 	NEED_SERVER;
 	NEED_PLAYER;
@@ -902,6 +920,7 @@ static void impl_Server_getCertificateList(const ::Murmur::AMD_Server_getCertifi
 	cb->ice_response(certs);
 }
 
+#define ACCESS_Server_getBans_READ
 static void impl_Server_getBans(const ::Murmur::AMD_Server_getBansPtr cb, int server_id) {
 	NEED_SERVER;
 	::Murmur::BanList bl;
@@ -945,6 +964,7 @@ static void impl_Server_sendMessage(const ::Murmur::AMD_Server_sendMessagePtr cb
 	cb->ice_response();
 }
 
+#define ACCESS_Server_hasPermission_READ
 static void impl_Server_hasPermission(const ::Murmur::AMD_Server_hasPermissionPtr cb, int server_id, ::Ice::Int session, ::Ice::Int channelid, ::Ice::Int perm) {
 	NEED_SERVER;
 	NEED_PLAYER;
@@ -964,7 +984,7 @@ static void impl_Server_addContextCallback(const Murmur::AMD_Server_addContextCa
 	}
 
 	try {
-		const Murmur::ServerContextCallbackPrx &oneway = Murmur::ServerContextCallbackPrx::checkedCast(cbptr->ice_oneway()->ice_connectionCached(false));
+		const Murmur::ServerContextCallbackPrx &oneway = Murmur::ServerContextCallbackPrx::checkedCast(cbptr->ice_oneway()->ice_connectionCached(false)->ice_timeout(5000));
 		qmPrx.insert(u8(action), oneway);
 		cb->ice_response();
 	} catch (...) {
@@ -985,7 +1005,7 @@ static void impl_Server_removeContextCallback(const Murmur::AMD_Server_removeCon
 	QMap<int, QMap<QString, ::Murmur::ServerContextCallbackPrx> > & qmPrx = mi->qmServerContextCallbacks[server_id];
 
 	try {
-		const Murmur::ServerContextCallbackPrx &oneway = Murmur::ServerContextCallbackPrx::uncheckedCast(cbptr->ice_oneway()->ice_connectionCached(false));
+		const Murmur::ServerContextCallbackPrx &oneway = Murmur::ServerContextCallbackPrx::uncheckedCast(cbptr->ice_oneway()->ice_connectionCached(false)->ice_timeout(5000));
 
 		foreach(int session, qmPrx.keys()) {
 			QMap<QString, ::Murmur::ServerContextCallbackPrx> qm = qmPrx[session];
@@ -999,6 +1019,7 @@ static void impl_Server_removeContextCallback(const Murmur::AMD_Server_removeCon
 	}
 }
 
+#define ACCESS_Server_getState_READ
 static void impl_Server_getState(const ::Murmur::AMD_Server_getStatePtr cb, int server_id,  ::Ice::Int session) {
 	NEED_SERVER;
 	NEED_PLAYER;
@@ -1015,7 +1036,7 @@ static void impl_Server_setState(const ::Murmur::AMD_Server_setStatePtr cb, int
 	NEED_PLAYER;
 	NEED_CHANNEL_VAR(channel, state.channel);
 
-	server->setUserState(user, channel, state.mute, state.deaf, state.suppress, u8(state.comment));
+	server->setUserState(user, channel, state.mute, state.deaf, state.suppress, state.prioritySpeaker, u8(state.comment));
 	cb->ice_response();
 }
 
@@ -1027,6 +1048,7 @@ static void impl_Server_sendMessageChannel(const ::Murmur::AMD_Server_sendMessag
 	cb->ice_response();
 }
 
+#define ACCESS_Server_getChannelState_READ
 static void impl_Server_getChannelState(const ::Murmur::AMD_Server_getChannelStatePtr cb, int server_id,  ::Ice::Int channelid) {
 	NEED_SERVER;
 	NEED_CHANNEL;
@@ -1092,6 +1114,7 @@ static void impl_Server_addChannel(const ::Murmur::AMD_Server_addChannelPtr cb,
 	cb->ice_response(newid);
 }
 
+#define ACCESS_Server_getACL_READ
 static void impl_Server_getACL(const ::Murmur::AMD_Server_getACLPtr cb, int server_id, ::Ice::Int channelid) {
 	NEED_SERVER;
 	NEED_CHANNEL;
@@ -1196,6 +1219,7 @@ static void impl_Server_setACL(const ::Murmur::AMD_Server_setACLPtr cb, int serv
 	cb->ice_response();
 }
 
+#define ACCESS_Server_getUserNames_READ
 static void impl_Server_getUserNames(const ::Murmur::AMD_Server_getUserNamesPtr cb, int server_id,  const ::Murmur::IdList& ids) {
 	NEED_SERVER;
 	::Murmur::NameMap nm;
@@ -1205,6 +1229,7 @@ static void impl_Server_getUserNames(const ::Murmur::AMD_Server_getUserNamesPtr
 	cb->ice_response(nm);
 }
 
+#define ACCESS_Server_getUserIds_READ
 static void impl_Server_getUserIds(const ::Murmur::AMD_Server_getUserIdsPtr cb, int server_id,  const ::Murmur::NameList& names) {
 	NEED_SERVER;
 	::Murmur::IdMap im;
@@ -1255,13 +1280,14 @@ static void impl_Server_updateRegistration(const ::Murmur::AMD_Server_updateRegi
 	if (info.contains(ServerDB::User_Comment)) {
 		foreach(ServerUser *u, server->qhUsers) {
 			if (u->iId == id)
-				server->setUserState(u, u->cChannel, u->bMute, u->bDeaf, u->bSuppress, info.value(ServerDB::User_Comment));
+				server->setUserState(u, u->cChannel, u->bMute, u->bDeaf, u->bSuppress, u->bPrioritySpeaker, info.value(ServerDB::User_Comment));
 		}
 	}
 
 	cb->ice_response();
 }
 
+#define ACCESS_Server_getRegistration_READ
 static void impl_Server_getRegistration(const ::Murmur::AMD_Server_getRegistrationPtr cb, int server_id,  ::Ice::Int userid) {
 	NEED_SERVER;
 
@@ -1277,6 +1303,7 @@ static void impl_Server_getRegistration(const ::Murmur::AMD_Server_getRegistrati
 	cb->ice_response(im);
 }
 
+#define ACCESS_Server_getRegisteredUsers_READ
 static void impl_Server_getRegisteredUsers(const ::Murmur::AMD_Server_getRegisteredUsersPtr cb, int server_id,  const ::std::string& filter) {
 	NEED_SERVER;
 	Murmur::NameMap rpl;
@@ -1290,12 +1317,14 @@ static void impl_Server_getRegisteredUsers(const ::Murmur::AMD_Server_getRegiste
 	cb->ice_response(rpl);
 }
 
+#define ACCESS_Server_verifyPassword_READ
 static void impl_Server_verifyPassword(const ::Murmur::AMD_Server_verifyPasswordPtr cb, int server_id,  const ::std::string& name,  const ::std::string& pw) {
 	NEED_SERVER;
 	QString uname = u8(name);
 	cb->ice_response(server->authenticate(uname, u8(pw)));
 }
 
+#define ACCESS_Server_getTexture_READ
 static void impl_Server_getTexture(const ::Murmur::AMD_Server_getTexturePtr cb, int server_id,  ::Ice::Int userid) {
 	NEED_SERVER;
 
@@ -1309,8 +1338,8 @@ static void impl_Server_getTexture(const ::Murmur::AMD_Server_getTexturePtr cb,
 	::Murmur::Texture tex;
 	tex.resize(qba.size());
 	const char *ptr = qba.constData();
-	for (int i=0;i<qba.size()-4;++i)
-		tex[i] = ptr[i+4];
+	for (int i=0;i<qba.size();++i)
+		tex[i] = ptr[i];
 
 	cb->ice_response(tex);
 }
@@ -1348,6 +1377,7 @@ static void impl_Server_setTexture(const ::Murmur::AMD_Server_setTexturePtr cb,
 	}
 }
 
+#define ACCESS_Server_getUptime_READ
 static void impl_Server_getUptime(const ::Murmur::AMD_Server_getUptimePtr cb, int server_id) {
 	NEED_SERVER;
 	cb->ice_response(static_cast<int>(server->tUptime.elapsed()/1000000LL));
@@ -1411,6 +1441,12 @@ static void impl_Server_redirectWhisperGroup(const ::Murmur::AMD_Server_redirect
 	cb->ice_response();
 }
 
+#define ACCESS_Meta_getSliceChecksums_ALL
+static void impl_Meta_getSliceChecksums(const ::Murmur::AMD_Meta_getSliceChecksumsPtr cb, const Ice::ObjectAdapterPtr adapter) {
+	cb->ice_response(::Ice::sliceChecksums());
+}
+
+#define ACCESS_Meta_getServer_READ
 static void impl_Meta_getServer(const ::Murmur::AMD_Meta_getServerPtr cb, const Ice::ObjectAdapterPtr adapter, ::Ice::Int id) {
 	QList<int> server_list = ServerDB::getAllServers();
 	if (! server_list.contains(id))
@@ -1423,6 +1459,7 @@ static void impl_Meta_newServer(const ::Murmur::AMD_Meta_newServerPtr cb, const
 	cb->ice_response(idToProxy(ServerDB::addServer(), adapter));
 }
 
+#define ACCESS_Meta_getAllServers_READ
 static void impl_Meta_getAllServers(const ::Murmur::AMD_Meta_getAllServersPtr cb, const Ice::ObjectAdapterPtr adapter) {
 	::Murmur::ServerList sl;
 
@@ -1431,6 +1468,7 @@ static void impl_Meta_getAllServers(const ::Murmur::AMD_Meta_getAllServersPtr cb
 	cb->ice_response(sl);
 }
 
+#define ACCESS_Meta_getDefaultConf_READ
 static void impl_Meta_getDefaultConf(const ::Murmur::AMD_Meta_getDefaultConfPtr cb, const Ice::ObjectAdapterPtr) {
 	::Murmur::ConfigMap cm;
 	QMap<QString, QString>::const_iterator i;
@@ -1440,6 +1478,7 @@ static void impl_Meta_getDefaultConf(const ::Murmur::AMD_Meta_getDefaultConfPtr
 	cb->ice_response(cm);
 }
 
+#define ACCESS_Meta_getBootedServers_READ
 static void impl_Meta_getBootedServers(const ::Murmur::AMD_Meta_getBootedServersPtr cb, const Ice::ObjectAdapterPtr adapter) {
 	::Murmur::ServerList sl;
 
@@ -1448,6 +1487,7 @@ static void impl_Meta_getBootedServers(const ::Murmur::AMD_Meta_getBootedServers
 	cb->ice_response(sl);
 }
 
+#define ACCESS_Meta_getVersion_ALL
 static void impl_Meta_getVersion(const ::Murmur::AMD_Meta_getVersionPtr cb, const Ice::ObjectAdapterPtr) {
 	int major, minor, patch;
 	QString txt;
@@ -1457,7 +1497,7 @@ static void impl_Meta_getVersion(const ::Murmur::AMD_Meta_getVersionPtr cb, cons
 
 static void impl_Meta_addCallback(const Murmur::AMD_Meta_addCallbackPtr cb, const Ice::ObjectAdapterPtr, const Murmur::MetaCallbackPrx& cbptr) {
 	try {
-		const Murmur::MetaCallbackPrx &oneway = Murmur::MetaCallbackPrx::checkedCast(cbptr->ice_oneway()->ice_connectionCached(false));
+		const Murmur::MetaCallbackPrx &oneway = Murmur::MetaCallbackPrx::checkedCast(cbptr->ice_oneway()->ice_connectionCached(false)->ice_timeout(5000));
 		if (! mi->qmMetaCallbacks.contains(oneway))
 			mi->qmMetaCallbacks.append(oneway);
 		cb->ice_response();
@@ -1468,7 +1508,7 @@ static void impl_Meta_addCallback(const Murmur::AMD_Meta_addCallbackPtr cb, cons
 
 static void impl_Meta_removeCallback(const Murmur::AMD_Meta_removeCallbackPtr cb, const Ice::ObjectAdapterPtr, const Murmur::MetaCallbackPrx& cbptr) {
 	try {
-		const Murmur::MetaCallbackPrx &oneway = Murmur::MetaCallbackPrx::uncheckedCast(cbptr->ice_oneway()->ice_connectionCached(false));
+		const Murmur::MetaCallbackPrx &oneway = Murmur::MetaCallbackPrx::uncheckedCast(cbptr->ice_oneway()->ice_connectionCached(false)->ice_timeout(5000));
 		mi->qmMetaCallbacks.removeAll(oneway);
 		cb->ice_response();
 	} catch (...) {
@@ -1476,6 +1516,7 @@ static void impl_Meta_removeCallback(const Murmur::AMD_Meta_removeCallbackPtr cb
 	}
 }
 
+#define ACCESS_Meta_getUptime_ALL
 static void impl_Meta_getUptime(const ::Murmur::AMD_Meta_getUptimePtr cb, const Ice::ObjectAdapterPtr) {
 	cb->ice_response(static_cast<int>(meta->tUptime.elapsed()/1000000LL));
 }
