@@ -189,7 +189,7 @@ Server::Server(int snum, QObject *p) : QThread(p) {
 	connect(this, SIGNAL(tcpTransmit(QByteArray, unsigned int)), this, SLOT(tcpTransmitData(QByteArray, unsigned int)), Qt::QueuedConnection);
 	connect(this, SIGNAL(reqSync(unsigned int)), this, SLOT(doSync(unsigned int)));
 
-	for (int i=1;i<5000;i++)
+	for (int i=1;i<iMaxUsers*2;++i)
 		qqIds.enqueue(i);
 
 	connect(qtTimeout, SIGNAL(timeout()), this, SLOT(checkTimeout()));
@@ -245,8 +245,6 @@ void Server::stopThread() {
 	if (isRunning()) {
 		log("Ending voice thread");
 
-		foreach(QSocketNotifier *qsn, qlUdpNotifier)
-			qsn->setEnabled(true);
 #ifdef Q_OS_UNIX
 		unsigned char val = 0;
 		::write(aiNotify[1], &val, 1);
@@ -254,6 +252,9 @@ void Server::stopThread() {
 		SetEvent(hNotify);
 #endif
 		wait();
+
+		foreach(QSocketNotifier *qsn, qlUdpNotifier)
+			qsn->setEnabled(true);
 	}
 	qtTimeout->stop();
 }
@@ -298,11 +299,13 @@ void Server::readParams() {
 	iMaxImageMessageLength = Meta::mp.iMaxImageMessageLength;
 	bAllowHTML = Meta::mp.bAllowHTML;
 	iDefaultChan = Meta::mp.iDefaultChan;
+	bRememberChan = Meta::mp.bRememberChan;
 	qsWelcomeText = Meta::mp.qsWelcomeText;
 	qlBind = Meta::mp.qlBind;
 	qsRegName = Meta::mp.qsRegName;
 	qsRegPassword = Meta::mp.qsRegPassword;
 	qsRegHost = Meta::mp.qsRegHost;
+	qsRegLocation = Meta::mp.qsRegLocation;
 	qurlRegWeb = Meta::mp.qurlRegWeb;
 	bBonjour = Meta::mp.bBonjour;
 	bAllowPing = Meta::mp.bAllowPing;
@@ -347,11 +350,13 @@ void Server::readParams() {
 	iMaxImageMessageLength = getConf("imagemessagelength", iMaxImageMessageLength).toInt();
 	bAllowHTML = getConf("allowhtml", bAllowHTML).toBool();
 	iDefaultChan = getConf("defaultchannel", iDefaultChan).toInt();
+	bRememberChan = getConf("rememberchannel", bRememberChan).toBool();
 	qsWelcomeText = getConf("welcometext", qsWelcomeText).toString();
 
 	qsRegName = getConf("registername", qsRegName).toString();
 	qsRegPassword = getConf("registerpassword", qsRegPassword).toString();
 	qsRegHost = getConf("registerhostname", qsRegHost).toString();
+	qsRegLocation = getConf("registerlocation", qsRegLocation).toString();
 	qurlRegWeb = QUrl(getConf("registerurl", qurlRegWeb.toString()).toString());
 	bBonjour = getConf("bonjour", bBonjour).toBool();
 	bAllowPing = getConf("allowping", bAllowPing).toBool();
@@ -376,9 +381,12 @@ void Server::setLiveConf(const QString &key, const QString &value) {
 			mpsc.set_max_bandwidth(length);
 			sendAll(mpsc);
 		}
-	} else if (key == "users")
-		iMaxUsers = i ? i : Meta::mp.iMaxUsers;
-	else if (key == "usersperchannel")
+	} else if (key == "users") {
+		int newmax = i ? i : Meta::mp.iMaxUsers;
+		for (int i=iMaxUsers*2;i<newmax*2;++i)
+			qqIds.enqueue(i);
+		iMaxUsers = newmax;
+	} else if (key == "usersperchannel")
 		iMaxUsersPerChannel = i ? i : Meta::mp.iMaxUsersPerChannel;
 	else if (key == "textmessagelength") {
 		int length = i ? i : Meta::mp.iMaxTextMessageLength;
@@ -406,6 +414,8 @@ void Server::setLiveConf(const QString &key, const QString &value) {
 		}
 	} else if (key == "defaultchannel")
 		iDefaultChan = i ? i : Meta::mp.iDefaultChan;
+	else if (key == "rememberchannel")
+		bRememberChan = !v.isNull() ? QVariant(v).toBool() : Meta::mp.bRememberChan;
 	else if (key == "welcometext") {
 		QString text = !v.isNull() ? v : Meta::mp.qsWelcomeText;
 		if (text != qsWelcomeText) {
@@ -431,6 +441,8 @@ void Server::setLiveConf(const QString &key, const QString &value) {
 		qsRegPassword = !v.isNull() ? v : Meta::mp.qsRegPassword;
 	else if (key == "registerhostname")
 		qsRegHost = !v.isNull() ? v : Meta::mp.qsRegHost;
+	else if (key == "registerlocation")
+		qsRegLocation = !v.isNull() ? v : Meta::mp.qsRegLocation;
 	else if (key == "registerurl")
 		qurlRegWeb = !v.isNull() ? v : Meta::mp.qurlRegWeb;
 	else if (key == "certrequired")
@@ -736,7 +748,7 @@ void Server::sendMessage(ServerUser *u, const char *data, int len, QByteArray &c
 		}
 
 void Server::processMsg(ServerUser *u, const char *data, int len) {
-	if (u->sState != ServerUser::Authenticated || u->bMute || u->bSuppress)
+	if (u->sState != ServerUser::Authenticated || u->bMute || u->bSuppress || u->bSelfMute)
 		return;
 
 	User *p;
@@ -773,11 +785,11 @@ void Server::processMsg(ServerUser *u, const char *data, int len) {
 
 	len = pds.size() + 1;
 
-	if (target == 0x1f) {
+	if (target == 0x1f) { // Server loopback
 		buffer[0] = static_cast<char>(type | 0);
 		sendMessage(u, buffer, len, qba);
 		return;
-	} else if (target == 0) {
+	} else if (target == 0) { // Normal speech
 		buffer[0] = static_cast<char>(type | 0);
 		foreach(p, c->qlUsers) {
 			ServerUser *pDst = static_cast<ServerUser *>(p);
@@ -799,7 +811,7 @@ void Server::processMsg(ServerUser *u, const char *data, int len) {
 				}
 			}
 		}
-	} else if (u->qmTargets.contains(target)) {
+	} else if (u->qmTargets.contains(target)) { // Whisper
 		QSet<ServerUser *> channel;
 		QSet<ServerUser *> direct;
 
@@ -943,6 +955,7 @@ void Server::newClient() {
 		sock->addCaCertificates(qlCA);
 
 		if (qqIds.isEmpty()) {
+			log(QString("Session ID pool (%1) empty, rejecting connection").arg(iMaxUsers));
 			sock->disconnectFromHost();
 			sock->deleteLater();
 			return;
@@ -1041,6 +1054,10 @@ void Server::connectionClosed(QAbstractSocket::SocketError err, const QString &r
 	Connection *c = qobject_cast<Connection *>(sender());
 	if (! c)
 		return;
+	if (c->bDisconnectedEmitted)
+		return;
+	c->bDisconnectedEmitted = true;
+
 	ServerUser *u = static_cast<ServerUser *>(c);
 
 	log(u, QString("Connection closed: %1 [%2]").arg(reason).arg(err));
@@ -1072,10 +1089,11 @@ void Server::connectionClosed(QAbstractSocket::SocketError err, const QString &r
 	if (old && old->bTemporary && old->qlUsers.isEmpty())
 		QCoreApplication::instance()->postEvent(this, new ExecEvent(boost::bind(&Server::removeChannel, this, old->iId)));
 
-	qqIds.enqueue(u->uiSession);
+	qqIds.enqueue(u->uiSession); // Reinsert session id into pool
 
-	if (u->sState == ServerUser::Authenticated)
-		clearACLCache(u);
+	if (u->sState == ServerUser::Authenticated) {
+		clearTempGroups(u); // Also clears ACL cache
+	}
 
 	u->deleteLater();
 
@@ -1088,6 +1106,10 @@ void Server::message(unsigned int uiType, const QByteArray &qbaMsg, ServerUser *
 		u = static_cast<ServerUser *>(sender());
 	}
 
+	if (u->sState == ServerUser::Authenticated) {
+		u->resetActivityTime();
+	}
+
 	if (uiType == MessageHandler::UDPTunnel) {
 		int l = qbaMsg.size();
 		if (l < 2)
@@ -1189,15 +1211,15 @@ void Server::sendProtoMessage(ServerUser *u, const ::google::protobuf::Message &
 	u->sendMessage(msg, msgType, cache);
 }
 
-void Server::sendProtoAll(const ::google::protobuf::Message &msg, unsigned int msgType, unsigned int minversion) {
-	sendProtoExcept(NULL, msg, msgType, minversion);
+void Server::sendProtoAll(const ::google::protobuf::Message &msg, unsigned int msgType, unsigned int version) {
+	sendProtoExcept(NULL, msg, msgType, version);
 }
 
-void Server::sendProtoExcept(ServerUser *u, const ::google::protobuf::Message &msg, unsigned int msgType, unsigned int minversion) {
+void Server::sendProtoExcept(ServerUser *u, const ::google::protobuf::Message &msg, unsigned int msgType, unsigned int version) {
 	QByteArray cache;
 	foreach(ServerUser *usr, qhUsers)
 		if ((usr != u) && (usr->sState == ServerUser::Authenticated))
-			if ((minversion == 0) || (usr->uiVersion >= minversion) || ((minversion & 0x80000000) && (usr->uiVersion < (~minversion))))
+			if ((version == 0) || (usr->uiVersion >= version) || ((version & 0x80000000) && (usr->uiVersion < (~version))))
 				usr->sendMessage(msg, msgType, cache);
 }
 
@@ -1292,8 +1314,8 @@ bool Server::unregisterUser(int id) {
 	return true;
 }
 
-void Server::userEnterChannel(User *p, Channel *c, MumbleProto::UserState &mpus, bool quiet) {
-	if (quiet && (p->cChannel == c))
+void Server::userEnterChannel(User *p, Channel *c, MumbleProto::UserState &mpus) {
+	if (p->cChannel == c)
 		return;
 
 	Channel *old = p->cChannel;
@@ -1305,9 +1327,6 @@ void Server::userEnterChannel(User *p, Channel *c, MumbleProto::UserState &mpus,
 
 	clearACLCache(p);
 
-	if (quiet)
-		return;
-
 	setLastChannel(p);
 
 	bool mayspeak = hasPermission(static_cast<ServerUser *>(p), c, ChanACL::Speak);
@@ -1413,8 +1432,7 @@ void Server::clearACLCache(User *p) {
 
 		if (p) {
 			ChanACL::ChanCache *h = acCache.take(p);
-			if (h)
-				delete h;
+			delete h;
 
 			flushClientPermissionCache(static_cast<ServerUser *>(p), mppq);
 		} else {
@@ -1458,8 +1476,8 @@ bool Server::validateChannelName(const QString &name) {
 }
 
 void Server::recheckCodecVersions() {
-	QMap<int, unsigned int> qmCodecUsercount;
-	QMap<int, unsigned int>::const_iterator i;
+	QMap<int, int> qmCodecUsercount;
+	QMap<int, int>::const_iterator i;
 	int users = 0;
 
 	// Count how many users use which codec
@@ -1497,7 +1515,7 @@ void Server::recheckCodecVersions() {
 	// it as alpha and announce it. If another codec now got the
 	// majority set it as the opposite of the currently valid bPreferAlpha
 	// and announce it.
-	if (version == static_cast<qint32>(0x8000000a))
+	if (version == static_cast<qint32>(0x8000000b))
 		bPreferAlpha = true;
 	else
 		bPreferAlpha = ! bPreferAlpha;
@@ -1507,8 +1525,8 @@ void Server::recheckCodecVersions() {
 	else
 		iCodecBeta = version;
 
-	mpcv.set_alpha(version);
-	mpcv.set_beta(version);
+	mpcv.set_alpha(iCodecAlpha);
+	mpcv.set_beta(iCodecBeta);
 	mpcv.set_prefer_alpha(bPreferAlpha);
 	sendAll(mpcv);
 
