@@ -108,6 +108,7 @@ class Server : public QThread {
 		int iMaxUsers;
 		int iMaxUsersPerChannel;
 		int iDefaultChan;
+		bool bRememberChan;
 		int iMaxTextMessageLength;
 		int iMaxImageMessageLength;
 		bool bAllowHTML;
@@ -118,6 +119,7 @@ class Server : public QThread {
 		QString qsRegName;
 		QString qsRegPassword;
 		QString qsRegHost;
+		QString qsRegLocation;
 		QUrl qurlRegWeb;
 		bool bBonjour;
 		bool bAllowPing;
@@ -217,6 +219,8 @@ class Server : public QThread {
 		void sendProtoExcept(ServerUser *, const ::google::protobuf::Message &msg, unsigned int msgType, unsigned int minversion);
 		void sendProtoMessage(ServerUser *, const ::google::protobuf::Message &msg, unsigned int msgType);
 
+		// sendAll sends a protobuf message to all users on the server whose version is either bigger than v or
+		// lower than ~v. If v == 0 the message is sent to everyone.
 #define MUMBLE_MH_MSG(x) \
 		void sendAll(const MumbleProto:: x &msg, unsigned int v = 0) { sendProtoAll(msg, MessageHandler:: x, v); } \
 		void sendExcept(ServerUser *u, const MumbleProto:: x &msg, unsigned int v = 0) { sendProtoExcept(u, msg, MessageHandler:: x, v); } \
@@ -238,7 +242,7 @@ class Server : public QThread {
 
 		void removeChannel(int id);
 		void removeChannel(Channel *c, Channel *dest = NULL);
-		void userEnterChannel(User *u, Channel *c, MumbleProto::UserState &mpus, bool quiet = false);
+		void userEnterChannel(User *u, Channel *c, MumbleProto::UserState &mpus);
 		bool unregisterUser(int id);
 
 		Server(int snum, QObject *parent = NULL);
@@ -250,6 +254,7 @@ class Server : public QThread {
 		void connectListener(QObject *p);
 		void disconnectListener(QObject *p);
 		void setTempGroups(const int userid, Channel *cChannel, const QStringList &groups);
+		void clearTempGroups(User *user, Channel *cChannel = NULL, bool recurse = true);
 	signals:
 		void registerUserSig(int &, const QMap<int, QString> &);
 		void unregisterUserSig(int &, int);
@@ -271,7 +276,7 @@ class Server : public QThread {
 
 		void contextAction(const User *, const QString &, unsigned int, int);
 	public:
-		void setUserState(User *p, Channel *parent, bool mute, bool deaf, bool suppressed, const QString &comment = QString());
+		void setUserState(User *p, Channel *parent, bool mute, bool deaf, bool suppressed, bool prioritySpeaker, const QString &comment = QString());
 		bool setChannelState(Channel *c, Channel *parent, const QString &qsName, const QSet<Channel *> &links, const QString &desc = QString(), const int position = 0);
 		void sendTextMessage(Channel *cChannel, ServerUser *pUser, bool tree, const QString &text);
 
