@@ -63,14 +63,15 @@ class TransactionHolder {
 
 QSqlDatabase *ServerDB::db;
 Timer ServerDB::tLogClean;
+QString ServerDB::qsUpgradeSuffix;
 
 ServerDB::ServerDB() {
 	if (! QSqlDatabase::isDriverAvailable(Meta::mp.qsDBDriver)) {
 		qFatal("ServerDB: Database driver %s not available", qPrintable(Meta::mp.qsDBDriver));
 	}
 	db = new QSqlDatabase(QSqlDatabase::addDatabase(Meta::mp.qsDBDriver));
-	QStringList datapaths;
-	int i;
+
+	qsUpgradeSuffix = QString::fromLatin1("_old_%1").arg(QDateTime::currentDateTime().toTime_t());
 
 	bool found = false;
 
@@ -79,6 +80,9 @@ ServerDB::ServerDB() {
 			db->setDatabaseName(Meta::mp.qsDatabase);
 			found = db->open();
 		} else {
+			QStringList datapaths;
+			int i;
+
 			datapaths << Meta::mp.qdBasePath.absolutePath();
 			datapaths << QDir::currentPath();
 			datapaths << QCoreApplication::instance()->applicationDirPath();
@@ -134,27 +138,35 @@ ServerDB::ServerDB() {
 	if (Meta::mp.qsDBDriver == "QSQLITE")
 		SQLDO("CREATE TABLE IF NOT EXISTS `%1meta` (`keystring` TEXT PRIMARY KEY, `value` TEXT)");
 	else
-		SQLDO("CREATE TABLE IF NOT EXISTS `%1meta`(`keystring` varchar(255) PRIMARY KEY, `value` varchar(255)) Type=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci");
+		SQLDO("CREATE TABLE IF NOT EXISTS `%1meta`(`keystring` varchar(255) PRIMARY KEY, `value` varchar(255)) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin");
 
 	SQLDO("SELECT `value` FROM `%1meta` WHERE `keystring` = 'version'");
 	if (query.next())
 		version = query.value(0).toInt();
 
-	if (version < 4) {
+	if (version < 5) {
 		if (version > 0) {
 			qWarning("Renaming old tables...");
-			SQLDO("ALTER TABLE `%1servers` RENAME TO `%1servers_old`");
+			SQLDO("ALTER TABLE `%1servers` RENAME TO `%1servers%2`");
 			if (version < 2)
 				SQLMAY("ALTER TABLE `%1log` RENAME TO `%1slog`");
-			SQLDO("ALTER TABLE `%1slog` RENAME TO `%1slog_old`");
-			SQLDO("ALTER TABLE `%1config` RENAME TO `%1config_old`");
-			SQLDO("ALTER TABLE `%1channels` RENAME TO `%1channels_old`");
-			SQLDO("ALTER TABLE `%1players` RENAME TO `%1players_old`");
-			SQLDO("ALTER TABLE `%1groups` RENAME TO `%1groups_old`");
-			SQLDO("ALTER TABLE `%1group_members` RENAME TO `%1group_members_old`");
-			SQLDO("ALTER TABLE `%1acl` RENAME TO `%1acl_old`");
-			SQLDO("ALTER TABLE `%1channel_links` RENAME TO `%1channel_links_old`");
-			SQLDO("ALTER TABLE `%1bans` RENAME TO `%1bans_old`");
+			SQLDO("ALTER TABLE `%1slog` RENAME TO `%1slog%2`");
+			SQLDO("ALTER TABLE `%1config` RENAME TO `%1config%2`");
+			SQLDO("ALTER TABLE `%1channels` RENAME TO `%1channels%2`");
+			if (version < 4)
+				SQLDO("ALTER TABLE `%1players` RENAME TO `%1players%2`");
+			else
+				SQLDO("ALTER TABLE `%1users` RENAME TO `%1users%2`");
+			SQLDO("ALTER TABLE `%1groups` RENAME TO `%1groups%2`");
+			SQLDO("ALTER TABLE `%1group_members` RENAME TO `%1group_members%2`");
+			SQLDO("ALTER TABLE `%1acl` RENAME TO `%1acl%2`");
+			SQLDO("ALTER TABLE `%1channel_links` RENAME TO `%1channel_links%2`");
+			SQLDO("ALTER TABLE `%1bans` RENAME TO `%1bans%2`");
+
+			if (version >= 4) {
+				SQLDO("ALTER TABLE `%1user_info` RENAME TO `%1user_info%2`");
+				SQLDO("ALTER TABLE `%1channel_info` RENAME TO `%1channel_info%2`");
+			}
 		}
 
 		qWarning("Generating new tables...");
@@ -167,13 +179,19 @@ ServerDB::ServerDB() {
 				SQLDO("DROP TRIGGER IF EXISTS `%1config_server_del`");
 				SQLDO("DROP TRIGGER IF EXISTS `%1channels_parent_del`");
 				SQLDO("DROP TRIGGER IF EXISTS `%1channels_server_del`");
+				SQLDO("DROP TRIGGER IF EXISTS `%1channel_info_del_channel`");
 				SQLDO("DROP TRIGGER IF EXISTS `%1players_server_del`");
 				SQLDO("DROP TRIGGER IF EXISTS `%1players_update_timestamp`");
+				SQLDO("DROP TRIGGER IF EXISTS `%1users_server_del`");
+				SQLDO("DROP TRIGGER IF EXISTS `%1users_update_timestamp`");
+				SQLDO("DROP TRIGGER IF EXISTS `%1user_info_del_user`");
 				SQLDO("DROP TRIGGER IF EXISTS `%1groups_del_channel`");
 				SQLDO("DROP TRIGGER IF EXISTS `%1groups_members_del_group`");
 				SQLDO("DROP TRIGGER IF EXISTS `%1groups_members_del_player`");
+				SQLDO("DROP TRIGGER IF EXISTS `%1groups_members_del_user`");
 				SQLDO("DROP TRIGGER IF EXISTS `%1acl_del_channel`");
 				SQLDO("DROP TRIGGER IF EXISTS `%1acl_del_player`");
+				SQLDO("DROP TRIGGER IF EXISTS `%1acl_del_user`");
 				SQLDO("DROP TRIGGER IF EXISTS `%1channel_links_del_channel`");
 				SQLDO("DROP TRIGGER IF EXISTS `%1bans_del_server`");
 
@@ -184,6 +202,9 @@ ServerDB::ServerDB() {
 				SQLDO("DROP INDEX IF EXISTS `%1channel_info_id`");
 				SQLDO("DROP INDEX IF EXISTS `%1players_name`");
 				SQLDO("DROP INDEX IF EXISTS `%1players_id`");
+				SQLDO("DROP INDEX IF EXISTS `%1users_name`");
+				SQLDO("DROP INDEX IF EXISTS `%1users_id`");
+				SQLDO("DROP INDEX IF EXISTS `%1user_info_id`");
 				SQLDO("DROP INDEX IF EXISTS `%1groups_name_channels`");
 				SQLDO("DROP INDEX IF EXISTS `%1acl_channel_pri`");
 			}
@@ -250,7 +271,8 @@ ServerDB::ServerDB() {
 					qlForeignKeys << qsp(query.value(0).toString(), query.value(1).toString());
 
 				foreach(const qsp &key, qlForeignKeys) {
-					ServerDB::exec(query, QString::fromLatin1("ALTER TABLE `%1` DROP FOREIGN KEY `%2`").arg(key.first).arg(key.second), true);
+					if (key.first.startsWith(Meta::mp.qsDBPrefix))
+						ServerDB::exec(query, QString::fromLatin1("ALTER TABLE `%1` DROP FOREIGN KEY `%2`").arg(key.first).arg(key.second), true);
 				}
 
 
@@ -261,7 +283,8 @@ ServerDB::ServerDB() {
 					qlIndexes << qsp(query.value(0).toString(), query.value(1).toString());
 
 				foreach(const qsp &key, qlIndexes) {
-					ServerDB::exec(query, QString::fromLatin1("ALTER TABLE `%1` DROP INDEX `%2`").arg(key.first).arg(key.second), true);
+					if (key.first.startsWith(Meta::mp.qsDBPrefix))
+						ServerDB::exec(query, QString::fromLatin1("ALTER TABLE `%1` DROP INDEX `%2`").arg(key.first).arg(key.second), true);
 				}
 
 				qlIndexes.clear();
@@ -273,129 +296,162 @@ ServerDB::ServerDB() {
 					qlIndexes << qsp(query.value(0).toString(), query.value(1).toString());
 
 				foreach(const qsp &key, qlIndexes) {
-					ServerDB::exec(query, QString::fromLatin1("ALTER TABLE `%1` DROP INDEX `%2`").arg(key.first).arg(key.second), true);
+					if (key.first.startsWith(Meta::mp.qsDBPrefix))
+						ServerDB::exec(query, QString::fromLatin1("ALTER TABLE `%1` DROP INDEX `%2`").arg(key.first).arg(key.second), true);
 				}
 			}
-			SQLDO("CREATE TABLE `%1servers`(`server_id` INTEGER PRIMARY KEY AUTO_INCREMENT) Type=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci");
+			SQLDO("CREATE TABLE `%1servers`(`server_id` INTEGER PRIMARY KEY AUTO_INCREMENT) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin");
 
-			SQLDO("CREATE TABLE `%1slog`(`server_id` INTEGER NOT NULL, `msg` TEXT, `msgtime` TIMESTAMP) Type=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci");
+			SQLDO("CREATE TABLE `%1slog`(`server_id` INTEGER NOT NULL, `msg` TEXT, `msgtime` TIMESTAMP) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin");
 			SQLDO("CREATE INDEX `%1slog_time` ON `%1slog`(`msgtime`)");
 			SQLDO("ALTER TABLE `%1slog` ADD CONSTRAINT `%1slog_server_del` FOREIGN KEY (`server_id`) REFERENCES `%1servers`(`server_id`) ON DELETE CASCADE");
 
-			SQLDO("CREATE TABLE `%1config` (`server_id` INTEGER NOT NULL, `key` varchar(255), `value` TEXT) Type=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci");
+			SQLDO("CREATE TABLE `%1config` (`server_id` INTEGER NOT NULL, `key` varchar(255), `value` TEXT) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin");
 			SQLDO("CREATE UNIQUE INDEX `%1config_key` ON `%1config`(`server_id`, `key`)");
 			SQLDO("ALTER TABLE `%1config` ADD CONSTRAINT `%1config_server_del` FOREIGN KEY (`server_id`) REFERENCES `%1servers`(`server_id`) ON DELETE CASCADE");
 
-			SQLDO("CREATE TABLE `%1channels` (`server_id` INTEGER NOT NULL, `channel_id` INTEGER NOT NULL, `parent_id` INTEGER, `name` varchar(255), `inheritacl` INTEGER) Type=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci");
+			SQLDO("CREATE TABLE `%1channels` (`server_id` INTEGER NOT NULL, `channel_id` INTEGER NOT NULL, `parent_id` INTEGER, `name` varchar(255), `inheritacl` INTEGER) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin");
 			SQLDO("CREATE UNIQUE INDEX `%1channel_id` ON `%1channels`(`server_id`, `channel_id`)");
 			SQLDO("ALTER TABLE `%1channels` ADD CONSTRAINT `%1channels_parent_del` FOREIGN KEY (`server_id`, `parent_id`) REFERENCES `%1channels`(`server_id`,`channel_id`) ON DELETE CASCADE");
 			SQLDO("ALTER TABLE `%1channels` ADD CONSTRAINT `%1channels_server_del` FOREIGN KEY (`server_id`) REFERENCES `%1servers`(`server_id`) ON DELETE CASCADE");
 
-			SQLDO("CREATE TABLE `%1channel_info` (`server_id` INTEGER NOT NULL, `channel_id` INTEGER NOT NULL, `key` INTEGER, `value` LONGTEXT) Type=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci");
+			SQLDO("CREATE TABLE `%1channel_info` (`server_id` INTEGER NOT NULL, `channel_id` INTEGER NOT NULL, `key` INTEGER, `value` LONGTEXT) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin");
 			SQLDO("CREATE UNIQUE INDEX `%1channel_info_id` ON `%1channel_info`(`server_id`, `channel_id`, `key`)");
 			SQLDO("ALTER TABLE `%1channel_info` ADD CONSTRAINT `%1channel_info_del_channel` FOREIGN KEY (`server_id`, `channel_id`) REFERENCES `%1channels`(`server_id`,`channel_id`) ON DELETE CASCADE");
 
-			SQLDO("CREATE TABLE `%1users` (`server_id` INTEGER NOT NULL, `user_id` INTEGER NOT NULL, `name` varchar(255), `pw` varchar(128), `lastchannel` INTEGER, `texture` LONGBLOB, `last_active` TIMESTAMP) Type=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci");
+			SQLDO("CREATE TABLE `%1users` (`server_id` INTEGER NOT NULL, `user_id` INTEGER NOT NULL, `name` varchar(255), `pw` varchar(128), `lastchannel` INTEGER, `texture` LONGBLOB, `last_active` TIMESTAMP) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin");
 			SQLDO("CREATE INDEX `%1users_channel` ON `%1users`(`server_id`, `lastchannel`)");
 			SQLDO("CREATE UNIQUE INDEX `%1users_name` ON `%1users` (`server_id`,`name`)");
 			SQLDO("CREATE UNIQUE INDEX `%1users_id` ON `%1users` (`server_id`, `user_id`)");
 			SQLDO("ALTER TABLE `%1users` ADD CONSTRAINT `%1users_server_del` FOREIGN KEY (`server_id`) REFERENCES `%1servers`(`server_id`) ON DELETE CASCADE");
 
-			SQLDO("CREATE TABLE `%1user_info` (`server_id` INTEGER NOT NULL, `user_id` INTEGER NOT NULL, `key` INTEGER, `value` LONGTEXT) Type=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci");
+			SQLDO("CREATE TABLE `%1user_info` (`server_id` INTEGER NOT NULL, `user_id` INTEGER NOT NULL, `key` INTEGER, `value` LONGTEXT) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin");
 			SQLDO("CREATE UNIQUE INDEX `%1user_info_id` ON `%1user_info`(`server_id`, `user_id`, `key`)");
 			SQLDO("ALTER TABLE `%1user_info` ADD CONSTRAINT `%1user_info_del_user` FOREIGN KEY (`server_id`, `user_id`) REFERENCES `%1users`(`server_id`,`user_id`) ON DELETE CASCADE");
 
-			SQLDO("CREATE TABLE `%1groups` (`group_id` INTEGER PRIMARY KEY AUTO_INCREMENT, `server_id` INTEGER NOT NULL, `name` varchar(255), `channel_id` INTEGER NOT NULL, `inherit` INTEGER, `inheritable` INTEGER) Type=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci");
+			SQLDO("CREATE TABLE `%1groups` (`group_id` INTEGER PRIMARY KEY AUTO_INCREMENT, `server_id` INTEGER NOT NULL, `name` varchar(255), `channel_id` INTEGER NOT NULL, `inherit` INTEGER, `inheritable` INTEGER) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin");
 			SQLDO("CREATE UNIQUE INDEX `%1groups_name_channels` ON `%1groups`(`server_id`, `channel_id`, `name`)");
 			SQLDO("ALTER TABLE `%1groups` ADD CONSTRAINT `%1groups_del_channel` FOREIGN KEY (`server_id`, `channel_id`) REFERENCES `%1channels`(`server_id`, `channel_id`) ON DELETE CASCADE");
 
-			SQLDO("CREATE TABLE `%1group_members` (`group_id` INTEGER NOT NULL, `server_id` INTEGER NOT NULL, `user_id` INTEGER NOT NULL, `addit` INTEGER) Type=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci");
+			SQLDO("CREATE TABLE `%1group_members` (`group_id` INTEGER NOT NULL, `server_id` INTEGER NOT NULL, `user_id` INTEGER NOT NULL, `addit` INTEGER) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin");
 			SQLDO("CREATE INDEX `%1group_members_users` ON `%1group_members`(`server_id`, `user_id`)");
 			SQLDO("ALTER TABLE `%1group_members` ADD CONSTRAINT `%1group_members_del_group` FOREIGN KEY (`group_id`) REFERENCES `%1groups`(`group_id`) ON DELETE CASCADE");
 			SQLDO("ALTER TABLE `%1group_members` ADD CONSTRAINT `%1group_members_del_user` FOREIGN KEY (`server_id`, `user_id`) REFERENCES `%1users`(`server_id`,`user_id`) ON DELETE CASCADE");
 
-			SQLDO("CREATE TABLE `%1acl` (`server_id` INTEGER NOT NULL, `channel_id` INTEGER NOT NULL, `priority` INTEGER, `user_id` INTEGER, `group_name` varchar(255), `apply_here` INTEGER, `apply_sub` INTEGER, `grantpriv` INTEGER, `revokepriv` INTEGER) Type=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci");
+			SQLDO("CREATE TABLE `%1acl` (`server_id` INTEGER NOT NULL, `channel_id` INTEGER NOT NULL, `priority` INTEGER, `user_id` INTEGER, `group_name` varchar(255), `apply_here` INTEGER, `apply_sub` INTEGER, `grantpriv` INTEGER, `revokepriv` INTEGER) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin");
 			SQLDO("CREATE UNIQUE INDEX `%1acl_channel_pri` ON `%1acl`(`server_id`, `channel_id`, `priority`)");
 			SQLDO("CREATE INDEX `%1acl_user` ON `%1acl`(`server_id`, `user_id`)");
 			SQLDO("ALTER TABLE `%1acl` ADD CONSTRAINT `%1acl_del_channel` FOREIGN KEY (`server_id`, `channel_id`) REFERENCES `%1channels`(`server_id`, `channel_id`) ON DELETE CASCADE");
 			SQLDO("ALTER TABLE `%1acl` ADD CONSTRAINT `%1acl_del_user` FOREIGN KEY (`server_id`, `user_id`) REFERENCES `%1users`(`server_id`, `user_id`) ON DELETE CASCADE");
 
-			SQLDO("CREATE TABLE `%1channel_links` (`server_id` INTEGER NOT NULL, `channel_id` INTEGER NOT NULL, `link_id` INTEGER NOT NULL) Type=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci");
+			SQLDO("CREATE TABLE `%1channel_links` (`server_id` INTEGER NOT NULL, `channel_id` INTEGER NOT NULL, `link_id` INTEGER NOT NULL) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin");
 			SQLDO("ALTER TABLE `%1channel_links` ADD CONSTRAINT `%1channel_links_del_channel` FOREIGN KEY(`server_id`, `channel_id`) REFERENCES `%1channels`(`server_id`, `channel_id`) ON DELETE CASCADE");
 			SQLDO("DELETE FROM `%1channel_links`");
 
-			SQLDO("CREATE TABLE `%1bans` (`server_id` INTEGER NOT NULL, `base` BINARY(16), `mask` INTEGER, `name` varchar(255), `hash` CHAR(40), `reason` TEXT, `start` DATETIME, `duration` INTEGER) Type=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci");
+			SQLDO("CREATE TABLE `%1bans` (`server_id` INTEGER NOT NULL, `base` BINARY(16), `mask` INTEGER, `name` varchar(255), `hash` CHAR(40), `reason` TEXT, `start` DATETIME, `duration` INTEGER) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin");
 			SQLDO("ALTER TABLE `%1bans` ADD CONSTRAINT `%1bans_del_server` FOREIGN KEY(`server_id`) REFERENCES `%1servers`(`server_id`) ON DELETE CASCADE");
 		}
 		if (version == 0) {
 			SQLDO("INSERT INTO `%1servers` (`server_id`) VALUES(1)");
-			SQLDO("INSERT INTO `%1meta` (`keystring`, `value`) VALUES('version','4')");
+			SQLDO("INSERT INTO `%1meta` (`keystring`, `value`) VALUES('version','5')");
 		} else {
 			qWarning("Importing old data...");
 
 			if (Meta::mp.qsDBDriver != "QSQLITE")
 				SQLDO("SET FOREIGN_KEY_CHECKS = 0;");
-			SQLDO("INSERT INTO `%1servers` (`server_id`) SELECT `server_id` FROM `%1servers_old`");
-			SQLDO("INSERT INTO `%1slog` (`server_id`, `msg`, `msgtime`) SELECT `server_id`, `msg`, `msgtime` FROM `%1slog_old`");
-			SQLDO("INSERT INTO `%1config` (`server_id`, `key`, `value`) SELECT `server_id`, `keystring`, `value` FROM `%1config_old`");
-			SQLDO("INSERT INTO `%1channels` (`server_id`, `channel_id`, `parent_id`, `name`, `inheritacl`) SELECT `server_id`, `channel_id`, `parent_id`, `name`, `inheritacl` FROM `%1channels_old` ORDER BY `parent_id`, `channel_id`");
-			SQLDO("INSERT INTO `%1users` (`server_id`, `user_id`, `name`, `pw`, `lastchannel`, `texture`, `last_active`) SELECT `server_id`, `player_id`, `name`, `pw`, `lastchannel`, `texture`, `last_active` FROM `%1players_old`");
-			SQLDO("INSERT INTO `%1groups` (`group_id`, `server_id`, `name`, `channel_id`, `inherit`, `inheritable`) SELECT `group_id`, `server_id`, `name`, `channel_id`, `inherit`, `inheritable` FROM `%1groups_old`");
-			SQLDO("INSERT INTO `%1group_members` (`group_id`, `server_id`, `user_id`, `addit`) SELECT `group_id`, `server_id`, `player_id`, `addit` FROM `%1group_members_old`");
-			SQLDO("INSERT INTO `%1acl` (`server_id`, `channel_id`, `priority`, `user_id`, `group_name`, `apply_here`, `apply_sub`, `grantpriv`, `revokepriv`) SELECT `server_id`, `channel_id`, `priority`, `player_id`, `group_name`, `apply_here`, `apply_sub`, `grantpriv`, `revokepriv` FROM `%1acl_old`");
-			SQLDO("INSERT INTO `%1channel_links` (`server_id`, `channel_id`, `link_id`) SELECT `server_id`, `channel_id`, `link_id` FROM `%1channel_links_old`");
+			SQLDO("INSERT INTO `%1servers` (`server_id`) SELECT `server_id` FROM `%1servers%2`");
+			SQLDO("INSERT INTO `%1slog` (`server_id`, `msg`, `msgtime`) SELECT `server_id`, `msg`, `msgtime` FROM `%1slog%2`");
+
+			if (version < 4)
+				SQLDO("INSERT INTO `%1config` (`server_id`, `key`, `value`) SELECT `server_id`, `keystring`, `value` FROM `%1config%2`");
+			else
+				SQLDO("INSERT INTO `%1config` (`server_id`, `key`, `value`) SELECT `server_id`, `key`, `value` FROM `%1config%2`");
+
+			SQLDO("INSERT INTO `%1channels` (`server_id`, `channel_id`, `parent_id`, `name`, `inheritacl`) SELECT `server_id`, `channel_id`, `parent_id`, `name`, `inheritacl` FROM `%1channels%2` ORDER BY `parent_id`, `channel_id`");
+
+			if (version < 4)
+				SQLDO("INSERT INTO `%1users` (`server_id`, `user_id`, `name`, `pw`, `lastchannel`, `texture`, `last_active`) SELECT `server_id`, `player_id`, `name`, `pw`, `lastchannel`, `texture`, `last_active` FROM `%1players%2`");
+			else
+				SQLDO("INSERT INTO `%1users` (`server_id`, `user_id`, `name`, `pw`, `lastchannel`, `texture`, `last_active`) SELECT `server_id`, `user_id`, `name`, `pw`, `lastchannel`, `texture`, `last_active` FROM `%1users%2`");
+
+			SQLDO("INSERT INTO `%1groups` (`group_id`, `server_id`, `name`, `channel_id`, `inherit`, `inheritable`) SELECT `group_id`, `server_id`, `name`, `channel_id`, `inherit`, `inheritable` FROM `%1groups%2`");
+
+			if (version < 4)
+				SQLDO("INSERT INTO `%1group_members` (`group_id`, `server_id`, `user_id`, `addit`) SELECT `group_id`, `server_id`, `player_id`, `addit` FROM `%1group_members%2`");
+			else
+				SQLDO("INSERT INTO `%1group_members` (`group_id`, `server_id`, `user_id`, `addit`) SELECT `group_id`, `server_id`, `user_id`, `addit` FROM `%1group_members%2`");
+
+			if (version < 4)
+				SQLDO("INSERT INTO `%1acl` (`server_id`, `channel_id`, `priority`, `user_id`, `group_name`, `apply_here`, `apply_sub`, `grantpriv`, `revokepriv`) SELECT `server_id`, `channel_id`, `priority`, `player_id`, `group_name`, `apply_here`, `apply_sub`, `grantpriv`, `revokepriv` FROM `%1acl%2`");
+			else
+				SQLDO("INSERT INTO `%1acl` (`server_id`, `channel_id`, `priority`, `user_id`, `group_name`, `apply_here`, `apply_sub`, `grantpriv`, `revokepriv`) SELECT `server_id`, `channel_id`, `priority`, `user_id`, `group_name`, `apply_here`, `apply_sub`, `grantpriv`, `revokepriv` FROM `%1acl%2`");
+
+			SQLDO("INSERT INTO `%1channel_links` (`server_id`, `channel_id`, `link_id`) SELECT `server_id`, `channel_id`, `link_id` FROM `%1channel_links%2`");
+			if (version < 4) {
+				QList<QList<QVariant> > ql;
+				SQLPREP("SELECT `server_id`, `base`, `mask` FROM `%1bans%2`");
+				SQLEXEC();
+				while (query.next()) {
+					QList<QVariant> l;
+					l << query.value(0);
+					l << query.value(1);
+					l << query.value(2);
+					ql << l;
+				}
+				SQLPREP("INSERT INTO `%1bans` (`server_id`, `base`, `mask`) VALUES (?, ?, ?)");
+				foreach(const QList<QVariant> &l, ql) {
+
+					quint32 addr = htonl(l.at(1).toUInt());
+					const char *ptr = reinterpret_cast<const char *>(&addr);
+
+					QByteArray qba(16, 0);
+					qba[10] = static_cast<char>(-1);
+					qba[11] = static_cast<char>(-1);
+					qba[12] = ptr[0];
+					qba[13] = ptr[1];
+					qba[14] = ptr[2];
+					qba[15] = ptr[3];
+
+					query.addBindValue(l.at(0));
+					query.addBindValue(qba);
+					query.addBindValue(l.at(2).toInt() + 96);
+					SQLEXEC();
+				}
+			} else {
+				SQLDO("INSERT INTO `%1bans` (`server_id`, `base`, `mask`) SELECT `server_id`, `base`, `mask` FROM `%1bans%2`");
+			}
+
+			if (version < 4)
+				SQLDO("INSERT INTO `%1user_info` SELECT `server_id`,`player_id`,1,`email` FROM `%1players%2` WHERE `email` IS NOT NULL");
+
+			if (version == 3) {
+				SQLDO("INSERT INTO `%1channel_info` SELECT `server_id`,`channel_id`,0,`description` FROM `%1channels%2` WHERE `description` IS NOT NULL");
+			}
+
+			if (version >= 4) {
+				SQLDO("INSERT INTO `%1user_info` SELECT * FROM `%1user_info%2`");
+				SQLDO("INSERT INTO `%1channel_info` SELECT * FROM `%1channel_info%2`");
+			}
+
 			if (Meta::mp.qsDBDriver != "QSQLITE")
 				SQLDO("SET FOREIGN_KEY_CHECKS = 1;");
 
-			QList<QList<QVariant> > ql;
-			SQLPREP("SELECT `server_id`, `base`, `mask` FROM `%1bans_old`");
-			SQLEXEC();
-			while (query.next()) {
-				QList<QVariant> l;
-				l << query.value(0);
-				l << query.value(1);
-				l << query.value(2);
-				ql << l;
-			}
-			SQLPREP("INSERT INTO `%1bans` (`server_id`, `base`, `mask`) VALUES (?, ?, ?)");
-			foreach(const QList<QVariant> &l, ql) {
-
-				quint32 addr = htonl(l.at(1).toUInt());
-				const char *ptr = reinterpret_cast<const char *>(&addr);
-
-				QByteArray qba(16, 0);
-				qba[10] = static_cast<char>(-1);
-				qba[11] = static_cast<char>(-1);
-				qba[12] = ptr[0];
-				qba[13] = ptr[1];
-				qba[14] = ptr[2];
-				qba[15] = ptr[3];
-
-				query.addBindValue(l.at(0));
-				query.addBindValue(qba);
-				query.addBindValue(l.at(2).toInt() + 96);
-				SQLEXEC();
-			}
-
-			SQLDO("INSERT INTO `%1user_info` SELECT `server_id`,`player_id`,1,`email` FROM `%1players_old` WHERE `email` IS NOT NULL");
-
-			if (version == 3) {
-				SQLDO("INSERT INTO `%1channel_info` SELECT `server_id`,`channel_id`,0,`description` FROM `%1channels_old` WHERE `description` IS NOT NULL");
-			}
-
 			qWarning("Removing old tables...");
-			SQLDO("DROP TABLE IF EXISTS `%1slog_old`");
-			SQLDO("DROP TABLE IF EXISTS `%1config_old`");
-			SQLDO("DROP TABLE IF EXISTS `%1channels_old`");
-			SQLDO("DROP TABLE IF EXISTS `%1players_old`");
-			SQLDO("DROP TABLE IF EXISTS `%1groups_old`");
-			SQLDO("DROP TABLE IF EXISTS `%1group_members_old`");
-			SQLDO("DROP TABLE IF EXISTS `%1acl_old`");
-			SQLDO("DROP TABLE IF EXISTS `%1channel_links_old`");
-			SQLDO("DROP TABLE IF EXISTS `%1bans_old`");
-			SQLDO("DROP TABLE IF EXISTS `%1servers_old`");
+			SQLDO("DROP TABLE IF EXISTS `%1slog%2`");
+			SQLDO("DROP TABLE IF EXISTS `%1config%2`");
+			SQLDO("DROP TABLE IF EXISTS `%1channel_info%2`");
+			SQLDO("DROP TABLE IF EXISTS `%1channels%2`");
+			SQLDO("DROP TABLE IF EXISTS `%1user_info%2`");
+			SQLDO("DROP TABLE IF EXISTS `%1users%2`");
+			SQLDO("DROP TABLE IF EXISTS `%1players%2`");
+			SQLDO("DROP TABLE IF EXISTS `%1groups%2`");
+			SQLDO("DROP TABLE IF EXISTS `%1group_members%2`");
+			SQLDO("DROP TABLE IF EXISTS `%1acl%2`");
+			SQLDO("DROP TABLE IF EXISTS `%1channel_links%2`");
+			SQLDO("DROP TABLE IF EXISTS `%1bans%2`");
+			SQLDO("DROP TABLE IF EXISTS `%1servers%2`");
 
-			SQLDO("UPDATE `%1meta` SET `value` = '4' WHERE `keystring` = 'version'");
+			SQLDO("UPDATE `%1meta` SET `value` = '5' WHERE `keystring` = 'version'");
 		}
 	}
 	query.clear();
@@ -413,10 +469,14 @@ bool ServerDB::prepare(QSqlQuery &query, const QString &str, bool fatal, bool wa
 		return false;
 	}
 	QString q;
-	if (str.contains(QLatin1String("%1")))
-		q = str.arg(Meta::mp.qsDBPrefix);
-	else
+	if (str.contains(QLatin1String("%1"))) {
+		if (str.contains(QLatin1String("%2")))
+			q = str.arg(Meta::mp.qsDBPrefix, qsUpgradeSuffix);
+		else
+			q = str.arg(Meta::mp.qsDBPrefix);
+	} else {
 		q = str;
+	}
 
 	if (query.prepare(q)) {
 		return true;
@@ -705,19 +765,22 @@ QMap<int, QString> Server::getRegistration(int id) {
 	TransactionHolder th;
 
 	QSqlQuery &query = *th.qsqQuery;
-	SQLPREP("SELECT `name` FROM `%1users` WHERE `server_id` = ? AND `user_id` = ?");
+	SQLPREP("SELECT `name`, `last_active` FROM `%1users` WHERE `server_id` = ? AND `user_id` = ?");
 	query.addBindValue(iServerNum);
 	query.addBindValue(id);
 	SQLEXEC();
 	if (query.next()) {
 		info.insert(ServerDB::User_Name, query.value(0).toString());
+		info.insert(ServerDB::User_LastActive, query.value(1).toString());
 
 		SQLPREP("SELECT `key`, `value` FROM `%1user_info` WHERE `server_id` = ? AND `user_id` = ?");
 		query.addBindValue(iServerNum);
 		query.addBindValue(id);
 		SQLEXEC();
 		while (query.next()) {
-			info.insert(query.value(0).toInt(), query.value(1).toString());
+			const int key = query.value(0).toInt();
+			if (!info.contains(key))
+				info.insert(key, query.value(1).toString());
 		}
 	}
 	return info;
@@ -758,7 +821,7 @@ int Server::authenticate(QString &name, const QString &pw, const QStringList &em
 	TransactionHolder th;
 	QSqlQuery &query = *th.qsqQuery;
 
-	SQLPREP("SELECT `user_id`,`name`,`pw` FROM `%1users` WHERE `server_id` = ? AND `name` like ?");
+	SQLPREP("SELECT `user_id`,`name`,`pw` FROM `%1users` WHERE `server_id` = ? AND LOWER(`name`) = LOWER(?)");
 	query.addBindValue(iServerNum);
 	query.addBindValue(name);
 	SQLEXEC();
@@ -856,6 +919,9 @@ bool Server::setInfo(int id, const QMap<int, QString> &setinfo) {
 	TransactionHolder th;
 	QSqlQuery &query = *th.qsqQuery;
 
+	if (info.contains(ServerDB::User_LastActive)) {
+		info.remove(ServerDB::User_LastActive);
+	}
 	if (info.contains(ServerDB::User_Password)) {
 		const QString &pw = info.value(ServerDB::User_Password);
 		QCryptographicHash hash(QCryptographicHash::Sha1);
@@ -935,10 +1001,22 @@ void ServerDB::setSUPW(int srvnum, const QString &pw) {
 	TransactionHolder th;
 
 	QCryptographicHash hash(QCryptographicHash::Sha1);
-
 	hash.addData(pw.toUtf8());
 
 	QSqlQuery &query = *th.qsqQuery;
+
+	SQLPREP("SELECT `user_id` FROM `%1users` WHERE `server_id` = ? AND `user_id` = ?");
+	query.addBindValue(srvnum);
+	query.addBindValue(0);
+	SQLEXEC();
+	if (! query.next()) {
+		SQLPREP("INSERT INTO `%1users` (`server_id`, `user_id`, `name`) VALUES (?, ?, ?)");
+		query.addBindValue(srvnum);
+		query.addBindValue(0);
+		query.addBindValue(QLatin1String("SuperUser"));
+		SQLEXEC();
+	}
+
 	SQLPREP("UPDATE `%1users` SET `pw`=? WHERE `server_id` = ? AND `user_id`=?");
 	query.addBindValue(QString::fromLatin1(hash.result().toHex()));
 	query.addBindValue(srvnum);
@@ -981,10 +1059,15 @@ int Server::getUserID(const QString &name) {
 		qhUserNameCache.insert(id, name);
 		return id;
 	}
+
+	if (!validateUserName(name)) {
+		return id;
+	}
+
 	TransactionHolder th;
 
 	QSqlQuery &query = *th.qsqQuery;
-	SQLPREP("SELECT `user_id` FROM `%1users` WHERE `server_id` = ? AND `name` like ?");
+	SQLPREP("SELECT `user_id` FROM `%1users` WHERE `server_id` = ? AND LOWER(`name`) = LOWER(?)");
 	query.addBindValue(iServerNum);
 	query.addBindValue(name);
 	SQLEXEC();
@@ -1340,7 +1423,11 @@ void Server::setLastChannel(const User *p) {
 	TransactionHolder th;
 	QSqlQuery &query = *th.qsqQuery;
 
-	SQLPREP("UPDATE `%1users` SET `lastchannel`=? WHERE `server_id` = ? AND `user_id` = ?");
+	if (Meta::mp.qsDBDriver == "QSQLITE") {
+		SQLPREP("UPDATE `%1users` SET `lastchannel`=? WHERE `server_id` = ? AND `user_id` = ?");
+	} else {
+		SQLPREP("UPDATE `%1users` SET `lastchannel`=?, `last_active` = now() WHERE `server_id` = ? AND `user_id` = ?");
+	}
 	query.addBindValue(p->cChannel->iId);
 	query.addBindValue(iServerNum);
 	query.addBindValue(p->iId);
@@ -1416,6 +1503,7 @@ void Server::getBans() {
 		ban.qsHash = query.value(3).toString();
 		ban.qsReason = query.value(4).toString();
 		ban.qdtStart = query.value(5).toDateTime();
+		ban.qdtStart.setTimeSpec(Qt::UTC);
 		ban.iDuration = query.value(6).toInt();
 
 		if (ban.isValid())
@@ -1486,6 +1574,10 @@ void Server::dblog(const QString &str) {
 	TransactionHolder th;
 	QSqlQuery &query = *th.qsqQuery;
 
+	// Is logging disabled?
+	if (Meta::mp.iLogDays < 0)
+		return;
+
 	// Once per hour
 	if (Meta::mp.iLogDays > 0) {
 		if (ServerDB::tLogClean.isElapsed(3600ULL * 1000000ULL)) {
@@ -1520,11 +1612,26 @@ QList<QPair<unsigned int, QString> > ServerDB::getLog(int server_id, unsigned in
 	while (query.next()) {
 		QDateTime qdt = query.value(0).toDateTime();
 		QString msg = query.value(1).toString();
-		ql << QPair<unsigned int, QString>(qdt.toTime_t(), msg);
+		ql << QPair<unsigned int, QString>(qdt.toLocalTime().toTime_t(), msg);
 	}
 	return ql;
 }
 
+int ServerDB::getLogLen(int server_id) {
+	TransactionHolder th;
+	QSqlQuery &query = *th.qsqQuery;
+
+	SQLPREP("SELECT COUNT(`msgtime`) FROM `%1slog` WHERE `server_id` = ?");
+	query.addBindValue(server_id);
+	SQLEXEC();
+
+	while (query.next()) {
+		return query.value(0).toInt();
+	}
+
+	return -1;
+}
+
 void ServerDB::setConf(int server_id, const QString &k, const QVariant &value) {
 	TransactionHolder th;
 
@@ -1593,7 +1700,7 @@ int ServerDB::addServer() {
 	SQLEXEC();
 	int id = 0;
 	if (query.next())
-		id = query.value(0).toInt();
+		id = qMax(1, query.value(0).toInt());
 	SQLPREP("INSERT INTO `%1servers` (`server_id`) VALUES (?)");
 	query.addBindValue(id);
 	SQLEXEC();
