@@ -41,12 +41,13 @@ class Connection;
 class ServerDB {
 	public:
 		enum ChannelInfo { Channel_Description, Channel_Position };
-		enum UserInfo { User_Name, User_Email, User_Comment, User_Hash, User_Password };
+		enum UserInfo { User_Name, User_Email, User_Comment, User_Hash, User_Password, User_LastActive };
 		ServerDB();
 		~ServerDB();
 		typedef QPair<unsigned int, QString> LogRecord;
 		static Timer tLogClean;
 		static QSqlDatabase *db;
+		static QString qsUpgradeSuffix;
 		static void setSUPW(int iServNum, const QString &pw);
 		static QList<int> getBootServers();
 		static QList<int> getAllServers();
@@ -57,6 +58,7 @@ class ServerDB {
 		static QVariant getConf(int server_id, const QString &key, QVariant def = QVariant());
 		static void setConf(int server_id, const QString &key, const QVariant &value = QVariant());
 		static QList<LogRecord> getLog(int server_id, unsigned int offs_min, unsigned int offs_max);
+		static int getLogLen(int server_id);
 		static bool prepare(QSqlQuery &, const QString &, bool fatal = true, bool warn = true);
 		static bool exec(QSqlQuery &, const QString &str = QString(), bool fatal= true, bool warn = true);
 		static bool execBatch(QSqlQuery &, const QString &str = QString(), bool fatal= true);
