@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -30,6 +30,7 @@
 
 #include "Tray.h"
 #include "Server.h"
+#include "Meta.h"
 #include "Version.h"
 
 Tray::Tray(QObject *p, LogEmitter *logger) : QObject(p) {
@@ -50,9 +51,9 @@ Tray::Tray(QObject *p, LogEmitter *logger) : QObject(p) {
 	// Can't construct a QMenu which decends from QObject, and qsti is a QObject.
 	// Qt bug?
 	qm = new QMenu(tr("Murmur"), NULL);
-	qm->addAction(qaQuit);
-	qm->addSeparator();
 	qm->addAction(qaShowLog);
+	qm->addSeparator();
+	qm->addAction(qaQuit);
 	qsti->setContextMenu(qm);
 
 	qsti->show();
@@ -69,7 +70,7 @@ void Tray::on_Tray_activated(QSystemTrayIcon::ActivationReason r) {
 }
 
 void Tray::on_Quit_triggered() {
-	if (QMessageBox::question(NULL, tr("Murmur"), tr("Are you sure you wish to quit murmur?"), QMessageBox::Yes | QMessageBox::No, QMessageBox::No) == QMessageBox::Yes) {
+	if (QMessageBox::question(NULL, tr("Murmur"), tr("Are you sure you want to quit Murmur?"), QMessageBox::Yes | QMessageBox::No, QMessageBox::No) == QMessageBox::Yes) {
 		qApp->quit();
 	}
 }
@@ -78,6 +79,7 @@ void Tray::on_ShowLog_triggered() {
 	QMainWindow *mw = new QMainWindow();
 	mw->setAttribute(Qt::WA_DeleteOnClose);
 	QTextBrowser *tb = new QTextBrowser();
+	mw->resize(675, 300);
 	mw->setCentralWidget(tb);
 	mw->setWindowTitle(QString::fromLatin1("Murmur -- %1").arg(MUMBLE_RELEASE));
 
